@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-#
 # Matplotlib documentation build configuration file, created by
 # sphinx-quickstart on Fri May  2 12:33:25 2008.
 #
@@ -11,12 +9,12 @@
 # All configuration values have a default value; values that are commented out
 # serve to show the default value.
 
-import matplotlib
 import os
+import shutil
 import sys
+
+import matplotlib
 import sphinx
-import six
-from glob import glob
 
 # If your extensions are in another directory, add it here. If the directory
 # is relative to the documentation root, use os.path.abspath to make it
@@ -40,9 +38,8 @@ extensions = [
     'IPython.sphinxext.ipython_directive',
     'numpydoc',  # Needs to be loaded *after* autodoc.
     'sphinx_gallery.gen_gallery',
-    'matplotlib.sphinxext.only_directives',
-    'matplotlib.sphinxext.plot_directive',
     'matplotlib.sphinxext.mathmpl',
+    'matplotlib.sphinxext.plot_directive',
     'sphinxext.custom_roles',
     'sphinxext.github',
     'sphinxext.math_symbol_table',
@@ -60,8 +57,6 @@ def _check_deps():
              "numpydoc": 'numpydoc',
              "PIL.Image": 'pillow',
              "sphinx_gallery": 'sphinx_gallery'}
-    if sys.version_info < (3, 3):
-        names["mock"] = 'mock'
     missing = []
     for name in names:
         try:
@@ -76,17 +71,13 @@ def _check_deps():
 _check_deps()
 
 # Import only after checking for dependencies.
-from sphinx_gallery.sorting import ExplicitOrder
-# This is only necessary to monkey patch the signature later on.
+# gallery_order.py from the sphinxext folder provides the classes that
+# allow custom ordering of sections and subsections of the gallery
+import sphinxext.gallery_order as gallery_order
+# The following import is only necessary to monkey patch the signature later on
 from sphinx_gallery import gen_rst
 
-if six.PY2:
-    from distutils.spawn import find_executable
-    has_dot = find_executable('dot') is not None
-else:
-    from shutil import which  # Python >= 3.3
-    has_dot = which('dot') is not None
-if not has_dot:
+if shutil.which('dot') is None:
     raise OSError(
         "No binary named dot - you need to install the Graph Visualization "
         "software (usually packaged as 'graphviz') to build the documentation")
@@ -107,27 +98,6 @@ intersphinx_mapping = {
     'cycler': ('https://matplotlib.org/cycler', None),
 }
 
-explicit_order_folders = [
-                          '../examples/api',
-                          '../examples/pyplots',
-                          '../examples/subplots_axes_and_figures',
-                          '../examples/color',
-                          '../examples/statistics',
-                          '../examples/lines_bars_and_markers',
-                          '../examples/images_contours_and_fields',
-                          '../examples/shapes_and_collections',
-                          '../examples/text_labels_and_annotations',
-                          '../examples/pie_and_polar_charts',
-                          '../examples/style_sheets',
-                          '../examples/axes_grid',
-                          '../examples/showcase',
-                          '../tutorials/introductory',
-                          '../tutorials/intermediate',
-                          '../tutorials/advanced']
-for folder in sorted(glob('../examples/*') + glob('../tutorials/*')):
-    if not os.path.isdir(folder) or folder in explicit_order_folders:
-        continue
-    explicit_order_folders.append(folder)
 
 # Sphinx gallery configuration
 sphinx_gallery_conf = {
@@ -141,7 +111,8 @@ sphinx_gallery_conf = {
         'scipy': 'https://docs.scipy.org/doc/scipy/reference',
     },
     'backreferences_dir': 'api/_as_gen',
-    'subsection_order': ExplicitOrder(explicit_order_folders),
+    'subsection_order': gallery_order.sectionorder,
+    'within_subsection_order': gallery_order.subsectionorder,
     'min_reported_time': 1,
 }
 
@@ -170,7 +141,7 @@ source_encoding = "utf-8"
 master_doc = 'contents'
 
 # General substitutions.
-from matplotlib.compat.subprocess import check_output
+from subprocess import check_output
 SHA = check_output(['git', 'describe', '--dirty']).decode('utf-8').strip()
 
 html_context = {'sha': SHA}
@@ -181,7 +152,6 @@ copyright = ('2002 - 2012 John Hunter, Darren Dale, Eric Firing, '
              'team; 2012 - 2018 The Matplotlib development team')
 
 
-
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
 #
@@ -263,7 +233,7 @@ html_index = 'index.html'
 
 # Custom sidebar templates, maps page names to templates.
 html_sidebars = {
-    'index': ['searchbox.html', 'sidebar_announcement.html', 'sidebar_versions.html',
+    'index': ['searchbox.html', 'sidebar_announcement.html',
               'donate_sidebar.html'],
     '**': ['searchbox.html', 'localtoc.html', 'relations.html',
            'pagesource.html']
@@ -365,3 +335,8 @@ texinfo_documents = [
 numpydoc_show_class_members = False
 
 latex_engine = 'xelatex'  # or 'lualatex'
+
+latex_elements = {
+    'babel': r'\usepackage{babel}',
+    'fontpkg': r'\setmainfont{DejaVu Serif}',
+}
