@@ -6,17 +6,15 @@ Contour Label Demo
 Illustrate some of the more advanced things that one can do with
 contour labels.
 
-See also contour_demo.py.
+See also the :doc:`contour demo example
+</gallery/images_contours_and_fields/contour_demo>`.
 """
+
 import matplotlib
 import numpy as np
-import matplotlib.cm as cm
 import matplotlib.ticker as ticker
 import matplotlib.pyplot as plt
 
-matplotlib.rcParams['xtick.direction'] = 'out'
-matplotlib.rcParams['ytick.direction'] = 'out'
-
 ###############################################################################
 # Define our surface
 
@@ -32,14 +30,10 @@ Z = (Z1 - Z2) * 2
 # Make contour labels using creative float classes
 # Follows suggestion of Manuel Metz
 
-plt.figure()
-
-# Basic contour plot
-CS = plt.contour(X, Y, Z)
-
-
 # Define a class that forces representation of float to look a certain way
 # This remove trailing zero so '1.0' becomes '1'
+
+
 class nf(float):
     def __repr__(self):
         str = '%.1f' % (self.__float__(),)
@@ -49,6 +43,10 @@ class nf(float):
             return '%.1f' % self.__float__()
 
 
+# Basic contour plot
+fig, ax = plt.subplots()
+CS = ax.contour(X, Y, Z)
+
 # Recast levels to new class
 CS.levels = [nf(val) for val in CS.levels]
 
@@ -57,33 +55,51 @@ if plt.rcParams["text.usetex"]:
     fmt = r'%r \%%'
 else:
     fmt = '%r %%'
-plt.clabel(CS, CS.levels, inline=True, fmt=fmt, fontsize=10)
+
+ax.clabel(CS, CS.levels, inline=True, fmt=fmt, fontsize=10)
 
 ###############################################################################
 # Label contours with arbitrary strings using a dictionary
 
-plt.figure()
+fig1, ax1 = plt.subplots()
 
 # Basic contour plot
-CS = plt.contour(X, Y, Z)
+CS1 = ax1.contour(X, Y, Z)
 
 fmt = {}
 strs = ['first', 'second', 'third', 'fourth', 'fifth', 'sixth', 'seventh']
-for l, s in zip(CS.levels, strs):
+for l, s in zip(CS1.levels, strs):
     fmt[l] = s
 
 # Label every other level using strings
-plt.clabel(CS, CS.levels[::2], inline=True, fmt=fmt, fontsize=10)
+ax1.clabel(CS1, CS1.levels[::2], inline=True, fmt=fmt, fontsize=10)
 
 ###############################################################################
 # Use a Formatter
 
-plt.figure()
+fig2, ax2 = plt.subplots()
 
-CS = plt.contour(X, Y, 100**Z, locator=plt.LogLocator())
+CS2 = ax2.contour(X, Y, 100**Z, locator=plt.LogLocator())
 fmt = ticker.LogFormatterMathtext()
 fmt.create_dummy_axis()
-plt.clabel(CS, CS.levels, fmt=fmt)
-plt.title("$100^Z$")
+ax2.clabel(CS2, CS2.levels, fmt=fmt)
+ax2.set_title("$100^Z$")
 
 plt.show()
+
+#############################################################################
+#
+# ------------
+#
+# References
+# """"""""""
+#
+# The use of the following functions, methods and classes is shown
+# in this example:
+
+matplotlib.axes.Axes.contour
+matplotlib.pyplot.contour
+matplotlib.axes.Axes.clabel
+matplotlib.pyplot.clabel
+matplotlib.ticker.LogFormatterMathtext
+matplotlib.ticker.TickHelper.create_dummy_axis
