@@ -3,6 +3,9 @@
 Scatter Star Poly
 =================
 
+Create multiple scatter plots with different
+star symbols.
+
 """
 import numpy as np
 import matplotlib.pyplot as plt
@@ -23,9 +26,7 @@ plt.scatter(x, y, s=80, c=z, marker=(5, 0))
 
 verts = np.array([[-1, -1], [1, -1], [1, 1], [-1, -1]])
 plt.subplot(323)
-plt.scatter(x, y, s=80, c=z, marker=(verts, 0))
-# equivalent:
-# plt.scatter(x, y, s=80, c=z, marker=None, verts=verts)
+plt.scatter(x, y, s=80, c=z, marker=verts)
 
 plt.subplot(324)
 plt.scatter(x, y, s=80, c=z, marker=(5, 1))
