@@ -4,11 +4,9 @@ Load Converter
 ==============
 
 """
-from __future__ import print_function
 import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib.cbook as cbook
-import matplotlib.dates as mdates
 from matplotlib.dates import bytespdate2num
 
 datafile = cbook.get_sample_data('msft.csv', asfileobj=False)
