@@ -7,7 +7,9 @@ Demonstrates using ax.plot's zdir keyword to plot 2D data on
 selective axes of a 3D plot.
 """
 
-from mpl_toolkits.mplot3d import Axes3D
+# This import registers the 3D projection, but is otherwise unused.
+from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
+
 import numpy as np
 import matplotlib.pyplot as plt
 
