@@ -7,7 +7,9 @@ Demonstrates making a 3D plot which has 2D bar graphs projected onto
 planes y=0, y=1, etc.
 """
 
-from mpl_toolkits.mplot3d import Axes3D
+# This import registers the 3D projection, but is otherwise unused.
+from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
+
 import matplotlib.pyplot as plt
 import numpy as np
 
