@@ -6,7 +6,9 @@ Custom hillshading in a 3D surface plot
 Demonstrates using custom hillshading in a 3D surface plot.
 """
 
-from mpl_toolkits.mplot3d import Axes3D
+# This import registers the 3D projection, but is otherwise unused.
+from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
+
 from matplotlib import cbook
 from matplotlib import cm
 from matplotlib.colors import LightSource
