@@ -5,7 +5,9 @@
 
 This example shows a how to plot a 2D and 3D plot on the same figure.
 """
-from mpl_toolkits.mplot3d import Axes3D
+# This import registers the 3D projection, but is otherwise unused.
+from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
+
 import matplotlib.pyplot as plt
 import numpy as np
 
