@@ -6,7 +6,9 @@
 Demonstration of a basic scatterplot in 3D.
 '''
 
-from mpl_toolkits.mplot3d import Axes3D
+# This import registers the 3D projection, but is otherwise unused.
+from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
+
 import matplotlib.pyplot as plt
 import numpy as np
 
