@@ -9,8 +9,10 @@ The data used is the same as in the second plot of trisurf3d_demo2.
 tricontourf3d_demo shows the filled version of this example.
 """
 
+# This import registers the 3D projection, but is otherwise unused.
+from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
+
 import matplotlib.pyplot as plt
-from mpl_toolkits.mplot3d import Axes3D
 import matplotlib.tri as tri
 import numpy as np
 
