@@ -3,15 +3,18 @@
 Embedding In GTK3
 =================
 
-demonstrate adding a FigureCanvasGTK3Agg widget to a Gtk.ScrolledWindow
-using GTK3 accessed via pygobject
+Demonstrate adding a FigureCanvasGTK3Agg widget to a Gtk.ScrolledWindow using
+GTK3 accessed via pygobject.
 """
 
+import gi
+gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk
 
+from matplotlib.backends.backend_gtk3agg import (
+    FigureCanvasGTK3Agg as FigureCanvas)
 from matplotlib.figure import Figure
-from numpy import arange, sin, pi
-from matplotlib.backends.backend_gtk3agg import FigureCanvasGTK3Agg as FigureCanvas
+import numpy as np
 
 win = Gtk.Window()
 win.connect("delete-event", Gtk.main_quit)
@@ -20,8 +23,8 @@ win.set_title("Embedding in GTK")
 
 f = Figure(figsize=(5, 4), dpi=100)
 a = f.add_subplot(111)
-t = arange(0.0, 3.0, 0.01)
-s = sin(2*pi*t)
+t = np.arange(0.0, 3.0, 0.01)
+s = np.sin(2*np.pi*t)
 a.plot(t, s)
 
 sw = Gtk.ScrolledWindow()
