@@ -27,15 +27,9 @@ class MyNavigationToolbar(NavigationToolbar):
 
         # for simplicity I'm going to reuse a bitmap from wx, you'll
         # probably want to add your own.
-        if 'phoenix' in wx.PlatformInfo:
-            self.AddTool(self.ON_CUSTOM, 'Click me',
-                         _load_bitmap('back.png'),
-                         'Activate custom contol')
-            self.Bind(wx.EVT_TOOL, self._on_custom, id=self.ON_CUSTOM)
-        else:
-            self.AddSimpleTool(self.ON_CUSTOM, _load_bitmap('back.png'),
-                               'Click me', 'Activate custom contol')
-            self.Bind(wx.EVT_TOOL, self._on_custom, id=self.ON_CUSTOM)
+        self.AddTool(self.ON_CUSTOM, 'Click me', _load_bitmap('back.png'),
+                     'Activate custom contol')
+        self.Bind(wx.EVT_TOOL, self._on_custom, id=self.ON_CUSTOM)
 
     def _on_custom(self, evt):
         # add some text to the axes in a random location in axes (0,1)
