@@ -57,9 +57,9 @@ html_content = """
     <link rel="stylesheet" href="_static/css/page.css" type="text/css">
     <link rel="stylesheet" href="_static/css/boilerplate.css" type="text/css" />
     <link rel="stylesheet" href="_static/css/fbm.css" type="text/css" />
-    <link rel="stylesheet" href="_static/jquery-ui-1.12.1/jquery-ui.min.css" >
-    <script src="_static/jquery-ui-1.12.1/external/jquery/jquery.js"></script>
-    <script src="_static/jquery-ui-1.12.1/jquery-ui.min.js"></script>
+    <link rel="stylesheet" href="_static/jquery/css/themes/base/jquery-ui.min.css" >
+    <script src="_static/jquery/js/jquery-1.11.3.min.js"></script>
+    <script src="_static/jquery/js/jquery-ui.min.js"></script>
     <script src="mpl.js"></script>
 
     <script>
@@ -214,10 +214,9 @@ class MyApplication(tornado.web.Application):
 
     def __init__(self, figure):
         self.figure = figure
-        self.manager = new_figure_manager_given_figure(
-            id(figure), figure)
+        self.manager = new_figure_manager_given_figure(id(figure), figure)
 
-        super(MyApplication, self).__init__([
+        super().__init__([
             # Static files for the CSS and JS
             (r'/_static/(.*)',
              tornado.web.StaticFileHandler,
