@@ -9,7 +9,6 @@ Demonstration of using norm to map colormaps onto data in non-linear ways.
 import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib.colors as colors
-from matplotlib.mlab import bivariate_normal
 
 """
 SymLogNorm: two humps, one negative and one positive, The positive
@@ -22,19 +21,19 @@ Note that colorbar labels do not come out looking very good.
 
 N = 100
 X, Y = np.mgrid[-3:3:complex(0, N), -2:2:complex(0, N)]
-Z1 = (bivariate_normal(X, Y, 1., 1., 1.0, 1.0)**2 -
-      0.4 * bivariate_normal(X, Y, 1.0, 1.0, -1.0, 0.0)**2)
-Z1 = Z1 / 0.03
+Z1 = np.exp(-X**2 - Y**2)
+Z2 = np.exp(-(X - 1)**2 - (Y - 1)**2)
+Z = (Z1 - Z2) * 2
 
 fig, ax = plt.subplots(2, 1)
 
-pcm = ax[0].pcolormesh(X, Y, Z1,
+pcm = ax[0].pcolormesh(X, Y, Z,
                        norm=colors.SymLogNorm(linthresh=0.03, linscale=0.03,
                                               vmin=-1.0, vmax=1.0),
                        cmap='RdBu_r')
 fig.colorbar(pcm, ax=ax[0], extend='both')
 
-pcm = ax[1].pcolormesh(X, Y, Z1, cmap='RdBu_r', vmin=-np.max(Z1))
+pcm = ax[1].pcolormesh(X, Y, Z, cmap='RdBu_r', vmin=-np.max(Z))
 fig.colorbar(pcm, ax=ax[1], extend='both')
 
 plt.show()
