@@ -11,20 +11,21 @@ These tools are used by `matplotlib.backend_managers.ToolManager`
     `matplotlib.backend_managers.ToolManager`
 """
 
+import re
+import time
+import warnings
+from weakref import WeakKeyDictionary
+
+import numpy as np
 
 from matplotlib import rcParams
 from matplotlib._pylab_helpers import Gcf
 import matplotlib.cbook as cbook
-from weakref import WeakKeyDictionary
-import six
-import time
-import warnings
-import numpy as np
 
 
 class Cursors(object):
     """Simple namespace for cursor reference"""
-    HAND, POINTER, SELECT_REGION, MOVE, WAIT = list(range(5))
+    HAND, POINTER, SELECT_REGION, MOVE, WAIT = range(5)
 cursors = Cursors()
 
 # Views positions tool
@@ -54,7 +55,7 @@ class ToolBase(object):
     Keymap to associate with this tool
 
     **String**: List of comma separated keys that will be used to call this
-    tool when the keypress event of *self.figure.canvas* is emited
+    tool when the keypress event of *self.figure.canvas* is emitted
     """
 
     description = None
@@ -154,7 +155,7 @@ class ToolToggleBase(ToolBase):
     ``*args``
         Variable length argument to be used by the Tool
     ``**kwargs``
-        `toggled` if present and True, sets the initial state ot the Tool
+        `toggled` if present and True, sets the initial state of the Tool
         Arbitrary keyword arguments to be consumed by the Tool
     """
 
@@ -228,8 +229,8 @@ class ToolToggleBase(ToolBase):
             if figure:
                 self.trigger(self, None)
             else:
-                # if there is no figure, triggen wont change the internal state
-                # we change it back
+                # if there is no figure, trigger won't change the internal
+                # state we change it back
                 self._toggled = True
 
 
@@ -336,7 +337,7 @@ class ToolCursorPosition(ToolBase):
             except (ValueError, OverflowError):
                 pass
             else:
-                artists = [a for a in event.inaxes.mouseover_set
+                artists = [a for a in event.inaxes._mouseover_set
                            if a.contains(event) and a.get_visible()]
 
                 if artists:
@@ -344,7 +345,9 @@ class ToolCursorPosition(ToolBase):
                     if a is not event.inaxes.patch:
                         data = a.get_cursor_data(event)
                         if data is not None:
-                            s += ' [%s]' % a.format_cursor_data(data)
+                            data_str = a.format_cursor_data(data)
+                            if data_str is not None:
+                                s = s + ' ' + data_str
 
                 message = s
         self.toolmanager.message_event(message, self)
@@ -401,7 +404,7 @@ class ToolQuitAll(ToolBase):
 class ToolEnableAllNavigation(ToolBase):
     """Tool to enable all axes for toolmanager interaction"""
 
-    description = 'Enables all axes toolmanager'
+    description = 'Enable all axes toolmanager'
     default_keymap = rcParams['keymap.all_axes']
 
     def trigger(self, sender, event, data=None):
@@ -417,7 +420,7 @@ class ToolEnableAllNavigation(ToolBase):
 class ToolEnableNavigation(ToolBase):
     """Tool to enable a specific axes for toolmanager interaction"""
 
-    description = 'Enables one axes toolmanager'
+    description = 'Enable one axes toolmanager'
     default_keymap = (1, 2, 3, 4, 5, 6, 7, 8, 9)
 
     def trigger(self, sender, event, data=None):
@@ -468,7 +471,7 @@ class _ToolGridBase(ToolBase):
 class ToolGrid(_ToolGridBase):
     """Tool to toggle the major grids of the figure"""
 
-    description = 'Toogle major grids'
+    description = 'Toggle major grids'
     default_keymap = rcParams['keymap.grid']
 
     def _get_next_grid_states(self, ax):
@@ -489,7 +492,7 @@ class ToolGrid(_ToolGridBase):
 class ToolMinorGrid(_ToolGridBase):
     """Tool to toggle the major and minor grids of the figure"""
 
-    description = 'Toogle major and minor grids'
+    description = 'Toggle major and minor grids'
     default_keymap = rcParams['keymap.grid_minor']
 
     def _get_next_grid_states(self, ax):
@@ -509,7 +512,7 @@ class ToolMinorGrid(_ToolGridBase):
 class ToolFullScreen(ToolToggleBase):
     """Tool to toggle full screen"""
 
-    description = 'Toogle Fullscreen mode'
+    description = 'Toggle fullscreen mode'
     default_keymap = rcParams['keymap.fullscreen']
 
     def enable(self, event):
@@ -539,7 +542,7 @@ class AxisScaleBase(ToolToggleBase):
 class ToolYScale(AxisScaleBase):
     """Tool to toggle between linear and logarithmic scales on the Y axis"""
 
-    description = 'Toogle Scale Y axis'
+    description = 'Toggle scale Y axis'
     default_keymap = rcParams['keymap.yscale']
 
     def set_scale(self, ax, scale):
@@ -549,7 +552,7 @@ class ToolYScale(AxisScaleBase):
 class ToolXScale(AxisScaleBase):
     """Tool to toggle between linear and logarithmic scales on the X axis"""
 
-    description = 'Toogle Scale X axis'
+    description = 'Toggle scale X axis'
     default_keymap = rcParams['keymap.xscale']
 
     def set_scale(self, ax, scale):
@@ -622,8 +625,8 @@ class ToolViewsPositions(ToolBase):
         if set(all_axes).issubset(pos):
             for a in all_axes:
                 # Restore both the original and modified positions
-                a.set_position(pos[a][0], 'original')
-                a.set_position(pos[a][1], 'active')
+                a._set_position(pos[a][0], 'original')
+                a._set_position(pos[a][1], 'active')
 
         self.figure.canvas.draw_idle()
 
@@ -724,7 +727,7 @@ class ToolHome(ViewsPositionsBase):
     """Restore the original view lim"""
 
     description = 'Reset original view'
-    image = 'home.png'
+    image = 'home'
     default_keymap = rcParams['keymap.home']
     _on_trigger = 'home'
 
@@ -733,7 +736,7 @@ class ToolBack(ViewsPositionsBase):
     """Move back up the view lim stack"""
 
     description = 'Back to previous view'
-    image = 'back.png'
+    image = 'back'
     default_keymap = rcParams['keymap.back']
     _on_trigger = 'back'
 
@@ -742,7 +745,7 @@ class ToolForward(ViewsPositionsBase):
     """Move forward in the view lim stack"""
 
     description = 'Forward to next view'
-    image = 'forward.png'
+    image = 'forward'
     default_keymap = rcParams['keymap.forward']
     _on_trigger = 'forward'
 
@@ -751,14 +754,14 @@ class ConfigureSubplotsBase(ToolBase):
     """Base tool for the configuration of subplots"""
 
     description = 'Configure subplots'
-    image = 'subplots.png'
+    image = 'subplots'
 
 
 class SaveFigureBase(ToolBase):
     """Base tool for figure saving"""
 
     description = 'Save the figure'
-    image = 'filesave.png'
+    image = 'filesave'
     default_keymap = rcParams['keymap.save']
 
 
@@ -830,7 +833,7 @@ class ToolZoom(ZoomPanBase):
     """Zoom to rectangle"""
 
     description = 'Zoom to rectangle'
-    image = 'zoom_to_rect.png'
+    image = 'zoom_to_rect'
     default_keymap = rcParams['keymap.zoom']
     cursor = cursors.SELECT_REGION
     radio_group = 'default'
@@ -957,7 +960,7 @@ class ToolPan(ZoomPanBase):
 
     default_keymap = rcParams['keymap.pan']
     description = 'Pan axes with left mouse, zoom with right'
-    image = 'move.png'
+    image = 'move'
     cursor = cursors.MOVE
     radio_group = 'default'
 
@@ -1018,6 +1021,59 @@ class ToolPan(ZoomPanBase):
         self.toolmanager.canvas.draw_idle()
 
 
+class ToolHelpBase(ToolBase):
+    description = 'Print tool list, shortcuts and description'
+    default_keymap = rcParams['keymap.help']
+    image = 'help.png'
+
+    @staticmethod
+    def format_shortcut(key_sequence):
+        """
+        Converts a shortcut string from the notation used in rc config to the
+        standard notation for displaying shortcuts, e.g. 'ctrl+a' -> 'Ctrl+A'.
+        """
+        return (key_sequence if len(key_sequence) == 1 else
+                re.sub(r"\+[A-Z]", r"+Shift\g<0>", key_sequence).title())
+
+    def _format_tool_keymap(self, name):
+        keymaps = self.toolmanager.get_tool_keymap(name)
+        return ", ".join(self.format_shortcut(keymap) for keymap in keymaps)
+
+    def _get_help_entries(self):
+        entries = []
+        for name, tool in sorted(self.toolmanager.tools.items()):
+            if not tool.description:
+                continue
+            entries.append((name, self._format_tool_keymap(name),
+                            tool.description))
+        return entries
+
+    def _get_help_text(self):
+        entries = self._get_help_entries()
+        entries = ["{}: {}\n\t{}".format(*entry) for entry in entries]
+        return "\n".join(entries)
+
+    def _get_help_html(self):
+        fmt = "<tr><td>{}</td><td>{}</td><td>{}</td></tr>"
+        rows = [fmt.format(
+            "<b>Action</b>", "<b>Shortcuts</b>", "<b>Description</b>")]
+        rows += [fmt.format(*row) for row in self._get_help_entries()]
+        return ("<style>td {padding: 0px 4px}</style>"
+                "<table><thead>" + rows[0] + "</thead>"
+                "<tbody>".join(rows[1:]) + "</tbody></table>")
+
+
+class ToolCopyToClipboardBase(ToolBase):
+    """Tool to copy the figure to the clipboard"""
+
+    description = 'Copy the canvas figure to clipboard'
+    default_keymap = rcParams['keymap.copy']
+
+    def trigger(self, *args, **kwargs):
+        message = "Copy tool is not available"
+        self.toolmanager.message_event(message, self)
+
+
 default_tools = {'home': ToolHome, 'back': ToolBack, 'forward': ToolForward,
                  'zoom': ToolZoom, 'pan': ToolPan,
                  'subplots': 'ToolConfigureSubplots',
@@ -1035,12 +1091,14 @@ default_tools = {'home': ToolHome, 'back': ToolBack, 'forward': ToolForward,
                  _views_positions: ToolViewsPositions,
                  'cursor': 'ToolSetCursor',
                  'rubberband': 'ToolRubberband',
+                 'help': 'ToolHelp',
+                 'copy': 'ToolCopyToClipboard',
                  }
 """Default tools"""
 
 default_toolbar_tools = [['navigation', ['home', 'back', 'forward']],
                          ['zoompan', ['pan', 'zoom', 'subplots']],
-                         ['io', ['save']]]
+                         ['io', ['save', 'help']]]
 """Default tools in the toolbar"""
 
 
@@ -1057,7 +1115,7 @@ def add_tools_to_manager(toolmanager, tools=default_tools):
         info.
     """
 
-    for name, tool in six.iteritems(tools):
+    for name, tool in tools.items():
         toolmanager.add_tool(name, tool)
 
 
