@@ -1,8 +1,9 @@
-from __future__ import print_function
-from matplotlib._image import frombuffer
+import numpy as np
+
 from matplotlib.backends.backend_agg import RendererAgg
 from matplotlib.tight_bbox import process_figure_for_rasterizing
 
+
 class MixedModeRenderer(object):
     """
     A helper class to implement a renderer that switches between
@@ -15,20 +16,30 @@ class MixedModeRenderer(object):
                  raster_renderer_class=None,
                  bbox_inches_restore=None):
         """
-        figure: The figure instance.
+        Parameters
+        ----------
+        figure : `matplotlib.figure.Figure`
+            The figure instance.
 
-        width: The width of the canvas in logical units
+        width : scalar
+            The width of the canvas in logical units
 
-        height: The height of the canvas in logical units
+        height : scalar
+            The height of the canvas in logical units
 
-        dpi: The dpi of the canvas
+        dpi : scalar
+            The dpi of the canvas
 
-        vector_renderer: An instance of a subclass of RendererBase
-        that will be used for the vector drawing.
+        vector_renderer : `matplotlib.backend_bases.RendererBase`
+            An instance of a subclass of
+            `~matplotlib.backend_bases.RendererBase` that will be used for the
+            vector drawing.
+
+        raster_renderer_class : `matplotlib.backend_bases.RendererBase`
+            The renderer class to use for the raster drawing.  If not provided,
+            this will use the Agg backend (which is currently the only viable
+            option anyway.)
 
-        raster_renderer_class: The renderer class to use for the
-        raster drawing.  If not provided, this will use the Agg
-        backend (which is currently the only viable option anyway.)
         """
         if raster_renderer_class is None:
             raster_renderer_class = RendererAgg
@@ -38,16 +49,15 @@ class MixedModeRenderer(object):
         self._height = height
         self.dpi = dpi
 
-        assert not vector_renderer.option_image_nocomposite()
         self._vector_renderer = vector_renderer
 
         self._raster_renderer = None
         self._rasterizing = 0
 
-        # A renference to the figure is needed as we need to change
+        # A reference to the figure is needed as we need to change
         # the figure dpi before and after the rasterization. Although
         # this looks ugly, I couldn't find a better solution. -JJL
-        self.figure=figure
+        self.figure = figure
         self._figdpi = figure.get_dpi()
 
         self._bbox_inches_restore = bbox_inches_restore
@@ -62,7 +72,9 @@ class MixedModeRenderer(object):
         option_image_nocomposite points_to_pixels strip_math
         start_filter stop_filter draw_gouraud_triangle
         draw_gouraud_triangles option_scale_image
+        _text2path _get_text_path_transform height width
         """.split()
+
     def _set_current_renderer(self, renderer):
         self._renderer = renderer
 
@@ -72,7 +84,6 @@ class MixedModeRenderer(object):
         renderer.start_rasterizing = self.start_rasterizing
         renderer.stop_rasterizing = self.stop_rasterizing
 
-
     def start_rasterizing(self):
         """
         Enter "raster" mode.  All subsequent drawing commands (until
@@ -86,21 +97,17 @@ class MixedModeRenderer(object):
         # change the dpi of the figure temporarily.
         self.figure.set_dpi(self.dpi)
 
-        if self._bbox_inches_restore: # when tight bbox is used
+        if self._bbox_inches_restore:  # when tight bbox is used
             r = process_figure_for_rasterizing(self.figure,
-                                               self._bbox_inches_restore,
-                                               mode="png")
-
+                                               self._bbox_inches_restore)
             self._bbox_inches_restore = r
 
-
         if self._rasterizing == 0:
             self._raster_renderer = self._raster_renderer_class(
                 self._width*self.dpi, self._height*self.dpi, self.dpi)
             self._set_current_renderer(self._raster_renderer)
         self._rasterizing += 1
 
-
     def stop_rasterizing(self):
         """
         Exit "raster" mode.  All of the drawing that was done since
@@ -114,21 +121,21 @@ class MixedModeRenderer(object):
         if self._rasterizing == 0:
             self._set_current_renderer(self._vector_renderer)
 
-            width, height = self._width * self.dpi, self._height * self.dpi
+            height = self._height * self.dpi
             buffer, bounds = self._raster_renderer.tostring_rgba_minimized()
             l, b, w, h = bounds
             if w > 0 and h > 0:
-                image = frombuffer(buffer, w, h, True)
-                image.is_grayscale = False
-                image.flipud_out()
+                image = np.frombuffer(buffer, dtype=np.uint8)
+                image = image.reshape((h, w, 4))
+                image = image[::-1]
                 gc = self._renderer.new_gc()
                 # TODO: If the mixedmode resolution differs from the figure's
                 #       dpi, the image must be scaled (dpi->_figdpi). Not all
                 #       backends support this.
                 self._renderer.draw_image(
                     gc,
-                    float(l) / self.dpi * self._figdpi,
-                    (float(height)-b-h) / self.dpi * self._figdpi,
+                    l * self._figdpi / self.dpi,
+                    (height-b-h) * self._figdpi / self.dpi,
                     image)
             self._raster_renderer = None
             self._rasterizing = False
@@ -139,5 +146,5 @@ class MixedModeRenderer(object):
         if self._bbox_inches_restore:  # when tight bbox is used
             r = process_figure_for_rasterizing(self.figure,
                                                self._bbox_inches_restore,
-                                               mode="pdf")
+                                               self._figdpi)
             self._bbox_inches_restore = r
