@@ -1,28 +1,30 @@
 """
-A PostScript backend, which can produce both PostScript .ps and .eps
+A PostScript backend, which can produce both PostScript .ps and .eps.
 """
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-from six.moves import StringIO
-
-import glob, os, shutil, sys, time, datetime
-import io
+import binascii
+import datetime
+import glob
+from io import StringIO, TextIOWrapper
 import logging
+import os
+import pathlib
+import re
+import shutil
+import subprocess
+import sys
+from tempfile import TemporaryDirectory
+import time
+
+import numpy as np
 
-from tempfile import mkstemp
 from matplotlib import cbook, __version__, rcParams, checkdep_ghostscript
 from matplotlib.afm import AFM
 from matplotlib.backend_bases import (
     _Backend, FigureCanvasBase, FigureManagerBase, GraphicsContextBase,
     RendererBase)
-
 from matplotlib.cbook import (get_realpath_and_stat, is_writable_file_like,
                               maxdict, file_requires_unicode)
-from matplotlib.compat.subprocess import subprocess
-
 from matplotlib.font_manager import findfont, is_opentype_cff_font, get_font
 from matplotlib.ft2font import KERNING_DEFAULT, LOAD_NO_HINTING
 from matplotlib.ttconv import convert_ttf_to_ps
@@ -31,14 +33,8 @@ from matplotlib._mathtext_data import uni2type1
 from matplotlib.path import Path
 from matplotlib import _path
 from matplotlib.transforms import Affine2D
-
 from matplotlib.backends.backend_mixed import MixedModeRenderer
 
-
-import numpy as np
-import binascii
-import re
-
 _log = logging.getLogger(__name__)
 
 backend_version = 'Level II'
@@ -78,13 +74,10 @@ class PsBackendHelper(object):
         except KeyError:
             pass
 
-        from matplotlib.compat.subprocess import Popen, PIPE
-        s = Popen([self.gs_exe, "--version"], stdout=PIPE)
+        s = subprocess.Popen(
+            [self.gs_exe, "--version"], stdout=subprocess.PIPE)
         pipe, stderr = s.communicate()
-        if six.PY3:
-            ver = pipe.decode('ascii')
-        else:
-            ver = pipe
+        ver = pipe.decode('ascii')
         try:
             gs_version = tuple(map(int, ver.strip().split(".")))
         except ValueError:
@@ -129,17 +122,16 @@ papersize = {'letter': (8.5,11),
              'b10': (1.26,1.76)}
 
 def _get_papertype(w, h):
-    keys = list(six.iterkeys(papersize))
-    keys.sort()
-    keys.reverse()
-    for key in keys:
-        if key.startswith('l'): continue
-        pw, ph = papersize[key]
-        if (w < pw) and (h < ph): return key
+    for key, (pw, ph) in sorted(papersize.items(), reverse=True):
+        if key.startswith('l'):
+            continue
+        if w < pw and h < ph:
+            return key
     return 'a0'
 
 def _num_to_str(val):
-    if isinstance(val, six.string_types): return val
+    if isinstance(val, str):
+        return val
 
     ival = int(val)
     if val == ival: return str(ival)
@@ -170,17 +162,13 @@ def _move_path_to_path_or_stream(src, dst):
     If *dst* is a path, the metadata of *src* are *not* copied.
     """
     if is_writable_file_like(dst):
-        fh = (io.open(src, 'r', encoding='latin-1')
+        fh = (open(src, 'r', encoding='latin-1')
               if file_requires_unicode(dst)
-              else io.open(src, 'rb'))
+              else open(src, 'rb'))
         with fh:
             shutil.copyfileobj(fh, dst)
     else:
-        # Py3: shutil.move(src, dst, copy_function=shutil.copyfile)
-        open(dst, 'w').close()
-        mode = os.stat(dst).st_mode
-        shutil.move(src, dst)
-        os.chmod(dst, mode)
+        shutil.move(src, dst, copy_function=shutil.copyfile)
 
 
 class RendererPS(RendererBase):
@@ -231,10 +219,10 @@ class RendererPS(RendererBase):
         realpath, stat_key = get_realpath_and_stat(font.fname)
         used_characters = self.used_characters.setdefault(
             stat_key, (realpath, set()))
-        used_characters[1].update([ord(x) for x in s])
+        used_characters[1].update(map(ord, s))
 
     def merge_used_characters(self, other):
-        for stat_key, (realpath, charset) in six.iteritems(other):
+        for stat_key, (realpath, charset) in other.items():
             used_characters = self.used_characters.setdefault(
                 stat_key, (realpath, set()))
             used_characters[1].update(charset)
@@ -278,15 +266,16 @@ class RendererPS(RendererBase):
             self.linedash = (offset, seq)
 
     def set_font(self, fontname, fontsize, store=1):
-        if rcParams['ps.useafm']: return
-        if (fontname,fontsize) != (self.fontname,self.fontsize):
+        if rcParams['ps.useafm']:
+            return
+        if (fontname, fontsize) != (self.fontname,self.fontsize):
             out = ("/%s findfont\n"
                    "%1.3f scalefont\n"
                    "setfont\n" % (fontname, fontsize))
-
             self._pswriter.write(out)
-            if store: self.fontname = fontname
-            if store: self.fontsize = fontsize
+            if store:
+                self.fontname = fontname
+                self.fontsize = fontsize
 
     def create_hatch(self, hatch):
         sidelen = 72
@@ -380,7 +369,7 @@ class RendererPS(RendererBase):
                     "Helvetica", fontext='afm', directory=self._afm_font_dir)
             font = self.afmfontd.get(fname)
             if font is None:
-                with io.open(fname, 'rb') as fh:
+                with open(fname, 'rb') as fh:
                     font = AFM(fh)
                 self.afmfontd[fname] = font
             self.afmfontd[key] = font
@@ -526,17 +515,21 @@ grestore
 
         if rgbFace:
             if len(rgbFace) == 4 and rgbFace[3] == 0:
-                return
-            if rgbFace[0] == rgbFace[1] == rgbFace[2]:
-                ps_color = '%1.3f setgray' % rgbFace[0]
+                ps_color = None
             else:
-                ps_color = '%1.3f %1.3f %1.3f setrgbcolor' % rgbFace[:3]
+                if rgbFace[0] == rgbFace[1] == rgbFace[2]:
+                    ps_color = '%1.3f setgray' % rgbFace[0]
+                else:
+                    ps_color = '%1.3f %1.3f %1.3f setrgbcolor' % rgbFace[:3]
 
         # construct the generic marker command:
         ps_cmd = ['/o {', 'gsave', 'newpath', 'translate'] # don't want the translate to be global
 
         lw = gc.get_linewidth()
-        stroke = lw != 0.0
+        alpha = (gc.get_alpha()
+                 if gc.get_forced_alpha() or len(gc.get_rgb()) == 3
+                 else gc.get_rgb()[3])
+        stroke = lw > 0 and alpha > 0
         if stroke:
             ps_cmd.append('%.1f setlinewidth' % lw)
             jint = gc.get_joinstyle()
@@ -550,7 +543,8 @@ grestore
         if rgbFace:
             if stroke:
                 ps_cmd.append('gsave')
-            ps_cmd.extend([ps_color, 'fill'])
+            if ps_color:
+                ps_cmd.extend([ps_color, 'fill'])
             if stroke:
                 ps_cmd.append('grestore')
 
@@ -714,12 +708,8 @@ grestore
             self.track_characters(font, s)
 
             self.set_color(*gc.get_rgb())
-            sfnt = font.get_sfnt()
-            try:
-                ps_name = sfnt[1, 0, 0, 6].decode('mac_roman')
-            except KeyError:
-                ps_name = sfnt[3, 1, 0x0409, 6].decode('utf-16be')
-            ps_name = ps_name.encode('ascii', 'replace').decode('ascii')
+            ps_name = (font.postscript_name
+                       .encode('ascii', 'replace').decode('ascii'))
             self.set_font(ps_name, prop.get_size_in_points())
 
             lastgind = None
@@ -930,8 +920,12 @@ class FigureCanvasPS(FigureCanvasBase):
     def print_eps(self, outfile, *args, **kwargs):
         return self._print_ps(outfile, 'eps', *args, **kwargs)
 
-    def _print_ps(self, outfile, format, *args, **kwargs):
-        papertype = kwargs.pop("papertype", rcParams['ps.papersize'])
+    def _print_ps(self, outfile, format, *args,
+                  papertype=None, dpi=72, facecolor='w', edgecolor='w',
+                  orientation='portrait',
+                  **kwargs):
+        if papertype is None:
+            papertype = rcParams['ps.papersize']
         papertype = papertype.lower()
         if papertype == 'auto':
             pass
@@ -939,28 +933,27 @@ class FigureCanvasPS(FigureCanvasBase):
             raise RuntimeError('%s is not a valid papertype. Use one of %s' %
                                (papertype, ', '.join(papersize)))
 
-        orientation = kwargs.pop("orientation", "portrait").lower()
+        orientation = orientation.lower()
         if orientation == 'landscape': isLandscape = True
         elif orientation == 'portrait': isLandscape = False
         else: raise RuntimeError('Orientation must be "portrait" or "landscape"')
 
         self.figure.set_dpi(72) # Override the dpi kwarg
-        imagedpi = kwargs.pop("dpi", 72)
-        facecolor = kwargs.pop("facecolor", "w")
-        edgecolor = kwargs.pop("edgecolor", "w")
 
         if rcParams['text.usetex']:
-            self._print_figure_tex(outfile, format, imagedpi, facecolor, edgecolor,
+            self._print_figure_tex(outfile, format, dpi, facecolor, edgecolor,
                                    orientation, isLandscape, papertype,
                                    **kwargs)
         else:
-            self._print_figure(outfile, format, imagedpi, facecolor, edgecolor,
+            self._print_figure(outfile, format, dpi, facecolor, edgecolor,
                                orientation, isLandscape, papertype,
                                **kwargs)
 
-    def _print_figure(self, outfile, format, dpi=72, facecolor='w', edgecolor='w',
-                      orientation='portrait', isLandscape=False, papertype=None,
-                      metadata=None, **kwargs):
+    def _print_figure(
+            self, outfile, format, dpi=72, facecolor='w', edgecolor='w',
+            orientation='portrait', isLandscape=False, papertype=None,
+            metadata=None, *,
+            dryrun=False, bbox_inches_restore=None, **kwargs):
         """
         Render the figure to hardcopy.  Set the figure patch face and
         edge colors.  This is useful because some of the GUIs have a
@@ -978,9 +971,9 @@ class FigureCanvasPS(FigureCanvasBase):
         the key 'Creator' is used.
         """
         isEPSF = format == 'eps'
-        if isinstance(outfile,
-                      (six.string_types, getattr(os, "PathLike", ()),)):
+        if isinstance(outfile, (str, getattr(os, "PathLike", ()),)):
             outfile = title = getattr(os, "fspath", lambda obj: obj)(outfile)
+            title = title.encode("latin-1", "replace").decode()
             passed_in_file_object = False
         elif is_writable_file_like(outfile):
             title = None
@@ -1030,8 +1023,6 @@ class FigureCanvasPS(FigureCanvasBase):
         self.figure.set_facecolor(facecolor)
         self.figure.set_edgecolor(edgecolor)
 
-
-        dryrun = kwargs.get("dryrun", False)
         if dryrun:
             class NullWriter(object):
                 def write(self, *kl, **kwargs):
@@ -1039,16 +1030,14 @@ class FigureCanvasPS(FigureCanvasBase):
 
             self._pswriter = NullWriter()
         else:
-            self._pswriter = io.StringIO()
-
+            self._pswriter = StringIO()
 
         # mixed mode rendering
-        _bbox_inches_restore = kwargs.pop("bbox_inches_restore", None)
         ps_renderer = self._renderer_class(width, height, self._pswriter,
                                            imagedpi=dpi)
         renderer = MixedModeRenderer(self.figure,
             width, height, dpi, ps_renderer,
-            bbox_inches_restore=_bbox_inches_restore)
+            bbox_inches_restore=bbox_inches_restore)
 
         self.figure.draw(renderer)
 
@@ -1072,7 +1061,7 @@ class FigureCanvasPS(FigureCanvasBase):
             else:
                 print("%!PS-Adobe-3.0", file=fh)
             if title:
-                print("%%Title: "+title, file=fh)
+                print("%%Title: " + title, file=fh)
             print("%%Creator: " + creator_str, file=fh)
             # get source date from SOURCE_DATE_EPOCH, if set
             # See https://reproducible-builds.org/specs/source-date-epoch/
@@ -1082,7 +1071,7 @@ class FigureCanvasPS(FigureCanvasBase):
                     int(source_date_epoch)).strftime("%a %b %d %H:%M:%S %Y")
             else:
                 source_date = time.ctime()
-            print("%%CreationDate: "+source_date, file=fh)
+            print("%%CreationDate: " + source_date, file=fh)
             print("%%Orientation: " + orientation, file=fh)
             if not isEPSF:
                 print("%%DocumentPaperSizes: "+papertype, file=fh)
@@ -1102,8 +1091,8 @@ class FigureCanvasPS(FigureCanvasBase):
                 for l in d.split('\n'):
                     print(l.strip(), file=fh)
             if not rcParams['ps.useafm']:
-                for font_filename, chars in six.itervalues(
-                        ps_renderer.used_characters):
+                for font_filename, chars in \
+                        ps_renderer.used_characters.values():
                     if len(chars):
                         font = get_font(font_filename)
                         glyph_ids = []
@@ -1129,10 +1118,8 @@ class FigureCanvasPS(FigureCanvasBase):
                                 "time; consider using the Cairo backend")
                         else:
                             fh.flush()
-                            convert_ttf_to_ps(
-                                font_filename.encode(
-                                    sys.getfilesystemencoding()),
-                                fh, fonttype, glyph_ids)
+                            convert_ttf_to_ps(os.fsencode(font_filename),
+                                              fh, fonttype, glyph_ids)
             print("end", file=fh)
             print("%%EndProlog", file=fh)
 
@@ -1148,7 +1135,7 @@ class FigureCanvasPS(FigureCanvasBase):
 
             # write the figure
             content = self._pswriter.getvalue()
-            if not isinstance(content, six.text_type):
+            if not isinstance(content, str):
                 content = content.decode('ascii')
             print(content, file=fh)
 
@@ -1162,31 +1149,25 @@ class FigureCanvasPS(FigureCanvasBase):
         if rcParams['ps.usedistiller']:
             # We are going to use an external program to process the output.
             # Write to a temporary file.
-            fd, tmpfile = mkstemp()
-            try:
-                with io.open(fd, 'w', encoding='latin-1') as fh:
+            with TemporaryDirectory() as tmpdir:
+                tmpfile = os.path.join(tmpdir, "tmp.ps")
+                with open(tmpfile, 'w', encoding='latin-1') as fh:
                     print_figure_impl(fh)
                 if rcParams['ps.usedistiller'] == 'ghostscript':
                     gs_distill(tmpfile, isEPSF, ptype=papertype, bbox=bbox)
                 elif rcParams['ps.usedistiller'] == 'xpdf':
                     xpdf_distill(tmpfile, isEPSF, ptype=papertype, bbox=bbox)
-
                 _move_path_to_path_or_stream(tmpfile, outfile)
-            finally:
-                if os.path.isfile(tmpfile):
-                    os.unlink(tmpfile)
 
         else:
             # Write directly to outfile.
             if passed_in_file_object:
                 requires_unicode = file_requires_unicode(outfile)
 
-                if (not requires_unicode and
-                        (six.PY3 or not isinstance(outfile, StringIO))):
-                    fh = io.TextIOWrapper(outfile, encoding="latin-1")
-
-                    # Prevent the io.TextIOWrapper from closing the
-                    # underlying file
+                if not requires_unicode:
+                    fh = TextIOWrapper(outfile, encoding="latin-1")
+                    # Prevent the TextIOWrapper from closing the underlying
+                    # file.
                     def do_nothing():
                         pass
                     fh.close = do_nothing
@@ -1195,12 +1176,13 @@ class FigureCanvasPS(FigureCanvasBase):
 
                 print_figure_impl(fh)
             else:
-                with io.open(outfile, 'w', encoding='latin-1') as fh:
+                with open(outfile, 'w', encoding='latin-1') as fh:
                     print_figure_impl(fh)
 
-    def _print_figure_tex(self, outfile, format, dpi, facecolor, edgecolor,
-                          orientation, isLandscape, papertype, metadata=None,
-                          **kwargs):
+    def _print_figure_tex(
+            self, outfile, format, dpi, facecolor, edgecolor,
+            orientation, isLandscape, papertype, metadata=None, *,
+            dryrun=False, bbox_inches_restore=None, **kwargs):
         """
         If text.usetex is True in rc, a temporary pair of tex/eps files
         are created to allow tex to manage the text layout via the PSFrags
@@ -1210,7 +1192,7 @@ class FigureCanvasPS(FigureCanvasBase):
         the key 'Creator' is used.
         """
         isEPSF = format == 'eps'
-        if isinstance(outfile, six.string_types):
+        if isinstance(outfile, str):
             title = outfile
         elif is_writable_file_like(outfile):
             title = None
@@ -1235,7 +1217,6 @@ class FigureCanvasPS(FigureCanvasBase):
         self.figure.set_facecolor(facecolor)
         self.figure.set_edgecolor(edgecolor)
 
-        dryrun = kwargs.get("dryrun", False)
         if dryrun:
             class NullWriter(object):
                 def write(self, *kl, **kwargs):
@@ -1243,15 +1224,14 @@ class FigureCanvasPS(FigureCanvasBase):
 
             self._pswriter = NullWriter()
         else:
-            self._pswriter = io.StringIO()
+            self._pswriter = StringIO()
 
         # mixed mode rendering
-        _bbox_inches_restore = kwargs.pop("bbox_inches_restore", None)
         ps_renderer = self._renderer_class(width, height,
                                            self._pswriter, imagedpi=dpi)
         renderer = MixedModeRenderer(self.figure,
                                      width, height, dpi, ps_renderer,
-                                     bbox_inches_restore=_bbox_inches_restore)
+                                     bbox_inches_restore=bbox_inches_restore)
 
         self.figure.draw(renderer)
 
@@ -1270,9 +1250,9 @@ class FigureCanvasPS(FigureCanvasBase):
 
         # write to a temp file, we'll move it to outfile when done
 
-        fd, tmpfile = mkstemp()
-        try:
-            with io.open(fd, 'w', encoding='latin-1') as fh:
+        with TemporaryDirectory() as tmpdir:
+            tmpfile = os.path.join(tmpdir, "tmp.ps")
+            with open(tmpfile, 'w', encoding='latin-1') as fh:
                 # write the Encapsulated PostScript headers
                 print("%!PS-Adobe-3.0 EPSF-3.0", file=fh)
                 if title:
@@ -1359,9 +1339,6 @@ class FigureCanvasPS(FigureCanvasBase):
                              rotated=psfrag_rotated)
 
             _move_path_to_path_or_stream(tmpfile, outfile)
-        finally:
-            if os.path.isfile(tmpfile):
-                os.unlink(tmpfile)
 
 
 def convert_psfrags(tmpfile, psfrags, font_preamble, custom_preamble,
@@ -1412,24 +1389,19 @@ def convert_psfrags(tmpfile, psfrags, font_preamble, custom_preamble,
       paperWidth, paperHeight,
       '\n'.join(psfrags), angle, os.path.split(epsfile)[-1])
 
-    with io.open(latexfile, 'wb') as latexh:
-        if rcParams['text.latex.unicode']:
-            latexh.write(s.encode('utf8'))
-        else:
-            try:
-                latexh.write(s.encode('ascii'))
-            except UnicodeEncodeError:
-                _log.info("You are using unicode and latex, but have "
-                          "not enabled the matplotlib 'text.latex.unicode' "
-                          "rcParam.")
-                raise
+    try:
+        pathlib.Path(latexfile).write_text(
+            s, encoding='utf-8' if rcParams['text.latex.unicode'] else 'ascii')
+    except UnicodeEncodeError:
+        _log.info("You are using unicode and latex, but have not enabled the "
+                  "Matplotlib 'text.latex.unicode' rcParam.")
+        raise
 
     # Replace \\ for / so latex does not think there is a function call
     latexfile = latexfile.replace("\\", "/")
     # Replace ~ so Latex does not think it is line break
     latexfile = latexfile.replace("~", "\\string~")
-    command = [str("latex"), "-interaction=nonstopmode",
-               '"%s"' % latexfile]
+    command = ["latex", "-interaction=nonstopmode", '"%s"' % latexfile]
     _log.debug('%s', command)
     try:
         report = subprocess.check_output(command, cwd=tmpdir,
@@ -1443,7 +1415,7 @@ def convert_psfrags(tmpfile, psfrags, font_preamble, custom_preamble,
                        exc.output.decode("utf-8"))))
     _log.debug(report)
 
-    command = [str('dvips'), '-q', '-R0', '-o', os.path.basename(psfile),
+    command = ['dvips', '-q', '-R0', '-o', os.path.basename(psfile),
                os.path.basename(dvifile)]
     _log.debug(command)
     try:
@@ -1467,7 +1439,7 @@ def convert_psfrags(tmpfile, psfrags, font_preamble, custom_preamble,
     # the generated ps file is in landscape and return this
     # information. The return value is used in pstoeps step to recover
     # the correct bounding box. 2010-06-05 JJL
-    with io.open(tmpfile) as fh:
+    with open(tmpfile) as fh:
         if "Landscape" in fh.read(1000):
             psfrag_rotated = True
         else:
@@ -1544,7 +1516,7 @@ def xpdf_distill(tmpfile, eps=False, ptype='letter', bbox=None, rotated=False):
 
     # Pass options as `-foo#bar` instead of `-foo=bar` to keep Windows happy
     # (https://www.ghostscript.com/doc/9.22/Use.htm#MS_Windows).
-    command = [str("ps2pdf"),
+    command = ["ps2pdf",
                "-dAutoFilterColorImages#false",
                "-dAutoFilterGrayImages#false",
                "-dAutoRotatePages#false",
@@ -1563,7 +1535,7 @@ def xpdf_distill(tmpfile, eps=False, ptype='letter', bbox=None, rotated=False):
              '\n\n' % exc.output.decode("utf-8")))
     _log.debug(report)
 
-    command = [str("pdftops"), "-paper", "match", "-level2", pdffile, psfile]
+    command = ["pdftops", "-paper", "match", "-level2", pdffile, psfile]
     _log.debug(command)
     try:
         report = subprocess.check_output(command, stderr=subprocess.STDOUT)
@@ -1591,7 +1563,7 @@ def get_bbox_header(lbrt, rotated=False):
 
     l, b, r, t = lbrt
     if rotated:
-        rotate = "%.2f %.2f  translate\n90 rotate" % (l+r, 0)
+        rotate = "%.2f %.2f translate\n90 rotate" % (l+r, 0)
     else:
         rotate = ""
     bbox_info = '%%%%BoundingBox: %d %d %d %d' % (l, b, np.ceil(r), np.ceil(t))
@@ -1602,6 +1574,7 @@ def get_bbox_header(lbrt, rotated=False):
 
 # get_bbox is deprecated. I don't see any reason to use ghostscript to
 # find the bounding box, as the required bounding box is alread known.
+@cbook.deprecated("3.0")
 def get_bbox(tmpfile, bbox):
     """
     Use ghostscript's bbox device to find the center of the bounding box.
@@ -1660,7 +1633,7 @@ def pstoeps(tmpfile, bbox=None, rotated=False):
         bbox_info, rotate = None, None
 
     epsfile = tmpfile + '.eps'
-    with io.open(epsfile, 'wb') as epsh, io.open(tmpfile, 'rb') as tmph:
+    with open(epsfile, 'wb') as epsh, open(tmpfile, 'rb') as tmph:
         write = epsh.write
         # Modify the header:
         for line in tmph:
@@ -1707,8 +1680,7 @@ def pstoeps(tmpfile, bbox=None, rotated=False):
     shutil.move(epsfile, tmpfile)
 
 
-class FigureManagerPS(FigureManagerBase):
-    pass
+FigureManagerPS = FigureManagerBase
 
 
 # The following Python dictionary psDefs contains the entries for the
@@ -1754,4 +1726,3 @@ psDefs = [
 @_Backend.export
 class _BackendPS(_Backend):
     FigureCanvas = FigureCanvasPS
-    FigureManager = FigureManagerPS
