@@ -1,35 +1,17 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-
-import matplotlib
-from matplotlib.figure import Figure
+import wx
 
 from .backend_agg import FigureCanvasAgg
-
-from . import wx_compat as wxc
-from . import backend_wx
-from .backend_wx import (_BackendWx, FigureManagerWx, FigureCanvasWx,
-    FigureFrameWx, DEBUG_MSG, NavigationToolbar2Wx, Toolbar)
-
-import wx
+from .backend_wx import (
+    _BackendWx, _FigureCanvasWxBase, FigureFrameWx,
+    NavigationToolbar2Wx as NavigationToolbar2WxAgg)
 
 
 class FigureFrameWxAgg(FigureFrameWx):
     def get_canvas(self, fig):
         return FigureCanvasWxAgg(self, -1, fig)
 
-    def _get_toolbar(self, statbar):
-        if matplotlib.rcParams['toolbar'] == 'toolbar2':
-            toolbar = NavigationToolbar2WxAgg(self.canvas)
-            toolbar.set_status_bar(statbar)
-        else:
-            toolbar = None
-        return toolbar
 
-
-class FigureCanvasWxAgg(FigureCanvasAgg, FigureCanvasWx):
+class FigureCanvasWxAgg(FigureCanvasAgg, _FigureCanvasWxBase):
     """
     The FigureCanvas contains the figure and does event handling.
 
@@ -44,7 +26,6 @@ class FigureCanvasWxAgg(FigureCanvasAgg, FigureCanvasWx):
         """
         Render the figure using agg.
         """
-        DEBUG_MSG("draw()", 1, self)
         FigureCanvasAgg.draw(self)
 
         self.bitmap = _convert_agg_to_wx_bitmap(self.get_renderer(), None)
@@ -82,25 +63,9 @@ class FigureCanvasWxAgg(FigureCanvasAgg, FigureCanvasWx):
 
     filetypes = FigureCanvasAgg.filetypes
 
-    def print_figure(self, filename, *args, **kwargs):
-        # Use pure Agg renderer to draw
-        FigureCanvasAgg.print_figure(self, filename, *args, **kwargs)
-        # Restore the current view; this is needed because the
-        # artist contains methods rely on particular attributes
-        # of the rendered figure for determining things like
-        # bounding boxes.
-        if self._isDrawn:
-            self.draw()
-
-
-class NavigationToolbar2WxAgg(NavigationToolbar2Wx):
-    def get_canvas(self, frame, fig):
-        return FigureCanvasWxAgg(frame, -1, fig)
-
 
 # agg/wxPython image conversion functions (wxPython >= 2.8)
 
-
 def _convert_agg_to_wx_image(agg, bbox):
     """
     Convert the region of the agg buffer bounded by bbox to a wx.Image.  If
@@ -110,7 +75,7 @@ def _convert_agg_to_wx_image(agg, bbox):
     """
     if bbox is None:
         # agg => rgb -> image
-        image = wxc.EmptyImage(int(agg.width), int(agg.height))
+        image = wx.Image(int(agg.width), int(agg.height))
         image.SetData(agg.tostring_rgb())
         return image
     else:
@@ -127,8 +92,8 @@ def _convert_agg_to_wx_bitmap(agg, bbox):
     """
     if bbox is None:
         # agg => rgba buffer -> bitmap
-        return wxc.BitmapFromBuffer(int(agg.width), int(agg.height),
-                                    agg.buffer_rgba())
+        return wx.Bitmap.FromBufferRGBA(int(agg.width), int(agg.height),
+                                        agg.buffer_rgba())
     else:
         # agg => rgba buffer -> bitmap => clipped bitmap
         return _WX28_clipped_agg_as_bitmap(agg, bbox)
@@ -144,12 +109,12 @@ def _WX28_clipped_agg_as_bitmap(agg, bbox):
     r = l + width
     t = b + height
 
-    srcBmp = wxc.BitmapFromBuffer(int(agg.width), int(agg.height),
-                                  agg.buffer_rgba())
+    srcBmp = wx.Bitmap.FromBufferRGBA(int(agg.width), int(agg.height),
+                                      agg.buffer_rgba())
     srcDC = wx.MemoryDC()
     srcDC.SelectObject(srcBmp)
 
-    destBmp = wxc.EmptyBitmap(int(width), int(height))
+    destBmp = wx.Bitmap(int(width), int(height))
     destDC = wx.MemoryDC()
     destDC.SelectObject(destBmp)
 
