@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 formlayout
 ==========
@@ -38,18 +37,14 @@ OTHER DEALINGS IN THE SOFTWARE.
 # 1.0.7: added support for "Apply" button
 # 1.0.6: code cleaning
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 __version__ = '1.0.10'
 __license__ = __doc__
 
 import copy
 import datetime
+from numbers import Integral, Real
 import warnings
 
-import six
-
 from matplotlib import colors as mcolors
 from matplotlib.backends.qt_compat import QtGui, QtWidgets, QtCore
 
@@ -98,7 +93,7 @@ def to_qcolor(color):
     try:
         rgba = mcolors.to_rgba(color)
     except ValueError:
-        warnings.warn('Ignoring invalid color %r' % color)
+        warnings.warn('Ignoring invalid color %r' % color, stacklevel=2)
         return qcolor  # return invalid QColor
     qcolor.setRgbF(*rgba)
     return qcolor
@@ -133,7 +128,7 @@ class ColorLayout(QtWidgets.QHBoxLayout):
 def font_is_installed(font):
     """Check if font is installed"""
     return [fam for fam in QtGui.QFontDatabase().families()
-            if six.text_type(fam) == font]
+            if str(fam) == font]
 
 
 def tuple_to_qfont(tup):
@@ -143,7 +138,7 @@ def tuple_to_qfont(tup):
     """
     if not (isinstance(tup, tuple) and len(tup) == 4
             and font_is_installed(tup[0])
-            and isinstance(tup[1], int)
+            and isinstance(tup[1], Integral)
             and isinstance(tup[2], bool)
             and isinstance(tup[3], bool)):
         return None
@@ -157,7 +152,7 @@ def tuple_to_qfont(tup):
 
 
 def qfont_to_tuple(font):
-    return (six.text_type(font.family()), int(font.pointSize()),
+    return (str(font.family()), int(font.pointSize()),
             font.italic(), font.bold())
 
 
@@ -176,7 +171,7 @@ class FontLayout(QtWidgets.QGridLayout):
         # Font size
         self.size = QtWidgets.QComboBox(parent)
         self.size.setEditable(True)
-        sizelist = list(range(6, 12)) + list(range(12, 30, 2)) + [36, 48, 72]
+        sizelist = [*range(6, 12), *range(12, 30, 2), 36, 48, 72]
         size = font.pointSize()
         if size not in sizelist:
             sizelist.append(size)
@@ -245,11 +240,15 @@ class FormWidget(QtWidgets.QWidget):
             elif (label.lower() not in BLACKLIST
                   and mcolors.is_color_like(value)):
                 field = ColorLayout(to_qcolor(value), self)
-            elif isinstance(value, six.string_types):
+            elif isinstance(value, str):
                 field = QtWidgets.QLineEdit(value, self)
             elif isinstance(value, (list, tuple)):
                 if isinstance(value, tuple):
                     value = list(value)
+                # Note: get() below checks the type of value[0] in self.data so
+                # it is essential that value gets modified in-place.
+                # This means that the code is actually broken in the case where
+                # value is a tuple, but fortunately we always pass a list...
                 selindex = value.pop(0)
                 field = QtWidgets.QComboBox(self)
                 if isinstance(value[0], (list, tuple)):
@@ -262,10 +261,10 @@ class FormWidget(QtWidgets.QWidget):
                     selindex = value.index(selindex)
                 elif selindex in keys:
                     selindex = keys.index(selindex)
-                elif not isinstance(selindex, int):
+                elif not isinstance(selindex, Integral):
                     warnings.warn(
                         "index '%s' is invalid (label: %s, value: %s)" %
-                        (selindex, label, value))
+                        (selindex, label, value), stacklevel=2)
                     selindex = 0
                 field.setCurrentIndex(selindex)
             elif isinstance(value, bool):
@@ -274,7 +273,11 @@ class FormWidget(QtWidgets.QWidget):
                     field.setCheckState(QtCore.Qt.Checked)
                 else:
                     field.setCheckState(QtCore.Qt.Unchecked)
-            elif isinstance(value, float):
+            elif isinstance(value, Integral):
+                field = QtWidgets.QSpinBox(self)
+                field.setRange(-1e9, 1e9)
+                field.setValue(value)
+            elif isinstance(value, Real):
                 field = QtWidgets.QLineEdit(repr(value), self)
                 field.setCursorPosition(0)
                 field.setValidator(QtGui.QDoubleValidator(field))
@@ -282,10 +285,6 @@ class FormWidget(QtWidgets.QWidget):
                 dialog = self.get_dialog()
                 dialog.register_float_field(field)
                 field.textChanged.connect(lambda text: dialog.update_buttons())
-            elif isinstance(value, int):
-                field = QtWidgets.QSpinBox(self)
-                field.setRange(-1e9, 1e9)
-                field.setValue(value)
             elif isinstance(value, datetime.datetime):
                 field = QtWidgets.QDateTimeEdit(self)
                 field.setDateTime(value)
@@ -306,9 +305,8 @@ class FormWidget(QtWidgets.QWidget):
                 continue
             elif tuple_to_qfont(value) is not None:
                 value = field.get_font()
-            elif (isinstance(value, six.string_types)
-                  or mcolors.is_color_like(value)):
-                value = six.text_type(field.text())
+            elif isinstance(value, str) or mcolors.is_color_like(value):
+                value = str(field.text())
             elif isinstance(value, (list, tuple)):
                 index = int(field.currentIndex())
                 if isinstance(value[0], (list, tuple)):
@@ -317,10 +315,10 @@ class FormWidget(QtWidgets.QWidget):
                     value = value[index]
             elif isinstance(value, bool):
                 value = field.checkState() == QtCore.Qt.Checked
-            elif isinstance(value, float):
-                value = float(str(field.text()))
-            elif isinstance(value, int):
+            elif isinstance(value, Integral):
                 value = int(field.value())
+            elif isinstance(value, Real):
+                value = float(str(field.text()))
             elif isinstance(value, datetime.datetime):
                 value = field.dateTime().toPyDateTime()
             elif isinstance(value, datetime.date):
