@@ -18,12 +18,8 @@ and :class:`Colorbar`; the :func:`~matplotlib.pyplot.colorbar` function
 is a thin wrapper over :meth:`~matplotlib.figure.Figure.colorbar`.
 
 '''
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-from six.moves import xrange, zip
 
+import logging
 import warnings
 
 import numpy as np
@@ -44,6 +40,8 @@ import matplotlib._layoutbox as layoutbox
 import matplotlib._constrained_layout as constrained_layout
 from matplotlib import docstring
 
+_log = logging.getLogger(__name__)
+
 make_axes_kw_doc = '''
 
     ============= ====================================================
@@ -152,7 +150,7 @@ mappable :
     default to the current image.
 
 cax : :class:`~matplotlib.axes.Axes` object, optional
-    Axis into which the colorbar will be drawn
+    Axes into which the colorbar will be drawn.
 
 ax : :class:`~matplotlib.axes.Axes`, list of Axes, optional
     Parent axes from which space for a new colorbar axes will be stolen.
@@ -217,6 +215,95 @@ def _set_ticks_on_axis_warn(*args, **kw):
     warnings.warn("Use the colorbar set_ticks() method instead.")
 
 
+class _ColorbarAutoLocator(ticker.MaxNLocator):
+    """
+    AutoLocator for Colorbar
+
+    This locator is just a `.MaxNLocator` except the min and max are
+    clipped by the norm's min and max (i.e. vmin/vmax from the
+    image/pcolor/contour object).  This is necessary so ticks don't
+    extrude into the "extend regions".
+    """
+
+    def __init__(self, colorbar):
+        """
+        This ticker needs to know the *colorbar* so that it can access
+        its *vmin* and *vmax*.  Otherwise it is the same as
+        `~.ticker.AutoLocator`.
+        """
+
+        self._colorbar = colorbar
+        nbins = 'auto'
+        steps = [1, 2, 2.5, 5, 10]
+        super().__init__(nbins=nbins, steps=steps)
+
+    def tick_values(self, vmin, vmax):
+        vmin = max(vmin, self._colorbar.norm.vmin)
+        vmax = min(vmax, self._colorbar.norm.vmax)
+        ticks = super().tick_values(vmin, vmax)
+        rtol = (vmax - vmin) * 1e-10
+        return ticks[(ticks >= vmin - rtol) & (ticks <= vmax + rtol)]
+
+
+class _ColorbarAutoMinorLocator(ticker.AutoMinorLocator):
+    """
+    AutoMinorLocator for Colorbar
+
+    This locator is just a `.AutoMinorLocator` except the min and max are
+    clipped by the norm's min and max (i.e. vmin/vmax from the
+    image/pcolor/contour object).  This is necessary so that the minorticks
+    don't extrude into the "extend regions".
+    """
+
+    def __init__(self, colorbar, n=None):
+        """
+        This ticker needs to know the *colorbar* so that it can access
+        its *vmin* and *vmax*.
+        """
+        self._colorbar = colorbar
+        self.ndivs = n
+        super().__init__(n=None)
+
+    def __call__(self):
+        vmin = self._colorbar.norm.vmin
+        vmax = self._colorbar.norm.vmax
+        ticks = super().__call__()
+        rtol = (vmax - vmin) * 1e-10
+        return ticks[(ticks >= vmin - rtol) & (ticks <= vmax + rtol)]
+
+
+class _ColorbarLogLocator(ticker.LogLocator):
+    """
+    LogLocator for Colorbarbar
+
+    This locator is just a `.LogLocator` except the min and max are
+    clipped by the norm's min and max (i.e. vmin/vmax from the
+    image/pcolor/contour object).  This is necessary so ticks don't
+    extrude into the "extend regions".
+
+    """
+    def __init__(self, colorbar, *args, **kwargs):
+        """
+        _ColorbarLogLocator(colorbar, *args, **kwargs)
+
+        This ticker needs to know the *colorbar* so that it can access
+        its *vmin* and *vmax*.  Otherwise it is the same as
+        `~.ticker.LogLocator`.  The ``*args`` and ``**kwargs`` are the
+        same as `~.ticker.LogLocator`.
+        """
+        self._colorbar = colorbar
+        super().__init__(*args, **kwargs)
+
+    def tick_values(self, vmin, vmax):
+        vmin = self._colorbar.norm.vmin
+        vmax = self._colorbar.norm.vmax
+        ticks = super().tick_values(vmin, vmax)
+        rtol = (np.log10(vmax) - np.log10(vmin)) * 1e-10
+        ticks = ticks[(np.log10(ticks) >= np.log10(vmin) - rtol) &
+              (np.log10(ticks) <= np.log10(vmax) + rtol)]
+        return ticks
+
+
 class ColorbarBase(cm.ScalarMappable):
     '''
     Draw a colorbar in an existing axes.
@@ -300,6 +387,7 @@ class ColorbarBase(cm.ScalarMappable):
         self.outline = None
         self.patch = None
         self.dividers = None
+        self._manual_tick_data_values = None
 
         if ticklocation == 'auto':
             ticklocation = 'bottom' if orientation == 'horizontal' else 'right'
@@ -318,12 +406,11 @@ class ColorbarBase(cm.ScalarMappable):
                                         linthresh=self.norm.linthresh)
             else:
                 self.formatter = ticker.ScalarFormatter()
-        elif isinstance(format, six.string_types):
+        elif isinstance(format, str):
             self.formatter = ticker.FormatStrFormatter(format)
         else:
             self.formatter = format  # Assume it is a Formatter
         # The rest is in a method so we can recalculate when clim changes.
-        self.config_axis()
         self.draw_all()
 
     def _extend_lower(self):
@@ -346,45 +433,122 @@ class ColorbarBase(cm.ScalarMappable):
         and do all the drawing.
         '''
 
+        # sets self._boundaries and self._values in real data units.
+        # takes into account extend values:
         self._process_values()
+        # sets self.vmin and vmax in data units, but just for
+        # the part of the colorbar that is not part of the extend
+        # patch:
         self._find_range()
+        # returns the X and Y mesh, *but* this was/is in normalized
+        # units:
         X, Y = self._mesh()
         C = self._values[:, np.newaxis]
+        self.config_axis()
         self._config_axes(X, Y)
         if self.filled:
             self._add_solids(X, Y, C)
 
     def config_axis(self):
         ax = self.ax
-        if self.orientation == 'vertical':
-            ax.xaxis.set_ticks([])
-            # location is either one of 'bottom' or 'top'
-            ax.yaxis.set_label_position(self.ticklocation)
-            ax.yaxis.set_ticks_position(self.ticklocation)
-        else:
-            ax.yaxis.set_ticks([])
-            # location is either one of 'left' or 'right'
-            ax.xaxis.set_label_position(self.ticklocation)
-            ax.xaxis.set_ticks_position(self.ticklocation)
+        if (isinstance(self.norm, colors.LogNorm)
+                and self._use_auto_colorbar_locator()):
+            # *both* axes are made log so that determining the
+            # mid point is easier.
+            ax.set_xscale('log')
+            ax.set_yscale('log')
 
+        if self.orientation == 'vertical':
+            long_axis, short_axis = ax.yaxis, ax.xaxis
+        else:
+            long_axis, short_axis = ax.xaxis, ax.yaxis
+
+        long_axis.set_label_position(self.ticklocation)
+        long_axis.set_ticks_position(self.ticklocation)
+        short_axis.set_ticks([])
+        short_axis.set_ticks([], minor=True)
         self._set_label()
 
+    def _get_ticker_locator_formatter(self):
+        """
+        This code looks at the norm being used by the colorbar
+        and decides what locator and formatter to use.  If ``locator`` has
+        already been set by hand, it just returns
+        ``self.locator, self.formatter``.
+        """
+        locator = self.locator
+        formatter = self.formatter
+        if locator is None:
+            if self.boundaries is None:
+                if isinstance(self.norm, colors.NoNorm):
+                    nv = len(self._values)
+                    base = 1 + int(nv / 10)
+                    locator = ticker.IndexLocator(base=base, offset=0)
+                elif isinstance(self.norm, colors.BoundaryNorm):
+                    b = self.norm.boundaries
+                    locator = ticker.FixedLocator(b, nbins=10)
+                elif isinstance(self.norm, colors.LogNorm):
+                    locator = _ColorbarLogLocator(self)
+                elif isinstance(self.norm, colors.SymLogNorm):
+                    # The subs setting here should be replaced
+                    # by logic in the locator.
+                    locator = ticker.SymmetricalLogLocator(
+                                      subs=np.arange(1, 10),
+                                      linthresh=self.norm.linthresh,
+                                      base=10)
+                else:
+                    if mpl.rcParams['_internal.classic_mode']:
+                        locator = ticker.MaxNLocator()
+                    else:
+                        locator = _ColorbarAutoLocator(self)
+            else:
+                b = self._boundaries[self._inside]
+                locator = ticker.FixedLocator(b, nbins=10)
+        _log.debug('locator: %r', locator)
+        return locator, formatter
+
+    def _use_auto_colorbar_locator(self):
+        """
+        Return if we should use an adjustable tick locator or a fixed
+        one.  (check is used twice so factored out here...)
+        """
+        return (self.boundaries is None
+                and self.values is None
+                and ((type(self.norm) == colors.Normalize)
+                    or (type(self.norm) == colors.LogNorm)))
+
     def update_ticks(self):
         """
         Force the update of the ticks and ticklabels. This must be
         called whenever the tick locator and/or tick formatter changes.
         """
         ax = self.ax
-        ticks, ticklabels, offset_string = self._ticker()
-        if self.orientation == 'vertical':
-            ax.yaxis.set_ticks(ticks)
-            ax.set_yticklabels(ticklabels)
-            ax.yaxis.get_major_formatter().set_offset_string(offset_string)
+        # get the locator and formatter.  Defaults to
+        # self.locator if not None..
+        locator, formatter = self._get_ticker_locator_formatter()
 
+        if self.orientation == 'vertical':
+            long_axis, short_axis = ax.yaxis, ax.xaxis
         else:
-            ax.xaxis.set_ticks(ticks)
-            ax.set_xticklabels(ticklabels)
-            ax.xaxis.get_major_formatter().set_offset_string(offset_string)
+            long_axis, short_axis = ax.xaxis, ax.yaxis
+
+        if self._use_auto_colorbar_locator():
+            _log.debug('Using auto colorbar locator on colorbar')
+            _log.debug('locator: %r', locator)
+            long_axis.set_major_locator(locator)
+            long_axis.set_major_formatter(formatter)
+            if type(self.norm) == colors.LogNorm:
+                long_axis.set_minor_locator(_ColorbarLogLocator(self,
+                            base=10., subs='auto'))
+                long_axis.set_minor_formatter(
+                    ticker.LogFormatterSciNotation()
+                )
+        else:
+            _log.debug('Using fixed locator on colorbar')
+            ticks, ticklabels, offset_string = self._ticker(locator, formatter)
+            long_axis.set_ticks(ticks)
+            long_axis.set_ticklabels(ticklabels)
+            long_axis.get_major_formatter().set_offset_string(offset_string)
 
     def set_ticks(self, ticks, update_ticks=True):
         """
@@ -411,7 +575,17 @@ class ColorbarBase(cm.ScalarMappable):
 
     def get_ticks(self, minor=False):
         """Return the x ticks as a list of locations"""
-        return self._tick_data_values
+        if self._manual_tick_data_values is None:
+            ax = self.ax
+            if self.orientation == 'vertical':
+                long_axis, short_axis = ax.yaxis, ax.xaxis
+            else:
+                long_axis, short_axis = ax.xaxis, ax.yaxis
+            return long_axis.get_majorticklocs()
+        else:
+            # We made the axes manually, the old way, and the ylim is 0-1,
+            # so the majorticklocs are in those units, not data units.
+            return self._manual_tick_data_values
 
     def set_ticklabels(self, ticklabels, update_ticks=True):
         """
@@ -435,6 +609,7 @@ class ColorbarBase(cm.ScalarMappable):
         ax.set_frame_on(False)
         ax.set_navigate(False)
         xy = self._outline(X, Y)
+        ax.ignore_existing_data_limits = True
         ax.update_datalim(xy)
         ax.set_xlim(*ax.dataLim.intervalx)
         ax.set_ylim(*ax.dataLim.intervaly)
@@ -498,9 +673,9 @@ class ColorbarBase(cm.ScalarMappable):
         # Using the non-array form of these line segments is much
         # simpler than making them into arrays.
         if self.orientation == 'vertical':
-            return [list(zip(X[i], Y[i])) for i in xrange(1, N - 1)]
+            return [list(zip(X[i], Y[i])) for i in range(1, N - 1)]
         else:
-            return [list(zip(Y[i], X[i])) for i in xrange(1, N - 1)]
+            return [list(zip(Y[i], X[i])) for i in range(1, N - 1)]
 
     def _add_solids(self, X, Y, C):
         '''
@@ -515,14 +690,9 @@ class ColorbarBase(cm.ScalarMappable):
                   norm=self.norm,
                   alpha=self.alpha,
                   edgecolors='None')
-        # Save, set, and restore hold state to keep pcolor from
-        # clearing the axes. Ordinarily this will not be needed,
-        # since the axes object should already have hold set.
-        _hold = self.ax._hold
-        self.ax._hold = True
+        _log.debug('Setting pcolormesh')
         col = self.ax.pcolormesh(*args, **kw)
-        self.ax._hold = _hold
-        #self.add_observer(col) # We should observe, not be observed...
+        # self.add_observer(col) # We should observe, not be observed...
 
         if self.solids is not None:
             self.solids.remove()
@@ -551,19 +721,18 @@ class ColorbarBase(cm.ScalarMappable):
         removing any previously added lines.
         '''
         y = self._locate(levels)
-        igood = (y < 1.001) & (y > -0.001)
+        rtol = (self._y[-1] - self._y[0]) * 1e-10
+        igood = (y < self._y[-1] + rtol) & (y > self._y[0] - rtol)
         y = y[igood]
         if cbook.iterable(colors):
             colors = np.asarray(colors)[igood]
         if cbook.iterable(linewidths):
             linewidths = np.asarray(linewidths)[igood]
-        N = len(y)
-        x = np.array([0.0, 1.0])
-        X, Y = np.meshgrid(x, y)
+        X, Y = np.meshgrid([self._y[0], self._y[-1]], y)
         if self.orientation == 'vertical':
-            xy = [list(zip(X[i], Y[i])) for i in xrange(N)]
+            xy = np.stack([X, Y], axis=-1)
         else:
-            xy = [list(zip(Y[i], X[i])) for i in xrange(N)]
+            xy = np.stack([Y, X], axis=-1)
         col = collections.LineCollection(xy, linewidths=linewidths)
 
         if erase and self.lines:
@@ -575,39 +744,11 @@ class ColorbarBase(cm.ScalarMappable):
         self.ax.add_collection(col)
         self.stale = True
 
-    def _ticker(self):
+    def _ticker(self, locator, formatter):
         '''
         Return the sequence of ticks (colorbar data locations),
         ticklabels (strings), and the corresponding offset string.
         '''
-        locator = self.locator
-        formatter = self.formatter
-        if locator is None:
-            if self.boundaries is None:
-                if isinstance(self.norm, colors.NoNorm):
-                    nv = len(self._values)
-                    base = 1 + int(nv / 10)
-                    locator = ticker.IndexLocator(base=base, offset=0)
-                elif isinstance(self.norm, colors.BoundaryNorm):
-                    b = self.norm.boundaries
-                    locator = ticker.FixedLocator(b, nbins=10)
-                elif isinstance(self.norm, colors.LogNorm):
-                    locator = ticker.LogLocator(subs='all')
-                elif isinstance(self.norm, colors.SymLogNorm):
-                    # The subs setting here should be replaced
-                    # by logic in the locator.
-                    locator = ticker.SymmetricalLogLocator(
-                                      subs=np.arange(1, 10),
-                                      linthresh=self.norm.linthresh,
-                                      base=10)
-                else:
-                    if mpl.rcParams['_internal.classic_mode']:
-                        locator = ticker.MaxNLocator()
-                    else:
-                        locator = ticker.AutoLocator()
-            else:
-                b = self._boundaries[self._inside]
-                locator = ticker.FixedLocator(b, nbins=10)
         if isinstance(self.norm, colors.NoNorm) and self.boundaries is None:
             intv = self._values[0], self._values[-1]
         else:
@@ -626,7 +767,7 @@ class ColorbarBase(cm.ScalarMappable):
         else:
             eps = (intv[1] - intv[0]) * 1e-10
             b = b[(b <= intv[1] + eps) & (b >= intv[0] - eps)]
-        self._tick_data_values = b
+        self._manual_tick_data_values = b
         ticks = self._locate(b)
         formatter.set_locs(b)
         ticklabels = [formatter(t, i) for i, t in enumerate(b)]
@@ -704,8 +845,9 @@ class ColorbarBase(cm.ScalarMappable):
 
             b = self.norm.inverse(self._uniform_y(self.cmap.N + 1))
 
-            if isinstance(self.norm, colors.LogNorm):
-                # If using a lognorm, ensure extensions don't go negative
+            if isinstance(self.norm, (colors.PowerNorm, colors.LogNorm)):
+                # If using a lognorm or powernorm, ensure extensions don't
+                # go negative
                 if self._extend_lower():
                     b[0] = 0.9 * b[0]
                 if self._extend_upper():
@@ -755,11 +897,10 @@ class ColorbarBase(cm.ScalarMappable):
         '''
         # Set the default value.
         extendlength = np.array([default, default])
-        if isinstance(frac, six.string_types):
+        if isinstance(frac, str):
             if frac.lower() == 'auto':
                 # Use the provided values when 'auto' is required.
-                extendlength[0] = automin
-                extendlength[1] = automax
+                extendlength[:] = [automin, automax]
             else:
                 # Any other string is invalid.
                 raise ValueError('invalid value for extendfrac')
@@ -847,17 +988,29 @@ class ColorbarBase(cm.ScalarMappable):
         transposition for a horizontal colorbar are done outside
         this function.
         '''
+        # if boundaries and values are None, then we can go ahead and
+        # scale this up for Auto tick location.  Otherwise we
+        # want to keep normalized between 0 and 1 and use manual tick
+        # locations.
+
         x = np.array([0.0, 1.0])
         if self.spacing == 'uniform':
             y = self._uniform_y(self._central_N())
         else:
             y = self._proportional_y()
+        if self._use_auto_colorbar_locator():
+            y = self.norm.inverse(y)
+            x = self.norm.inverse(x)
         self._y = y
         X, Y = np.meshgrid(x, y)
+        if self._use_auto_colorbar_locator():
+            xmid = self.norm.inverse(0.5)
+        else:
+            xmid = 0.5
         if self._extend_lower() and not self.extendrect:
-            X[0, :] = 0.5
+            X[0, :] = xmid
         if self._extend_upper() and not self.extendrect:
-            X[-1, :] = 0.5
+            X[-1, :] = xmid
         return X, Y
 
     def _locate(self, x):
@@ -1014,7 +1167,6 @@ class Colorbar(ColorbarBase):
         self.set_alpha(mappable.get_alpha())
         self.cmap = mappable.cmap
         self.norm = mappable.norm
-        self.config_axis()
         self.draw_all()
         if isinstance(self.mappable, contour.ContourSet):
             CS = self.mappable
@@ -1055,6 +1207,33 @@ class Colorbar(ColorbarBase):
             # use_gridspec was True
             ax.set_subplotspec(subplotspec)
 
+    def minorticks_on(self):
+        """
+        Turns on the minor ticks on the colorbar without extruding
+        into the "extend regions".
+        """
+        ax = self.ax
+        long_axis = ax.yaxis if self.orientation == 'vertical' else ax.xaxis
+
+        if long_axis.get_scale() == 'log':
+            warnings.warn('minorticks_on() has no effect on a '
+                          'logarithmic colorbar axis')
+        else:
+            long_axis.set_minor_locator(_ColorbarAutoMinorLocator(self))
+
+    def minorticks_off(self):
+        """
+        Turns off the minor ticks on the colorbar.
+        """
+        ax = self.ax
+        long_axis = ax.yaxis if self.orientation == 'vertical' else ax.xaxis
+
+        if long_axis.get_scale() == 'log':
+            warnings.warn('minorticks_off() has no effect on a '
+                          'logarithmic colorbar axis')
+        else:
+            long_axis.set_minor_locator(ticker.NullLocator())
+
 
 @docstring.Substitution(make_axes_kw_doc)
 def make_axes(parents, location=None, orientation=None, fraction=0.15,
@@ -1079,6 +1258,7 @@ def make_axes(parents, location=None, orientation=None, fraction=0.15,
     Returns (cax, kw), the child axes and the reduced kw dictionary to be
     passed when creating the colorbar instance.
     '''
+
     locations = ["left", "right", "top", "bottom"]
     if orientation is not None and location is not None:
         raise TypeError('position and orientation are mutually exclusive. '
@@ -1177,7 +1357,7 @@ def make_axes(parents, location=None, orientation=None, fraction=0.15,
 
     # transform each of the axes in parents using the new transform
     for ax in parents:
-        new_posn = shrinking_trans.transform(ax.get_position())
+        new_posn = shrinking_trans.transform(ax.get_position(original=True))
         new_posn = mtransforms.Bbox(new_posn)
         ax._set_position(new_posn)
         if parent_anchor is not False:
@@ -1214,7 +1394,7 @@ def make_axes(parents, location=None, orientation=None, fraction=0.15,
 
 
 @docstring.Substitution(make_axes_kw_doc)
-def make_axes_gridspec(parent, **kw):
+def make_axes_gridspec(parent, *, fraction=0.15, shrink=1.0, aspect=20, **kw):
     '''
     Resize and reposition a parent axes, and return a child axes
     suitable for a colorbar. This function is similar to
@@ -1251,10 +1431,6 @@ def make_axes_gridspec(parent, **kw):
     orientation = kw.setdefault('orientation', 'vertical')
     kw['ticklocation'] = 'auto'
 
-    fraction = kw.pop('fraction', 0.15)
-    shrink = kw.pop('shrink', 1.0)
-    aspect = kw.pop('aspect', 20)
-
     x1 = 1 - fraction
 
     # for shrinking
@@ -1328,12 +1504,6 @@ class ColorbarPatch(Colorbar):
         Draw the colors using :class:`~matplotlib.patches.Patch`;
         optionally add separators.
         """
-        # Save, set, and restore hold state to keep pcolor from
-        # clearing the axes. Ordinarily this will not be needed,
-        # since the axes object should already have hold set.
-        _hold = self.ax._hold
-        self.ax._hold = True
-
         kw = {'alpha': self.alpha, }
 
         n_segments = len(C)
@@ -1342,7 +1512,7 @@ class ColorbarPatch(Colorbar):
         hatches = self.mappable.hatches * n_segments
 
         patches = []
-        for i in xrange(len(X) - 1):
+        for i in range(len(X) - 1):
             val = C[i][0]
             hatch = hatches[i]
 
@@ -1379,8 +1549,6 @@ class ColorbarPatch(Colorbar):
                     linewidths=(0.5 * mpl.rcParams['axes.linewidth'],))
             self.ax.add_collection(self.dividers)
 
-        self.ax._hold = _hold
-
 
 def colorbar_factory(cax, mappable, **kwargs):
     """
@@ -1393,7 +1561,7 @@ def colorbar_factory(cax, mappable, **kwargs):
     # if the given mappable is a contourset with any hatching, use
     # ColorbarPatch else use Colorbar
     if (isinstance(mappable, contour.ContourSet)
-            and any([hatch is not None for hatch in mappable.hatches])):
+            and any(hatch is not None for hatch in mappable.hatches)):
         cb = ColorbarPatch(cax, mappable, **kwargs)
     else:
         cb = Colorbar(cax, mappable, **kwargs)
