@@ -1,13 +1,16 @@
 """
 This module defines default legend handlers.
 
+It is strongly encouraged to have read the :doc:`legend guide
+</tutorials/intermediate/legend_guide>` before this documentation.
+
 Legend handlers are expected to be a callable object with a following
 signature. ::
 
     legend_handler(legend, orig_handle, fontsize, handlebox)
 
 Where *legend* is the legend itself, *orig_handle* is the original
-plot, *fontsize* is the fontsize in pixles, and *handlebox* is a
+plot, *fontsize* is the fontsize in pixels, and *handlebox* is a
 OffsetBox instance. Within the call, you should create relevant
 artists (using relevant properties from the *legend* and/or
 *orig_handle*) and add them into the handlebox. The artists needs to
@@ -15,20 +18,20 @@ be scaled according to the fontsize (note that the size is in pixel,
 i.e., this is dpi-scaled value).
 
 This module includes definition of several legend handler classes
-derived from the base class (HandlerBase) with a following method.
-
-    def __call__(self, legend, orig_handle,
-                 fontsize,
-                 handlebox):
+derived from the base class (HandlerBase) with the following method::
 
+    def legend_artist(self, legend, orig_handle, fontsize, handlebox)
 
 """
 
+from itertools import cycle
+
 import numpy as np
 
 from matplotlib.lines import Line2D
 from matplotlib.patches import Rectangle
 import matplotlib.collections as mcoll
+import matplotlib.colors as mcolors
 
 
 def update_from_first_child(tgt, src):
@@ -81,43 +84,67 @@ class HandlerBase(object):
         height = height - self._ypad * fontsize
         return xdescent, ydescent, width, height
 
-    def __call__(self, legend, orig_handle,
-                 fontsize,
-                 handlebox):
+    def legend_artist(self, legend, orig_handle,
+                      fontsize, handlebox):
         """
-        x, y, w, h in display coordinate w/ default dpi (72)
-        fontsize in points
+        Return the artist that this HandlerBase generates for the given
+        original artist/handle.
+
+        Parameters
+        ----------
+        legend : :class:`matplotlib.legend.Legend` instance
+            The legend for which these legend artists are being created.
+        orig_handle : :class:`matplotlib.artist.Artist` or similar
+            The object for which these legend artists are being created.
+        fontsize : float or int
+            The fontsize in pixels. The artists being created should
+            be scaled according to the given fontsize.
+        handlebox : :class:`matplotlib.offsetbox.OffsetBox` instance
+            The box which has been created to hold this legend entry's
+            artists. Artists created in the `legend_artist` method must
+            be added to this handlebox inside this method.
+
         """
-        width, height, xdescent, ydescent = handlebox.width, \
-                                            handlebox.height, \
-                                            handlebox.xdescent, \
-                                            handlebox.ydescent
-
-        xdescent, ydescent, width, height = \
-                  self.adjust_drawing_area(legend, orig_handle,
-                                           xdescent, ydescent, width, height,
-                                           fontsize)
-
-        a_list = self.create_artists(legend, orig_handle,
-                                     xdescent, ydescent, width, height, fontsize,
-                                     handlebox.get_transform())
+        xdescent, ydescent, width, height = self.adjust_drawing_area(
+                 legend, orig_handle,
+                 handlebox.xdescent, handlebox.ydescent,
+                 handlebox.width, handlebox.height,
+                 fontsize)
+        artists = self.create_artists(legend, orig_handle,
+                                      xdescent, ydescent, width, height,
+                                      fontsize, handlebox.get_transform())
 
         # create_artists will return a list of artists.
-        for a in a_list:
+        for a in artists:
             handlebox.add_artist(a)
 
         # we only return the first artist
-        return a_list[0]
+        return artists[0]
 
     def create_artists(self, legend, orig_handle,
                        xdescent, ydescent, width, height, fontsize,
                        trans):
-
         raise NotImplementedError('Derived must override')
 
 
 class HandlerNpoints(HandlerBase):
+    """
+    A legend handler that shows *numpoints* points in the legend entry.
+    """
     def __init__(self, marker_pad=0.3, numpoints=None, **kw):
+        """
+        Parameters
+        ----------
+        marker_pad : float
+            Padding between points in legend entry.
+
+        numpoints : int
+            Number of points to show in legend entry.
+
+        Notes
+        -----
+        Any other keyword arguments are given to `HandlerBase`.
+        """
         HandlerBase.__init__(self, **kw)
 
         self._numpoints = numpoints
@@ -131,23 +158,39 @@ class HandlerNpoints(HandlerBase):
 
     def get_xdata(self, legend, xdescent, ydescent, width, height, fontsize):
         numpoints = self.get_numpoints(legend)
-
         if numpoints > 1:
-            # we put some pad here to compensate the size of the
-            # marker
-            xdata = np.linspace(-xdescent + self._marker_pad * fontsize,
-                                width - self._marker_pad * fontsize,
+            # we put some pad here to compensate the size of the marker
+            pad = self._marker_pad * fontsize
+            xdata = np.linspace(-xdescent + pad,
+                                -xdescent + width - pad,
                                 numpoints)
             xdata_marker = xdata
-        elif numpoints == 1:
-            xdata = np.linspace(-xdescent, width, 2)
-            xdata_marker = [0.5 * width - 0.5 * xdescent]
-
+        else:
+            xdata = np.linspace(-xdescent, -xdescent + width, 2)
+            xdata_marker = [-xdescent + 0.5 * width]
         return xdata, xdata_marker
 
 
 class HandlerNpointsYoffsets(HandlerNpoints):
+    """
+    A legend handler that shows *numpoints* in the legend, and allows them to
+    be individually offest in the y-direction.
+    """
     def __init__(self, numpoints=None, yoffsets=None, **kw):
+        """
+        Parameters
+        ----------
+        numpoints : int
+            Number of points to show in legend entry.
+
+        yoffsets : array of floats
+            Length *numpoints* list of y offsets for each point in
+            legend entry.
+
+        Notes
+        -----
+        Any other keyword arguments are given to `HandlerNpoints`.
+        """
         HandlerNpoints.__init__(self, numpoints=numpoints, **kw)
         self._yoffsets = yoffsets
 
@@ -162,10 +205,24 @@ class HandlerNpointsYoffsets(HandlerNpoints):
 
 class HandlerLine2D(HandlerNpoints):
     """
-    Handler for Line2D instances
+    Handler for `.Line2D` instances.
     """
     def __init__(self, marker_pad=0.3, numpoints=None, **kw):
-        HandlerNpoints.__init__(self, marker_pad=marker_pad, numpoints=numpoints, **kw)
+        """
+        Parameters
+        ----------
+        marker_pad : float
+            Padding between points in legend entry.
+
+        numpoints : int
+            Number of points to show in legend entry.
+
+        Notes
+        -----
+        Any other keyword arguments are given to `HandlerNpoints`.
+        """
+        HandlerNpoints.__init__(self, marker_pad=marker_pad,
+                                numpoints=numpoints, **kw)
 
     def create_artists(self, legend, orig_handle,
                        xdescent, ydescent, width, height, fontsize,
@@ -197,13 +254,31 @@ class HandlerLine2D(HandlerNpoints):
 
         return [legline, legline_marker]
 
+
 class HandlerPatch(HandlerBase):
     """
-    Handler for Patches
+    Handler for `.Patch` instances.
     """
     def __init__(self, patch_func=None, **kw):
-        HandlerBase.__init__(self, **kw)
+        """
+        Parameters
+        ----------
+        patch_func : callable, optional
+            The function that creates the legend key artist.
+            *patch_func* should have the signature::
 
+                def patch_func(legend=legend, orig_handle=orig_handle,
+                               xdescent=xdescent, ydescent=ydescent,
+                               width=width, height=height, fontsize=fontsize)
+
+            Subsequently the created artist will have its ``update_prop`` method
+            called and the appropriate transform will be applied.
+
+        Notes
+        -----
+        Any other keyword arguments are given to `HandlerBase`.
+        """
+        HandlerBase.__init__(self, **kw)
         self._patch_func = patch_func
 
     def _create_patch(self, legend, orig_handle,
@@ -219,10 +294,8 @@ class HandlerPatch(HandlerBase):
 
     def create_artists(self, legend, orig_handle,
                        xdescent, ydescent, width, height, fontsize, trans):
-
         p = self._create_patch(legend, orig_handle,
                                xdescent, ydescent, width, height, fontsize)
-
         self.update_prop(p, orig_handle, legend)
         p.set_transform(trans)
         return [p]
@@ -230,7 +303,7 @@ class HandlerPatch(HandlerBase):
 
 class HandlerLineCollection(HandlerLine2D):
     """
-    Handler for LineCollections
+    Handler for `.LineCollection` instances.
     """
     def get_numpoints(self, legend):
         if self._numpoints is None:
@@ -239,13 +312,12 @@ class HandlerLineCollection(HandlerLine2D):
             return self._numpoints
 
     def _default_update_prop(self, legend_handle, orig_handle):
-        lw = orig_handle.get_linewidth()[0]
-        dashes = orig_handle.get_dashes()[0]
+        lw = orig_handle.get_linewidths()[0]
+        dashes = orig_handle._us_linestyles[0]
         color = orig_handle.get_colors()[0]
         legend_handle.set_color(color)
+        legend_handle.set_linestyle(dashes)
         legend_handle.set_linewidth(lw)
-        if dashes[0] is not None: # dashed line
-            legend_handle.set_dashes(dashes[1])
 
     def create_artists(self, legend, orig_handle,
                        xdescent, ydescent, width, height, fontsize, trans):
@@ -263,7 +335,7 @@ class HandlerLineCollection(HandlerLine2D):
 
 class HandlerRegularPolyCollection(HandlerNpointsYoffsets):
     """
-    Handler for RegularPolyCollections.
+    Handler for `.RegularPolyCollections`.
     """
     def __init__(self, yoffsets=None, sizes=None, **kw):
         HandlerNpointsYoffsets.__init__(self, yoffsets=yoffsets, **kw)
@@ -277,15 +349,18 @@ class HandlerRegularPolyCollection(HandlerNpointsYoffsets):
             return self._numpoints
 
     def get_sizes(self, legend, orig_handle,
-                 xdescent, ydescent, width, height, fontsize):
+                  xdescent, ydescent, width, height, fontsize):
         if self._sizes is None:
-            size_max = max(orig_handle.get_sizes()) * legend.markerscale ** 2
-            size_min = min(orig_handle.get_sizes()) * legend.markerscale ** 2
+            handle_sizes = orig_handle.get_sizes()
+            if not len(handle_sizes):
+                handle_sizes = [1]
+            size_max = max(handle_sizes) * legend.markerscale ** 2
+            size_min = min(handle_sizes) * legend.markerscale ** 2
 
             numpoints = self.get_numpoints(legend)
             if numpoints < 4:
                 sizes = [.5 * (size_max + size_min), size_max,
-                         size_min]
+                         size_min][:numpoints]
             else:
                 rng = (size_max - size_min)
                 sizes = rng * np.linspace(0, 1, numpoints) + size_min
@@ -325,7 +400,7 @@ class HandlerRegularPolyCollection(HandlerNpointsYoffsets):
                                width, height, fontsize)
 
         p = self.create_collection(orig_handle, sizes,
-                                   offsets=zip(xdata_marker, ydata),
+                                   offsets=list(zip(xdata_marker, ydata)),
                                    transOffset=trans)
 
         self.update_prop(p, orig_handle, legend)
@@ -335,7 +410,7 @@ class HandlerRegularPolyCollection(HandlerNpointsYoffsets):
 
 class HandlerPathCollection(HandlerRegularPolyCollection):
     """
-    Handler for PathCollections, which are used by scatter
+    Handler for `.PathCollections`, which are used by `~.Axes.scatter`.
     """
     def create_collection(self, orig_handle, sizes, offsets, transOffset):
         p = type(orig_handle)([orig_handle.get_paths()[0]],
@@ -348,7 +423,7 @@ class HandlerPathCollection(HandlerRegularPolyCollection):
 
 class HandlerCircleCollection(HandlerRegularPolyCollection):
     """
-    Handler for CircleCollections
+    Handler for `.CircleCollections`.
     """
     def create_collection(self, orig_handle, sizes, offsets, transOffset):
         p = type(orig_handle)(sizes,
@@ -360,7 +435,7 @@ class HandlerCircleCollection(HandlerRegularPolyCollection):
 
 class HandlerErrorbar(HandlerLine2D):
     """
-    Handler for Errorbars
+    Handler for Errorbars.
     """
     def __init__(self, xerr_size=0.5, yerr_size=None,
                  marker_pad=0.3, numpoints=None, **kw):
@@ -368,10 +443,11 @@ class HandlerErrorbar(HandlerLine2D):
         self._xerr_size = xerr_size
         self._yerr_size = yerr_size
 
-        HandlerLine2D.__init__(self, marker_pad=marker_pad, numpoints=numpoints,
-                               **kw)
+        HandlerLine2D.__init__(self, marker_pad=marker_pad,
+                               numpoints=numpoints, **kw)
 
-    def get_err_size(self, legend, xdescent, ydescent, width, height, fontsize):
+    def get_err_size(self, legend, xdescent, ydescent,
+                     width, height, fontsize):
         xerr_size = self._xerr_size * fontsize
 
         if self._yerr_size is None:
@@ -393,7 +469,6 @@ class HandlerErrorbar(HandlerLine2D):
         ydata = ((height - ydescent) / 2.) * np.ones(xdata.shape, float)
         legline = Line2D(xdata, ydata)
 
-
         xdata_marker = np.asarray(xdata_marker)
         ydata_marker = np.asarray(ydata[:len(xdata_marker)])
 
@@ -470,12 +545,32 @@ class HandlerErrorbar(HandlerLine2D):
 
         return artists
 
+
 class HandlerStem(HandlerNpointsYoffsets):
     """
-    Handler for Errorbars
+    Handler for plots produced by `~.Axes.stem`.
     """
     def __init__(self, marker_pad=0.3, numpoints=None,
                  bottom=None, yoffsets=None, **kw):
+        """
+        Parameters
+        ----------
+        marker_pad : float
+            Padding between points in legend entry. Default is 0.3.
+
+        numpoints : int, optional
+            Number of points to show in legend entry.
+
+        bottom : float, optional
+
+        yoffsets : array of floats, optional
+            Length *numpoints* list of y offsets for each point in
+            legend entry.
+
+        Notes
+        -----
+        Any other keyword arguments are given to `HandlerNpointsYoffsets`.
+        """
 
         HandlerNpointsYoffsets.__init__(self, marker_pad=marker_pad,
                                         numpoints=numpoints,
@@ -519,7 +614,7 @@ class HandlerStem(HandlerNpointsYoffsets):
         for lm, m in zip(leg_stemlines, stemlines):
             self.update_prop(lm, m, legend)
 
-        leg_baseline = Line2D([np.amin(xdata), np.amax(xdata)],
+        leg_baseline = Line2D([np.min(xdata), np.max(xdata)],
                               [bottom, bottom])
 
         self.update_prop(leg_baseline, baseline, legend)
@@ -536,9 +631,25 @@ class HandlerStem(HandlerNpointsYoffsets):
 
 class HandlerTuple(HandlerBase):
     """
-    Handler for Tuple
+    Handler for Tuple.
+
+    Additional kwargs are passed through to `HandlerBase`.
+
+    Parameters
+    ----------
+    ndivide : int, optional
+        The number of sections to divide the legend area into. If None,
+        use the length of the input tuple. Default is 1.
+
+
+    pad : float, optional
+        If None, fall back to ``legend.borderpad`` as the default.
+        In units of fraction of font size. Default is None.
     """
-    def __init__(self, **kwargs):
+    def __init__(self, ndivide=1, pad=None, **kwargs):
+
+        self._ndivide = ndivide
+        self._pad = pad
         HandlerBase.__init__(self, **kwargs)
 
     def create_artists(self, legend, orig_handle,
@@ -546,13 +657,70 @@ class HandlerTuple(HandlerBase):
                        trans):
 
         handler_map = legend.get_legend_handler_map()
+
+        if self._ndivide is None:
+            ndivide = len(orig_handle)
+        else:
+            ndivide = self._ndivide
+
+        if self._pad is None:
+            pad = legend.borderpad * fontsize
+        else:
+            pad = self._pad * fontsize
+
+        if ndivide > 1:
+            width = (width - pad * (ndivide - 1)) / ndivide
+
+        xds_cycle = cycle(xdescent - (width + pad) * np.arange(ndivide))
+
         a_list = []
         for handle1 in orig_handle:
             handler = legend.get_legend_handler(handler_map, handle1)
-            _a_list = handler.create_artists(legend, handle1,
-                                             xdescent, ydescent, width, height,
-                                             fontsize,
-                                             trans)
+            _a_list = handler.create_artists(
+                legend, handle1,
+                next(xds_cycle), ydescent, width, height, fontsize, trans)
             a_list.extend(_a_list)
 
         return a_list
+
+
+class HandlerPolyCollection(HandlerBase):
+    """
+    Handler for `.PolyCollection` used in `~.Axes.fill_between` and `~.Axes.stackplot`.
+    """
+    def _update_prop(self, legend_handle, orig_handle):
+        def first_color(colors):
+            if colors is None:
+                return None
+            colors = mcolors.to_rgba_array(colors)
+            if len(colors):
+                return colors[0]
+            else:
+                return "none"
+
+        def get_first(prop_array):
+            if len(prop_array):
+                return prop_array[0]
+            else:
+                return None
+        edgecolor = getattr(orig_handle, '_original_edgecolor',
+                            orig_handle.get_edgecolor())
+        legend_handle.set_edgecolor(first_color(edgecolor))
+        facecolor = getattr(orig_handle, '_original_facecolor',
+                            orig_handle.get_facecolor())
+        legend_handle.set_facecolor(first_color(facecolor))
+        legend_handle.set_fill(orig_handle.get_fill())
+        legend_handle.set_hatch(orig_handle.get_hatch())
+        legend_handle.set_linewidth(get_first(orig_handle.get_linewidths()))
+        legend_handle.set_linestyle(get_first(orig_handle.get_linestyles()))
+        legend_handle.set_transform(get_first(orig_handle.get_transforms()))
+        legend_handle.set_figure(orig_handle.get_figure())
+        legend_handle.set_alpha(orig_handle.get_alpha())
+
+    def create_artists(self, legend, orig_handle,
+                       xdescent, ydescent, width, height, fontsize, trans):
+        p = Rectangle(xy=(-xdescent, -ydescent),
+                      width=width, height=height)
+        self.update_prop(p, orig_handle, legend)
+        p.set_transform(trans)
+        return [p]
