@@ -4,19 +4,15 @@ variety of line styles, markers and colors.
 """
 
 # TODO: expose cap and join style attrs
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-
+from numbers import Integral, Number, Real
 import warnings
 
 import numpy as np
 
-from . import artist, colors as mcolors, docstring, rcParams
+from . import artist, cbook, colors as mcolors, docstring, rcParams
 from .artist import Artist, allow_rasterization
 from .cbook import (
-    _to_unmasked_float_array, iterable, is_numlike, ls_mapper, ls_mapper_r,
+    _to_unmasked_float_array, iterable, ls_mapper, ls_mapper_r,
     STEP_LOOKUP_MAP)
 from .markers import MarkerStyle
 from .path import Path
@@ -24,7 +20,6 @@ from .transforms import Bbox, TransformedPath, IdentityTransform
 
 # Imported here for backward compatibility, even though they don't
 # really belong.
-from numpy import ma
 from . import _path
 from .markers import (
     CARETLEFT, CARETRIGHT, CARETUP, CARETDOWN,
@@ -36,7 +31,7 @@ def _get_dash_pattern(style):
     """Convert linestyle -> dash pattern
     """
     # go from short hand -> full strings
-    if isinstance(style, six.string_types):
+    if isinstance(style, str):
         style = ls_mapper.get(style, style)
     # un-dashed styles
     if style in ['solid', 'None']:
@@ -94,25 +89,21 @@ def segment_hits(cx, cy, x, y, radius):
     Lnorm_sq = dx ** 2 + dy ** 2  # Possibly want to eliminate Lnorm==0
     u = ((cx - xr) * dx + (cy - yr) * dy) / Lnorm_sq
     candidates = (u >= 0) & (u <= 1)
-    #if any(candidates): print "candidates",xr[candidates]
 
     # Note that there is a little area near one side of each point
     # which will be near neither segment, and another which will
     # be near both, depending on the angle of the lines.  The
     # following radius test eliminates these ambiguities.
     point_hits = (cx - x) ** 2 + (cy - y) ** 2 <= radius ** 2
-    #if any(point_hits): print "points",xr[candidates]
     candidates = candidates & ~(point_hits[:-1] | point_hits[1:])
 
     # For those candidates which remain, determine how far they lie away
     # from the line.
     px, py = xr + u * dx, yr + u * dy
     line_hits = (cx - px) ** 2 + (cy - py) ** 2 <= radius ** 2
-    #if any(line_hits): print "lines",xr[candidates]
     line_hits = line_hits & candidates
     points, = point_hits.ravel().nonzero()
     lines, = line_hits.ravel().nonzero()
-    #print points,lines
     return np.concatenate((points, lines))
 
 
@@ -136,90 +127,73 @@ def _mark_every_path(markevery, tpath, affine, ax_transform):
             return None
         return in_v[slc]
 
-    # if just a float, assume starting at 0.0 and make a tuple
-    if isinstance(markevery, float):
-        markevery = (0.0, markevery)
     # if just an int, assume starting at 0 and make a tuple
-    elif isinstance(markevery, int):
+    if isinstance(markevery, Integral):
         markevery = (0, markevery)
-    # if just an numpy int, assume starting at 0 and make a tuple
-    elif isinstance(markevery, np.integer):
-        markevery = (0, markevery.item())
+    # if just a float, assume starting at 0.0 and make a tuple
+    elif isinstance(markevery, Real):
+        markevery = (0.0, markevery)
 
     if isinstance(markevery, tuple):
         if len(markevery) != 2:
-            raise ValueError('`markevery` is a tuple but its '
-                'len is not 2; '
-                'markevery=%s' % (markevery,))
+            raise ValueError('`markevery` is a tuple but its len is not 2; '
+                             'markevery={}'.format(markevery))
         start, step = markevery
         # if step is an int, old behavior
-        if isinstance(step, int):
-            #tuple of 2 int is for backwards compatibility,
-            if not(isinstance(start, int)):
-                raise ValueError('`markevery` is a tuple with '
-                    'len 2 and second element is an int, but '
-                    'the first element is not an int; '
-                    'markevery=%s' % (markevery,))
+        if isinstance(step, Integral):
+            # tuple of 2 int is for backwards compatibility,
+            if not isinstance(start, Integral):
+                raise ValueError(
+                    '`markevery` is a tuple with len 2 and second element is '
+                    'an int, but the first element is not an int; markevery={}'
+                    .format(markevery))
             # just return, we are done here
 
             return Path(verts[slice(start, None, step)],
                         _slice_or_none(codes, slice(start, None, step)))
 
-        elif isinstance(step, float):
-            if not (isinstance(start, int) or
-                    isinstance(start, float)):
-                raise ValueError('`markevery` is a tuple with '
-                    'len 2 and second element is a float, but '
-                    'the first element is not a float or an '
-                    'int; '
-                    'markevery=%s' % (markevery,))
-            #calc cumulative distance along path (in display
-            # coords):
+        elif isinstance(step, Real):
+            if not isinstance(start, Real):
+                raise ValueError(
+                    '`markevery` is a tuple with len 2 and second element is '
+                    'a float, but the first element is not a float or an int; '
+                    'markevery={}'.format(markevery))
+            # calc cumulative distance along path (in display coords):
             disp_coords = affine.transform(tpath.vertices)
-            delta = np.empty((len(disp_coords), 2),
-                             dtype=float)
-            delta[0, :] = 0.0
-            delta[1:, :] = (disp_coords[1:, :] -
-                                disp_coords[:-1, :])
+            delta = np.empty((len(disp_coords), 2))
+            delta[0, :] = 0
+            delta[1:, :] = disp_coords[1:, :] - disp_coords[:-1, :]
             delta = np.sum(delta**2, axis=1)
             delta = np.sqrt(delta)
             delta = np.cumsum(delta)
-            #calc distance between markers along path based on
-            # the axes bounding box diagonal being a distance
-            # of unity:
-            scale = ax_transform.transform(
-                np.array([[0, 0], [1, 1]]))
+            # calc distance between markers along path based on the axes
+            # bounding box diagonal being a distance of unity:
+            scale = ax_transform.transform(np.array([[0, 0], [1, 1]]))
             scale = np.diff(scale, axis=0)
             scale = np.sum(scale**2)
             scale = np.sqrt(scale)
-            marker_delta = np.arange(start * scale,
-                                     delta[-1],
-                                     step * scale)
-            #find closest actual data point that is closest to
+            marker_delta = np.arange(start * scale, delta[-1], step * scale)
+            # find closest actual data point that is closest to
             # the theoretical distance along the path:
-            inds = np.abs(delta[np.newaxis, :] -
-                            marker_delta[:, np.newaxis])
+            inds = np.abs(delta[np.newaxis, :] - marker_delta[:, np.newaxis])
             inds = inds.argmin(axis=1)
             inds = np.unique(inds)
             # return, we are done here
             return Path(verts[inds],
                         _slice_or_none(codes, inds))
         else:
-            raise ValueError('`markevery` is a tuple with '
-                'len 2, but its second element is not an int '
-                'or a float; '
-                'markevery=%s' % (markevery,))
+            raise ValueError(
+                '`markevery` is a tuple with len 2, but its second element is '
+                'not an int or a float; markevery=%s' % (markevery,))
 
     elif isinstance(markevery, slice):
         # mazol tov, it's already a slice, just return
-        return Path(verts[markevery],
-                    _slice_or_none(codes, markevery))
+        return Path(verts[markevery], _slice_or_none(codes, markevery))
 
     elif iterable(markevery):
         #fancy indexing
         try:
-            return Path(verts[markevery],
-                    _slice_or_none(codes, markevery))
+            return Path(verts[markevery], _slice_or_none(codes, markevery))
 
         except (ValueError, IndexError):
             raise ValueError('`markevery` is iterable but '
@@ -231,15 +205,25 @@ def _mark_every_path(markevery, tpath, affine, ax_transform):
             'markevery=%s' % (markevery,))
 
 
+@cbook._define_aliases({
+    "antialiased": ["aa"],
+    "color": ["c"],
+    "linestyle": ["ls"],
+    "linewidth": ["lw"],
+    "markeredgecolor": ["mec"],
+    "markeredgewidth": ["mew"],
+    "markerfacecolor": ["mfc"],
+    "markerfacecoloralt": ["mfcalt"],
+    "markersize": ["ms"],
+})
 class Line2D(Artist):
     """
     A line - the line can have both a solid linestyle connecting all
     the vertices, and a marker at each vertex.  Additionally, the
     drawing of the solid line is influenced by the drawstyle, e.g., one
     can create "stepped" lines in various styles.
-
-
     """
+
     lineStyles = _lineStyles = {  # hidden names deprecated
         '-':    '_draw_solid',
         '--':   '_draw_dashed',
@@ -262,11 +246,9 @@ class Line2D(Artist):
     }
 
     # drawStyles should now be deprecated.
-    drawStyles = {}
-    drawStyles.update(_drawStyles_l)
-    drawStyles.update(_drawStyles_s)
+    drawStyles = {**_drawStyles_l, **_drawStyles_s}
     # Need a list ordered with long names first:
-    drawStyleKeys = list(_drawStyles_l) + list(_drawStyles_s)
+    drawStyleKeys = [*_drawStyles_l, *_drawStyles_s]
 
     # Referenced here to maintain API.  These are defined in
     # MarkerStyle
@@ -321,7 +303,7 @@ class Line2D(Artist):
 
         %(Line2D)s
 
-        See :meth:`set_linestyle` for a decription of the line styles,
+        See :meth:`set_linestyle` for a description of the line styles,
         :meth:`set_marker` for a description of the markers, and
         :meth:`set_drawstyle` for a description of the draw styles.
 
@@ -341,6 +323,10 @@ class Line2D(Artist):
             linestyle = rcParams['lines.linestyle']
         if marker is None:
             marker = rcParams['lines.marker']
+        if markerfacecolor is None:
+            markerfacecolor = rcParams['lines.markerfacecolor']
+        if markeredgecolor is None:
+            markeredgecolor = rcParams['lines.markeredgecolor']
         if color is None:
             color = rcParams['lines.color']
 
@@ -357,13 +343,11 @@ class Line2D(Artist):
         if solid_joinstyle is None:
             solid_joinstyle = rcParams['lines.solid_joinstyle']
 
-        if isinstance(linestyle, six.string_types):
+        if isinstance(linestyle, str):
             ds, ls = self._split_drawstyle_linestyle(linestyle)
             if ds is not None and drawstyle is not None and ds != drawstyle:
-                raise ValueError("Inconsistent drawstyle ({0!r}) and "
-                                 "linestyle ({1!r})".format(drawstyle,
-                                                            linestyle)
-                                 )
+                raise ValueError("Inconsistent drawstyle ({!r}) and linestyle "
+                                 "({!r})".format(drawstyle, linestyle))
             linestyle = ls
 
             if ds is not None:
@@ -393,9 +377,9 @@ class Line2D(Artist):
         self._us_dashSeq = None
         self._us_dashOffset = 0
 
+        self.set_linewidth(linewidth)
         self.set_linestyle(linestyle)
         self.set_drawstyle(drawstyle)
-        self.set_linewidth(linewidth)
 
         self._color = None
         self.set_color(color)
@@ -426,7 +410,7 @@ class Line2D(Artist):
         self.update(kwargs)
         self.pickradius = pickradius
         self.ind_offset = 0
-        if is_numlike(self._picker):
+        if isinstance(self._picker, Number):
             self.pickradius = self._picker
 
         self._xorig = np.asarray([])
@@ -461,7 +445,7 @@ class Line2D(Artist):
         if callable(self._contains):
             return self._contains(self, mouseevent)
 
-        if not is_numlike(self.pickradius):
+        if not isinstance(self.pickradius, Number):
             raise ValueError("pick radius should be a distance")
 
         # Make sure we have data to plot
@@ -512,9 +496,12 @@ class Line2D(Artist):
         return self.pickradius
 
     def set_pickradius(self, d):
-        """Sets the pick radius used for containment tests
+        """Set the pick radius used for containment tests.
 
-        ACCEPTS: float distance in points
+        Parameters
+        ----------
+        d : float
+            Pick radius, in points.
         """
         self.pickradius = d
 
@@ -529,7 +516,9 @@ class Line2D(Artist):
         Set the marker fill style; 'full' means fill the whole marker.
         'none' means no filling; other options are for half-filled markers.
 
-        ACCEPTS: ['full' | 'left' | 'right' | 'bottom' | 'top' | 'none']
+        Parameters
+        ----------
+        fs : {'full', 'left', 'right', 'bottom', 'top', 'none'}
         """
         self._marker.set_fillstyle(fs)
         self.stale = True
@@ -539,13 +528,10 @@ class Line2D(Artist):
 
         e.g., if `every=5`, every 5-th marker will be plotted.
 
-        ACCEPTS: [None | int | length-2 tuple of int | slice |
-        list/array of int | float | length-2 tuple of float]
-
         Parameters
         ----------
-        every: None | int | length-2 tuple of int | slice | list/array of int |
-        float | length-2 tuple of float
+        every: None or int or (int, int) or slice or List[int] or float or \
+(float, float)
             Which markers to plot.
 
             - every=None, every point will be plotted.
@@ -554,7 +540,7 @@ class Line2D(Artist):
             - every=(start, N), every N-th marker, starting at point
               start, will be plotted.
             - every=slice(start, end, N), every N-th marker, starting at
-              point start, upto but not including point end, will be plotted.
+              point start, up to but not including point end, will be plotted.
             - every=[i, j, m, n], only markers at points i, j, m, and n
               will be plotted.
             - every=0.1, (i.e. a float) then markers will be spaced at
@@ -596,8 +582,10 @@ class Line2D(Artist):
     def set_picker(self, p):
         """Sets the event picker details for the line.
 
-        ACCEPTS: float distance in points or callable pick function
-        ``fn(artist, event)``
+        Parameters
+        ----------
+        p : float or callable[[Artist, Event], Tuple[bool, dict]]
+            If a float, it is used as the pick radius in points.
         """
         if callable(p):
             self._contains = p
@@ -716,7 +704,9 @@ class Line2D(Artist):
         """
         set the Transformation instance used by this artist
 
-        ACCEPTS: a :class:`matplotlib.transforms.Transform` instance
+        Parameters
+        ----------
+        t : matplotlib.transforms.Transform
         """
         Artist.set_transform(self, t)
         self._invalidx = True
@@ -744,8 +734,8 @@ class Line2D(Artist):
             subslice = slice(max(i0 - 1, 0), i1 + 1)
             self.ind_offset = subslice.start
             self._transform_path(subslice)
-
-        transf_path = self._get_transformed_path()
+        else:
+            subslice = None
 
         if self.get_path_effects():
             from matplotlib.patheffects import PathEffectRenderer
@@ -753,14 +743,14 @@ class Line2D(Artist):
 
         renderer.open_group('line2d', self.get_gid())
         if self._lineStyles[self._linestyle] != '_draw_nothing':
-            tpath, affine = transf_path.get_transformed_path_and_affine()
+            tpath, affine = (self._get_transformed_path()
+                             .get_transformed_path_and_affine())
             if len(tpath.vertices):
                 gc = renderer.new_gc()
                 self._set_gc_clip(gc)
 
-                ln_color_rgba = self._get_rgba_ln_color()
-                gc.set_foreground(ln_color_rgba, isRGBA=True)
-                gc.set_alpha(ln_color_rgba[3])
+                lc_rgba = mcolors.to_rgba(self._color, self._alpha)
+                gc.set_foreground(lc_rgba, isRGBA=True)
 
                 gc.set_antialiased(self._antialiased)
                 gc.set_linewidth(self._linewidth)
@@ -784,25 +774,41 @@ class Line2D(Artist):
         if self._marker and self._markersize > 0:
             gc = renderer.new_gc()
             self._set_gc_clip(gc)
-            rgbaFace = self._get_rgba_face()
-            rgbaFaceAlt = self._get_rgba_face(alt=True)
-            edgecolor = self.get_markeredgecolor()
-            if (isinstance(edgecolor, six.string_types)
-                    and edgecolor.lower() == 'none'):
-                gc.set_linewidth(0)
-                gc.set_foreground(rgbaFace, isRGBA=True)
-            else:
-                gc.set_foreground(edgecolor)
-                gc.set_linewidth(self._markeredgewidth)
-                mec = self._markeredgecolor
-                if (isinstance(mec, six.string_types) and mec == 'auto' and
-                        rgbaFace is not None):
-                    gc.set_alpha(rgbaFace[3])
-                else:
-                    gc.set_alpha(self.get_alpha())
+            gc.set_linewidth(self._markeredgewidth)
+            gc.set_antialiased(self._antialiased)
+
+            ec_rgba = mcolors.to_rgba(
+                self.get_markeredgecolor(), self._alpha)
+            fc_rgba = mcolors.to_rgba(
+                self._get_markerfacecolor(), self._alpha)
+            fcalt_rgba = mcolors.to_rgba(
+                self._get_markerfacecolor(alt=True), self._alpha)
+            # If the edgecolor is "auto", it is set according to the *line*
+            # color but inherits the alpha value of the *face* color, if any.
+            if (cbook._str_equal(self._markeredgecolor, "auto")
+                    and not cbook._str_lower_equal(
+                        self.get_markerfacecolor(), "none")):
+                ec_rgba = ec_rgba[:3] + (fc_rgba[3],)
+            gc.set_foreground(ec_rgba, isRGBA=True)
+            if self.get_sketch_params() is not None:
+                scale, length, randomness = self.get_sketch_params()
+                gc.set_sketch_params(scale/2, length/2, 2*randomness)
 
             marker = self._marker
-            tpath, affine = transf_path.get_transformed_points_and_affine()
+
+            # Markers *must* be drawn ignoring the drawstyle (but don't pay the
+            # recaching if drawstyle is already "default").
+            if self.get_drawstyle() != "default":
+                with cbook._setattr_cm(
+                        self, _drawstyle="default", _transformed_path=None):
+                    self.recache()
+                    self._transform_path(subslice)
+                    tpath, affine = (self._get_transformed_path()
+                                    .get_transformed_path_and_affine())
+            else:
+                tpath, affine = (self._get_transformed_path()
+                                 .get_transformed_path_and_affine())
+
             if len(tpath.vertices):
                 # subsample the markers if markevery is not None
                 markevery = self.get_markevery()
@@ -813,7 +819,7 @@ class Line2D(Artist):
                     subsampled = tpath
 
                 snap = marker.get_snap_threshold()
-                if type(snap) == float:
+                if isinstance(snap, Real):
                     snap = renderer.points_to_pixels(self._markersize) >= snap
                 gc.set_snap(snap)
                 gc.set_joinstyle(marker.get_joinstyle())
@@ -822,30 +828,22 @@ class Line2D(Artist):
                 marker_trans = marker.get_transform()
                 w = renderer.points_to_pixels(self._markersize)
 
-                if (isinstance(marker.get_marker(), six.string_types) and
-                        marker.get_marker() == ','):
+                if cbook._str_equal(marker.get_marker(), ","):
                     gc.set_linewidth(0)
                 else:
                     # Don't scale for pixels, and don't stroke them
                     marker_trans = marker_trans.scale(w)
-
                 renderer.draw_markers(gc, marker_path, marker_trans,
                                       subsampled, affine.frozen(),
-                                      rgbaFace)
+                                      fc_rgba)
 
                 alt_marker_path = marker.get_alt_path()
                 if alt_marker_path:
                     alt_marker_trans = marker.get_alt_transform()
                     alt_marker_trans = alt_marker_trans.scale(w)
-                    if (isinstance(mec, six.string_types) and mec == 'auto' and
-                            rgbaFaceAlt is not None):
-                        gc.set_alpha(rgbaFaceAlt[3])
-                    else:
-                        gc.set_alpha(self.get_alpha())
-
                     renderer.draw_markers(
                             gc, alt_marker_path, alt_marker_trans, subsampled,
-                            affine.frozen(), rgbaFaceAlt)
+                            affine.frozen(), fcalt_rgba)
 
             gc.restore()
 
@@ -872,12 +870,12 @@ class Line2D(Artist):
 
     def get_markeredgecolor(self):
         mec = self._markeredgecolor
-        if isinstance(mec, six.string_types) and mec == 'auto':
+        if cbook._str_equal(mec, 'auto'):
             if rcParams['_internal.classic_mode']:
                 if self._marker.get_marker() in ('.', ','):
                     return self._color
                 if self._marker.is_filled() and self.get_fillstyle() != 'none':
-                     return 'k'  # Bad hard-wired default...
+                    return 'k'  # Bad hard-wired default...
             return self._color
         else:
             return mec
@@ -886,12 +884,8 @@ class Line2D(Artist):
         return self._markeredgewidth
 
     def _get_markerfacecolor(self, alt=False):
-        if alt:
-            fc = self._markerfacecoloralt
-        else:
-            fc = self._markerfacecolor
-
-        if (isinstance(fc, six.string_types) and fc.lower() == 'auto'):
+        fc = self._markerfacecoloralt if alt else self._markerfacecolor
+        if cbook._str_lower_equal(fc, 'auto'):
             if self.get_fillstyle() == 'none':
                 return 'none'
             else:
@@ -961,9 +955,11 @@ class Line2D(Artist):
 
     def set_antialiased(self, b):
         """
-        True if line should be drawin with antialiased rendering
+        Set whether to use antialiased rendering.
 
-        ACCEPTS: [True | False]
+        Parameters
+        ----------
+        b : bool
         """
         if self._antialiased != b:
             self.stale = True
@@ -973,7 +969,9 @@ class Line2D(Artist):
         """
         Set the color of the line
 
-        ACCEPTS: any matplotlib color
+        Parameters
+        ----------
+        color : color
         """
         self._color = color
         self.stale = True
@@ -986,8 +984,10 @@ class Line2D(Artist):
         produce step-plots. 'steps' is equivalent to 'steps-pre' and
         is maintained for backward-compatibility.
 
-        ACCEPTS: ['default' | 'steps' | 'steps-pre' | 'steps-mid' |
-                  'steps-post']
+        Parameters
+        ----------
+        drawstyle : {'default', 'steps', 'steps-pre', 'steps-mid', \
+'steps-post'}
         """
         if drawstyle is None:
             drawstyle = 'default'
@@ -995,13 +995,17 @@ class Line2D(Artist):
             raise ValueError('Unrecognized drawstyle {!r}'.format(drawstyle))
         if self._drawstyle != drawstyle:
             self.stale = True
+            # invalidate to trigger a recache of the path
+            self._invalidx = True
         self._drawstyle = drawstyle
 
     def set_linewidth(self, w):
         """
         Set the line width in points
 
-        ACCEPTS: float value in points
+        Parameters
+        ----------
+        w : float
         """
         w = float(w)
 
@@ -1032,17 +1036,10 @@ class Line2D(Artist):
         ls : str
             The linestyle with the drawstyle (if any) stripped.
         '''
-        ret_ds = None
         for ds in self.drawStyleKeys:  # long names are first in the list
             if ls.startswith(ds):
-                ret_ds = ds
-                if len(ls) > len(ds):
-                    ls = ls[len(ds):]
-                else:
-                    ls = '-'
-                break
-
-        return ret_ds, ls
+                return ds, ls[len(ds):] or '-'
+        return None, ls
 
     def set_linestyle(self, ls):
         """
@@ -1069,14 +1066,7 @@ class Line2D(Artist):
 
             (offset, onoffseq),
 
-        where ``onoffseq`` is an even length tuple of on and off ink
-        in points.
-
-
-        ACCEPTS: ['solid' | 'dashed', 'dashdot', 'dotted' |
-                   (offset, on-off-dash-seq) |
-                   ``'-'`` | ``'--'`` | ``'-.'`` | ``':'`` | ``'None'`` |
-                   ``' '`` | ``''``]
+        where ``onoffseq`` is an even length tuple of on and off ink in points.
 
         .. seealso::
 
@@ -1085,10 +1075,10 @@ class Line2D(Artist):
 
         Parameters
         ----------
-        ls : { ``'-'``,  ``'--'``, ``'-.'``, ``':'``} and more see description
+        ls : {'-', '--', '-.', ':', '', (offset, on-off-seq), ...}
             The line style.
         """
-        if isinstance(ls, six.string_types):
+        if isinstance(ls, str):
             ds, ls = self._split_drawstyle_linestyle(ls)
             if ds is not None:
                 self.set_drawstyle(ds)
@@ -1100,10 +1090,9 @@ class Line2D(Artist):
                 try:
                     ls = ls_mapper_r[ls]
                 except KeyError:
-                    raise ValueError(("You passed in an invalid linestyle, "
-                                      "`{0}`.  See "
-                                      "docs of Line2D.set_linestyle for "
-                                      "valid values.").format(ls))
+                    raise ValueError("Invalid linestyle {!r}; see docs of "
+                                     "Line2D.set_linestyle for valid values"
+                                     .format(ls))
             self._linestyle = ls
         else:
             self._linestyle = '--'
@@ -1117,39 +1106,39 @@ class Line2D(Artist):
     @docstring.dedent_interpd
     def set_marker(self, marker):
         """
-        Set the line marker
-
-        ACCEPTS: :mod:`A valid marker style <matplotlib.markers>`
+        Set the line marker.
 
         Parameters
         ----------
-
         marker: marker style
             See `~matplotlib.markers` for full description of possible
-            argument
-
+            arguments.
         """
         self._marker.set_marker(marker)
         self.stale = True
 
     def set_markeredgecolor(self, ec):
         """
-        Set the marker edge color
+        Set the marker edge color.
 
-        ACCEPTS: any matplotlib color
+        Parameters
+        ----------
+        ec : color
         """
         if ec is None:
             ec = 'auto'
-        if self._markeredgecolor is None or \
-           np.any(self._markeredgecolor != ec):
+        if (self._markeredgecolor is None
+                or np.any(self._markeredgecolor != ec)):
             self.stale = True
         self._markeredgecolor = ec
 
     def set_markeredgewidth(self, ew):
         """
-        Set the marker edge width in points
+        Set the marker edge width in points.
 
-        ACCEPTS: float value in points
+        Parameters
+        ----------
+        ew : float
         """
         if ew is None:
             ew = rcParams['lines.markeredgewidth']
@@ -1161,7 +1150,9 @@ class Line2D(Artist):
         """
         Set the marker face color.
 
-        ACCEPTS: any matplotlib color
+        Parameters
+        ----------
+        fc : color
         """
         if fc is None:
             fc = 'auto'
@@ -1173,7 +1164,9 @@ class Line2D(Artist):
         """
         Set the alternate marker face color.
 
-        ACCEPTS: any matplotlib color
+        Parameters
+        ----------
+        fc : color
         """
         if fc is None:
             fc = 'auto'
@@ -1183,9 +1176,11 @@ class Line2D(Artist):
 
     def set_markersize(self, sz):
         """
-        Set the marker size in points
+        Set the marker size in points.
 
-        ACCEPTS: float
+        Parameters
+        ----------
+        sz : float
         """
         sz = float(sz)
         if self._markersize != sz:
@@ -1194,9 +1189,11 @@ class Line2D(Artist):
 
     def set_xdata(self, x):
         """
-        Set the data np.array for x
+        Set the data array for x.
 
-        ACCEPTS: 1D array
+        Parameters
+        ----------
+        x : 1D array
         """
         self._xorig = x
         self._invalidx = True
@@ -1204,9 +1201,11 @@ class Line2D(Artist):
 
     def set_ydata(self, y):
         """
-        Set the data np.array for y
+        Set the data array for y.
 
-        ACCEPTS: 1D array
+        Parameters
+        ----------
+        y : 1D array
         """
         self._yorig = y
         self._invalidy = True
@@ -1218,7 +1217,9 @@ class Line2D(Artist):
         points.  If seq is empty or if seq = (None, None), the
         linestyle will be set to solid.
 
-        ACCEPTS: sequence of on/off ink in points
+        Parameters
+        ----------
+        seq : sequence of floats (on/off ink in points) or (None, None)
         """
         if seq == (None, None) or len(seq) == 0:
             self.set_linestyle('-')
@@ -1250,95 +1251,13 @@ class Line2D(Artist):
                                    other._marker.get_fillstyle())
         self._drawstyle = other._drawstyle
 
-    def _get_rgba_face(self, alt=False):
-        facecolor = self._get_markerfacecolor(alt=alt)
-        if (isinstance(facecolor, six.string_types)
-                and facecolor.lower() == 'none'):
-            rgbaFace = None
-        else:
-            rgbaFace = mcolors.to_rgba(facecolor, self._alpha)
-        return rgbaFace
-
-    def _get_rgba_ln_color(self, alt=False):
-        return mcolors.to_rgba(self._color, self._alpha)
-
-    # some aliases....
-    def set_aa(self, val):
-        'alias for set_antialiased'
-        self.set_antialiased(val)
-
-    def set_c(self, val):
-        'alias for set_color'
-        self.set_color(val)
-
-    def set_ls(self, val):
-        """alias for set_linestyle"""
-        self.set_linestyle(val)
-
-    def set_lw(self, val):
-        """alias for set_linewidth"""
-        self.set_linewidth(val)
-
-    def set_mec(self, val):
-        """alias for set_markeredgecolor"""
-        self.set_markeredgecolor(val)
-
-    def set_mew(self, val):
-        """alias for set_markeredgewidth"""
-        self.set_markeredgewidth(val)
-
-    def set_mfc(self, val):
-        """alias for set_markerfacecolor"""
-        self.set_markerfacecolor(val)
-
-    def set_mfcalt(self, val):
-        """alias for set_markerfacecoloralt"""
-        self.set_markerfacecoloralt(val)
-
-    def set_ms(self, val):
-        """alias for set_markersize"""
-        self.set_markersize(val)
-
-    def get_aa(self):
-        """alias for get_antialiased"""
-        return self.get_antialiased()
-
-    def get_c(self):
-        """alias for get_color"""
-        return self.get_color()
-
-    def get_ls(self):
-        """alias for get_linestyle"""
-        return self.get_linestyle()
-
-    def get_lw(self):
-        """alias for get_linewidth"""
-        return self.get_linewidth()
-
-    def get_mec(self):
-        """alias for get_markeredgecolor"""
-        return self.get_markeredgecolor()
-
-    def get_mew(self):
-        """alias for get_markeredgewidth"""
-        return self.get_markeredgewidth()
-
-    def get_mfc(self):
-        """alias for get_markerfacecolor"""
-        return self.get_markerfacecolor()
-
-    def get_mfcalt(self, alt=False):
-        """alias for get_markerfacecoloralt"""
-        return self.get_markerfacecoloralt()
-
-    def get_ms(self):
-        """alias for get_markersize"""
-        return self.get_markersize()
-
     def set_dash_joinstyle(self, s):
         """
-        Set the join style for dashed linestyles
-        ACCEPTS: ['miter' | 'round' | 'bevel']
+        Set the join style for dashed linestyles.
+
+        Parameters
+        ----------
+        s : {'miter', 'round', 'bevel'}
         """
         s = s.lower()
         if s not in self.validJoin:
@@ -1350,8 +1269,11 @@ class Line2D(Artist):
 
     def set_solid_joinstyle(self, s):
         """
-        Set the join style for solid linestyles
-        ACCEPTS: ['miter' | 'round' | 'bevel']
+        Set the join style for solid linestyles.
+
+        Parameters
+        ----------
+        s : {'miter', 'round', 'bevel'}
         """
         s = s.lower()
         if s not in self.validJoin:
@@ -1376,9 +1298,11 @@ class Line2D(Artist):
 
     def set_dash_capstyle(self, s):
         """
-        Set the cap style for dashed linestyles
+        Set the cap style for dashed linestyles.
 
-        ACCEPTS: ['butt' | 'round' | 'projecting']
+        Parameters
+        ----------
+        s : {'butt', 'round', 'projecting'}
         """
         s = s.lower()
         if s not in self.validCap:
@@ -1390,9 +1314,11 @@ class Line2D(Artist):
 
     def set_solid_capstyle(self, s):
         """
-        Set the cap style for solid linestyles
+        Set the cap style for solid linestyles.
 
-        ACCEPTS: ['butt' | 'round' |  'projecting']
+        Parameters
+        ----------
+        s : {'butt', 'round', 'projecting'}
         """
         s = s.lower()
         if s not in self.validCap:
@@ -1442,8 +1368,7 @@ class VertexSelector(object):
                 self.markers.set_data(xs, ys)
                 self.canvas.draw()
 
-        fig = plt.figure()
-        ax = fig.add_subplot(111)
+        fig, ax = plt.subplots()
         x, y = np.random.rand(2, 30)
         line, = ax.plot(x, y, 'bs-', picker=5)
 
@@ -1483,7 +1408,7 @@ class VertexSelector(object):
         pass
 
     def onpick(self, event):
-        """When the line is picked, update the set of selected indicies."""
+        """When the line is picked, update the set of selected indices."""
         if event.artist is not self.line:
             return
         self.ind ^= set(event.ind)
