@@ -131,7 +131,7 @@ _Matrix commands
   cumsum    - the cumulative sum along a dimension
   detrend   - remove the mean or besdt fit line from an array
   diag      - the k-th diagonal of matrix
-  diff      - the n-th differnce of an array
+  diff      - the n-th difference of an array
   eig       - the eigenvalues and eigen vectors of v
   eye       - a matrix where the k-th diagonal is ones, else zero
   find      - return the indices where a condition is nonzero
@@ -212,22 +212,10 @@ __end
 
 
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
-import six
-
-import sys, warnings
-
-from matplotlib.cbook import (
-    flatten, exception_to_str, silent_list, iterable, dedent)
+from matplotlib.cbook import flatten, silent_list, iterable, dedent
 
 import matplotlib as mpl
-# make mpl.finance module available for backwards compatability, in case folks
-# using pylab interface depended on not having to import it
-with warnings.catch_warnings():
-    warnings.simplefilter("ignore")  # deprecation: moved to a toolkit
-    import matplotlib.finance
 
 from matplotlib.dates import (
     date2num, num2date, datestr2num, strpdate2num, drange, epoch2num,
@@ -270,4 +258,4 @@ import datetime
 
 # This is needed, or bytes will be numpy.random.bytes from
 # "from numpy.random import *" above
-bytes = six.moves.builtins.bytes
+bytes = __import__("builtins").bytes
