@@ -2,24 +2,23 @@
 Provides a collection of utilities for comparing (image) results.
 
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-from matplotlib.externals import six
 
+import atexit
+import functools
 import hashlib
 import os
+from pathlib import Path
+import re
 import shutil
+import subprocess
+import sys
+from tempfile import TemporaryFile
 
 import numpy as np
 
 import matplotlib
-from matplotlib.compat import subprocess
 from matplotlib.testing.exceptions import ImageComparisonFailure
-from matplotlib import _png
-from matplotlib import _get_cachedir
-from matplotlib import cbook
-from distutils import version
+from matplotlib import _png, cbook
 
 __all__ = ['compare_float', 'compare_images', 'comparable_formats']
 
@@ -33,6 +32,7 @@ def make_test_filename(fname, purpose):
     return '%s-%s%s' % (base, purpose, ext)
 
 
+@cbook.deprecated("3.0")
 def compare_float(expected, actual, relTol=None, absTol=None):
     """
     Fail if the floating point values are not close enough, with
@@ -77,15 +77,14 @@ def compare_float(expected, actual, relTol=None, absTol=None):
 
 
 def get_cache_dir():
-    cachedir = _get_cachedir()
+    cachedir = matplotlib.get_cachedir()
     if cachedir is None:
         raise RuntimeError('Could not find a suitable configuration directory')
     cache_dir = os.path.join(cachedir, 'test_cache')
-    if not os.path.exists(cache_dir):
-        try:
-            cbook.mkdirs(cache_dir)
-        except IOError:
-            return None
+    try:
+        Path(cache_dir).mkdir(parents=True, exist_ok=True)
+    except IOError:
+        return None
     if not os.access(cache_dir, os.W_OK):
         return None
     return cache_dir
@@ -99,14 +98,22 @@ def get_file_hash(path, block_size=2 ** 20):
             if not data:
                 break
             md5.update(data)
+
+    if path.endswith('.pdf'):
+        from matplotlib import checkdep_ghostscript
+        md5.update(checkdep_ghostscript()[1].encode('utf-8'))
+    elif path.endswith('.svg'):
+        from matplotlib import checkdep_inkscape
+        md5.update(checkdep_inkscape().encode('utf-8'))
+
     return md5.hexdigest()
 
 
 def make_external_conversion_command(cmd):
     def convert(old, new):
         cmdline = cmd(old, new)
-        pipe = subprocess.Popen(
-            cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        pipe = subprocess.Popen(cmdline, universal_newlines=True,
+                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         stdout, stderr = pipe.communicate()
         errcode = pipe.wait()
         if not os.path.exists(new) or errcode:
@@ -120,19 +127,149 @@ def make_external_conversion_command(cmd):
     return convert
 
 
+# Modified from https://bugs.python.org/issue25567.
+_find_unsafe_bytes = re.compile(br'[^a-zA-Z0-9_@%+=:,./-]').search
+
+
+def _shlex_quote_bytes(b):
+    return (b if _find_unsafe_bytes(b) is None
+            else b"'" + b.replace(b"'", b"'\"'\"'") + b"'")
+
+
+class _ConverterError(Exception):
+    pass
+
+
+class _Converter(object):
+    def __init__(self):
+        self._proc = None
+        # Explicitly register deletion from an atexit handler because if we
+        # wait until the object is GC'd (which occurs later), then some module
+        # globals (e.g. signal.SIGKILL) has already been set to None, and
+        # kill() doesn't work anymore...
+        atexit.register(self.__del__)
+
+    def __del__(self):
+        if self._proc:
+            self._proc.kill()
+            self._proc.wait()
+            for stream in filter(None, [self._proc.stdin,
+                                        self._proc.stdout,
+                                        self._proc.stderr]):
+                stream.close()
+            self._proc = None
+
+    def _read_until(self, terminator):
+        """Read until the prompt is reached."""
+        buf = bytearray()
+        while True:
+            c = self._proc.stdout.read(1)
+            if not c:
+                raise _ConverterError
+            buf.extend(c)
+            if buf.endswith(terminator):
+                return bytes(buf[:-len(terminator)])
+
+
+class _GSConverter(_Converter):
+    def __call__(self, orig, dest):
+        if not self._proc:
+            self._stdout = TemporaryFile()
+            self._proc = subprocess.Popen(
+                [matplotlib.checkdep_ghostscript.executable,
+                 "-dNOPAUSE", "-sDEVICE=png16m"],
+                # As far as I can see, ghostscript never outputs to stderr.
+                stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+            try:
+                self._read_until(b"\nGS")
+            except _ConverterError:
+                raise OSError("Failed to start Ghostscript")
+
+        def encode_and_escape(name):
+            return (os.fsencode(name)
+                    .replace(b"\\", b"\\\\")
+                    .replace(b"(", br"\(")
+                    .replace(b")", br"\)"))
+
+        self._proc.stdin.write(
+            b"<< /OutputFile ("
+            + encode_and_escape(dest)
+            + b") >> setpagedevice ("
+            + encode_and_escape(orig)
+            + b") run flush\n")
+        self._proc.stdin.flush()
+        # GS> if nothing left on the stack; GS<n> if n items left on the stack.
+        err = self._read_until(b"GS")
+        stack = self._read_until(b">")
+        if stack or not os.path.exists(dest):
+            stack_size = int(stack[1:]) if stack else 0
+            self._proc.stdin.write(b"pop\n" * stack_size)
+            # Using the systemencoding should at least get the filenames right.
+            raise ImageComparisonFailure(
+                (err + b"GS" + stack + b">")
+                .decode(sys.getfilesystemencoding(), "replace"))
+
+
+class _SVGConverter(_Converter):
+    def __call__(self, orig, dest):
+        if (not self._proc  # First run.
+                or self._proc.poll() is not None):  # Inkscape terminated.
+            env = os.environ.copy()
+            # If one passes e.g. a png file to Inkscape, it will try to
+            # query the user for conversion options via a GUI (even with
+            # `--without-gui`).  Unsetting `DISPLAY` prevents this (and causes
+            # GTK to crash and Inkscape to terminate, but that'll just be
+            # reported as a regular exception below).
+            env.pop("DISPLAY", None)  # May already be unset.
+            # Do not load any user options.
+            env["INKSCAPE_PROFILE_DIR"] = os.devnull
+            # Old versions of Inkscape (0.48.3.1, used on Travis as of now)
+            # seem to sometimes deadlock when stderr is redirected to a pipe,
+            # so we redirect it to a temporary file instead.  This is not
+            # necessary anymore as of Inkscape 0.92.1.
+            stderr = TemporaryFile()
+            self._proc = subprocess.Popen(
+                ["inkscape", "--without-gui", "--shell"],
+                stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+                stderr=stderr, env=env)
+            # Slight abuse, but makes shutdown handling easier.
+            self._proc.stderr = stderr
+            try:
+                self._read_until(b"\n>")
+            except _ConverterError:
+                raise OSError("Failed to start Inkscape in interactive mode")
+
+        # Inkscape uses glib's `g_shell_parse_argv`, which has a consistent
+        # behavior across platforms, so we can just use `shlex.quote`.
+        orig_b, dest_b = map(_shlex_quote_bytes,
+                             map(os.fsencode, [orig, dest]))
+        if b"\n" in orig_b or b"\n" in dest_b:
+            # Who knows whether the current folder name has a newline, or if
+            # our encoding is even ASCII compatible...  Just fall back on the
+            # slow solution (Inkscape uses `fgets` so it will always stop at a
+            # newline).
+            return make_external_conversion_command(lambda old, new: [
+                'inkscape', '-z', old, '--export-png', new])(orig, dest)
+        self._proc.stdin.write(orig_b + b" --export-png=" + dest_b + b"\n")
+        self._proc.stdin.flush()
+        try:
+            self._read_until(b"\n>")
+        except _ConverterError:
+            # Inkscape's output is not localized but gtk's is, so the output
+            # stream probably has a mixed encoding.  Using the filesystem
+            # encoding should at least get the filenames right...
+            self._stderr.seek(0)
+            raise ImageComparisonFailure(
+                self._stderr.read().decode(
+                    sys.getfilesystemencoding(), "replace"))
+
+
 def _update_converter():
     gs, gs_v = matplotlib.checkdep_ghostscript()
     if gs_v is not None:
-        def cmd(old, new):
-            return [gs, '-q', '-sDEVICE=png16m', '-dNOPAUSE', '-dBATCH',
-             '-sOutputFile=' + new, old]
-        converter['pdf'] = make_external_conversion_command(cmd)
-        converter['eps'] = make_external_conversion_command(cmd)
-
+        converter['pdf'] = converter['eps'] = _GSConverter()
     if matplotlib.checkdep_inkscape() is not None:
-        def cmd(old, new):
-            return ['inkscape', '-z', old, '--export-png', new]
-        converter['svg'] = make_external_conversion_command(cmd)
+        converter['svg'] = _SVGConverter()
 
 
 #: A dictionary that maps filename extensions to functions which
@@ -149,25 +286,28 @@ def comparable_formats():
     on this system.
 
     """
-    return ['png'] + list(six.iterkeys(converter))
+    return ['png', *converter]
 
 
 def convert(filename, cache):
     """
-    Convert the named file into a png file.  Returns the name of the
-    created file.
+    Convert the named file to png; return the name of the created file.
 
     If *cache* is True, the result of the conversion is cached in
-    `matplotlib._get_cachedir() + '/test_cache/'`.  The caching is based
-    on a hash of the exact contents of the input file.  The is no limit
-    on the size of the cache, so it may need to be manually cleared
-    periodically.
-
+    `matplotlib.get_cachedir() + '/test_cache/'`.  The caching is based on a
+    hash of the exact contents of the input file.  There is no limit on the
+    size of the cache, so it may need to be manually cleared periodically.
     """
     base, extension = filename.rsplit('.', 1)
     if extension not in converter:
-        raise ImageComparisonFailure(
-            "Don't know how to convert %s files to png" % extension)
+        reason = "Don't know how to convert %s files to png" % extension
+        from . import is_called_from_pytest
+        if is_called_from_pytest():
+            import pytest
+            pytest.skip(reason)
+        else:
+            from nose import SkipTest
+            raise SkipTest(reason)
     newname = base + '_' + extension + '.png'
     if not os.path.exists(filename):
         raise IOError("'%s' does not exist" % filename)
@@ -196,45 +336,13 @@ def convert(filename, cache):
 
     return newname
 
-#: Maps file extensions to a function which takes a filename as its
-#: only argument to return a list suitable for execution with Popen.
-#: The purpose of this is so that the result file (with the given
-#: extension) can be verified with tools such as xmllint for svg.
-verifiers = {}
-
-# Turning this off, because it seems to cause multiprocessing issues
-if matplotlib.checkdep_xmllint() and False:
-    verifiers['svg'] = lambda filename: [
-        'xmllint', '--valid', '--nowarning', '--noout', filename]
-
-
-def verify(filename):
-    """Verify the file through some sort of verification tool."""
-    if not os.path.exists(filename):
-        raise IOError("'%s' does not exist" % filename)
-    base, extension = filename.rsplit('.', 1)
-    verifier = verifiers.get(extension, None)
-    if verifier is not None:
-        cmd = verifier(filename)
-        pipe = subprocess.Popen(
-            cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        stdout, stderr = pipe.communicate()
-        errcode = pipe.wait()
-        if errcode != 0:
-            msg = "File verification command failed:\n%s\n" % ' '.join(cmd)
-            if stdout:
-                msg += "Standard output:\n%s\n" % stdout
-            if stderr:
-                msg += "Standard error:\n%s\n" % stderr
-            raise IOError(msg)
-
 
 def crop_to_same(actual_path, actual_image, expected_path, expected_image):
     # clip the images to the same size -- this is useful only when
     # comparing eps to pdf
     if actual_path[-7:-4] == 'eps' and expected_path[-7:-4] == 'pdf':
-        aw, ah = actual_image.shape
-        ew, eh = expected_image.shape
+        aw, ah, ad = actual_image.shape
+        ew, eh, ed = expected_image.shape
         actual_image = actual_image[int(aw / 2 - ew / 2):int(
             aw / 2 + ew / 2), int(ah / 2 - eh / 2):int(ah / 2 + eh / 2)]
     return actual_image, expected_image
@@ -242,22 +350,12 @@ def crop_to_same(actual_path, actual_image, expected_path, expected_image):
 
 def calculate_rms(expectedImage, actualImage):
     "Calculate the per-pixel errors, then compute the root mean square error."
-    num_values = np.prod(expectedImage.shape)
-    abs_diff_image = abs(expectedImage - actualImage)
-
-    # On Numpy 1.6, we can use bincount with minlength, which is much
-    # faster than using histogram
-    expected_version = version.LooseVersion("1.6")
-    found_version = version.LooseVersion(np.__version__)
-    if found_version >= expected_version:
-        histogram = np.bincount(abs_diff_image.ravel(), minlength=256)
-    else:
-        histogram = np.histogram(abs_diff_image, bins=np.arange(257))[0]
-
-    sum_of_squares = np.sum(histogram * np.arange(len(histogram)) ** 2)
-    rms = np.sqrt(float(sum_of_squares) / num_values)
-
-    return rms
+    if expectedImage.shape != actualImage.shape:
+        raise ImageComparisonFailure(
+            "Image sizes do not match expected size: {} "
+            "actual size {}".format(expectedImage.shape, actualImage.shape))
+    # Convert to float to avoid overflowing finite integer types.
+    return np.sqrt(((expectedImage - actualImage).astype(float) ** 2).mean())
 
 
 def compare_images(expected, actual, tol, in_decorator=False):
@@ -282,22 +380,18 @@ def compare_images(expected, actual, tol, in_decorator=False):
         If called from image_comparison decorator, this should be
         True. (default=False)
 
-    Example
-    -------
+    Examples
+    --------
     img1 = "./baseline/plot.png"
     img2 = "./output/plot.png"
-    compare_images( img1, img2, 0.001 ):
+    compare_images(img1, img2, 0.001):
 
     """
     if not os.path.exists(actual):
-        msg = "Output image %s does not exist." % actual
-        raise Exception(msg)
+        raise Exception("Output image %s does not exist." % actual)
 
     if os.stat(actual).st_size == 0:
-        msg = "Output image file %s is empty." % actual
-        raise Exception(msg)
-
-    verify(actual)
+        raise Exception("Output image file %s is empty." % actual)
 
     # Convert the image to png
     extension = expected.split('.')[-1]
@@ -318,6 +412,12 @@ def compare_images(expected, actual, tol, in_decorator=False):
     actualImage, expectedImage = crop_to_same(
         actual, actualImage, expected, expectedImage)
 
+    diff_image = make_test_filename(actual, 'failed-diff')
+
+    if tol <= 0:
+        if np.array_equal(expectedImage, actualImage):
+            return None
+
     # convert to signed integers, so that the images can be subtracted without
     # overflow
     expectedImage = expectedImage.astype(np.int16)
@@ -325,11 +425,7 @@ def compare_images(expected, actual, tol, in_decorator=False):
 
     rms = calculate_rms(expectedImage, actualImage)
 
-    diff_image = make_test_filename(actual, 'failed-diff')
-
     if rms <= tol:
-        if os.path.exists(diff_image):
-            os.unlink(diff_image)
         return None
 
     save_diff_image(expected, actual, diff_image)
@@ -350,15 +446,28 @@ def compare_images(expected, actual, tol, in_decorator=False):
 
 
 def save_diff_image(expected, actual, output):
-    expectedImage = _png.read_png(expected)
-    actualImage = _png.read_png(actual)
+    '''
+    Parameters
+    ----------
+    expected : str
+        File path of expected image.
+    actual : str
+        File path of actual image.
+    output : str
+        File path to save difference image to.
+    '''
+    # Drop alpha channels, similarly to compare_images.
+    expectedImage = _png.read_png(expected)[..., :3]
+    actualImage = _png.read_png(actual)[..., :3]
     actualImage, expectedImage = crop_to_same(
         actual, actualImage, expected, expectedImage)
-    expectedImage = np.array(expectedImage).astype(np.float)
-    actualImage = np.array(actualImage).astype(np.float)
-    assert expectedImage.ndim == actualImage.ndim
-    assert expectedImage.shape == actualImage.shape
-    absDiffImage = abs(expectedImage - actualImage)
+    expectedImage = np.array(expectedImage).astype(float)
+    actualImage = np.array(actualImage).astype(float)
+    if expectedImage.shape != actualImage.shape:
+        raise ImageComparisonFailure(
+            "Image sizes do not match expected size: {} "
+            "actual size {}".format(expectedImage.shape, actualImage.shape))
+    absDiffImage = np.abs(expectedImage - actualImage)
 
     # expand differences in luminance domain
     absDiffImage *= 255 * 10
