@@ -1,6 +1,5 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
+import pytest
+import platform
 import matplotlib.pyplot as plt
 from matplotlib.testing.decorators import image_comparison
 import matplotlib.patches as mpatches
@@ -63,13 +62,14 @@ def __prepare_fancyarrow_dpi_cor_test():
     ax.set_xlim([0, 1])
     ax.set_ylim([0, 1])
     ax.add_patch(mpatches.FancyArrowPatch(posA=(0.3, 0.4), posB=(0.8, 0.6),
-                                          lw=3, arrowstyle=u'->',
+                                          lw=3, arrowstyle='->',
                                           mutation_scale=100))
     return fig2
 
 
 @image_comparison(baseline_images=['fancyarrow_dpi_cor_100dpi'],
                   remove_text=True, extensions=['png'],
+                  tol={'aarch64': 0.02}.get(platform.machine(), 0.0),
                   savefig_kwarg=dict(dpi=100))
 def test_fancyarrow_dpi_cor_100dpi():
     """
@@ -85,6 +85,7 @@ def test_fancyarrow_dpi_cor_100dpi():
 
 @image_comparison(baseline_images=['fancyarrow_dpi_cor_200dpi'],
                   remove_text=True, extensions=['png'],
+                  tol={'aarch64': 0.02}.get(platform.machine(), 0.0),
                   savefig_kwarg=dict(dpi=200))
 def test_fancyarrow_dpi_cor_200dpi():
     """
@@ -119,3 +120,51 @@ def test_fancyarrow_dash():
                          color='k')
     ax.add_patch(e)
     ax.add_patch(e2)
+
+
+@image_comparison(baseline_images=['arrow_styles'], extensions=['png'],
+                  style='mpl20', remove_text=True)
+def test_arrow_styles():
+    styles = mpatches.ArrowStyle.get_styles()
+
+    n = len(styles)
+    fig, ax = plt.subplots(figsize=(6, 10))
+    ax.set_xlim(0, 1)
+    ax.set_ylim(-1, n)
+
+    for i, stylename in enumerate(sorted(styles)):
+        patch = mpatches.FancyArrowPatch((0.1, i), (0.8, i),
+                                         arrowstyle=stylename,
+                                         mutation_scale=25)
+        ax.add_patch(patch)
+
+
+@image_comparison(baseline_images=['connection_styles'], extensions=['png'],
+                  style='mpl20', remove_text=True)
+def test_connection_styles():
+    styles = mpatches.ConnectionStyle.get_styles()
+
+    n = len(styles)
+    fig, ax = plt.subplots(figsize=(6, 10))
+    ax.set_xlim(0, 1)
+    ax.set_ylim(-1, n)
+
+    for i, stylename in enumerate(sorted(styles)):
+        patch = mpatches.FancyArrowPatch((0.1, i), (0.8, i + 0.5),
+                                         arrowstyle="->",
+                                         connectionstyle=stylename,
+                                         mutation_scale=25)
+        ax.add_patch(patch)
+
+
+def test_invalid_intersection():
+    conn_style_1 = mpatches.ConnectionStyle.Angle3(angleA=20, angleB=200)
+    p1 = mpatches.FancyArrowPatch((.2, .2), (.5, .5),
+                                  connectionstyle=conn_style_1)
+    with pytest.raises(ValueError):
+        plt.gca().add_patch(p1)
+
+    conn_style_2 = mpatches.ConnectionStyle.Angle3(angleA=20, angleB=199.9)
+    p2 = mpatches.FancyArrowPatch((.2, .2), (.5, .5),
+                                  connectionstyle=conn_style_2)
+    plt.gca().add_patch(p2)
