@@ -1,14 +1,7 @@
-# -*- encoding: utf-8 -*-
-
-from __future__ import absolute_import, division, print_function
-
-import six
-
 import io
 import os
 import sys
 import tempfile
-import warnings
 
 import numpy as np
 import pytest
@@ -21,11 +14,9 @@ from matplotlib.testing.determinism import (_determinism_source_date_epoch,
                                             _determinism_check)
 
 
-with warnings.catch_warnings():
-    warnings.simplefilter('ignore')
-    needs_usetex = pytest.mark.skipif(
-        not checkdep_usetex(True),
-        reason="This test needs a TeX installation")
+needs_usetex = pytest.mark.skipif(
+    not checkdep_usetex(True),
+    reason="This test needs a TeX installation")
 
 
 @image_comparison(baseline_images=['pdf_use14corefonts'],
@@ -37,7 +28,7 @@ def test_use14corefonts():
     rcParams['font.sans-serif'] = ['Helvetica']
     rcParams['pdf.compression'] = 0
 
-    text = u'''A three-line text positioned just above a blue line
+    text = '''A three-line text positioned just above a blue line
 and containing some French characters and the euro symbol:
 "Merci pépé pour les 10 €"'''
 
@@ -243,9 +234,8 @@ def test_failing_latex(tmpdir):
         plt.savefig(path)
 
 
-def test_empty_rasterised():
+def test_empty_rasterized():
     # Check that emtpy figures that are rasterised save to pdf files fine
-    with PdfPages(io.BytesIO()) as pdf:
-        fig, ax = plt.subplots()
-        ax.plot([], [], rasterized=True)
-        fig.savefig(pdf, format="pdf")
+    fig, ax = plt.subplots()
+    ax.plot([], [], rasterized=True)
+    fig.savefig(io.BytesIO(), format="pdf")
