@@ -1,7 +1,4 @@
-from __future__ import absolute_import, division, print_function
-
-import six
-import sys
+import builtins
 
 import matplotlib
 
@@ -24,14 +21,6 @@ def test_override_builtins():
         'sum',
         'divmod'
     }
-
-    # We could use six.moves.builtins here, but that seems
-    # to do a little more than just this.
-    if six.PY3:
-        builtins = sys.modules['builtins']
-    else:
-        builtins = sys.modules['__builtin__']
-
     overridden = False
     for key in dir(pylab):
         if key in dir(builtins):
