@@ -1,16 +1,14 @@
-from __future__ import absolute_import, division, print_function
 import itertools
 import pickle
 from weakref import ref
 import warnings
-
-import six
+from unittest.mock import patch, Mock
 
 from datetime import datetime
 
 import numpy as np
-from numpy.testing.utils import (assert_array_equal, assert_approx_equal,
-                                 assert_array_almost_equal)
+from numpy.testing import (assert_array_equal, assert_approx_equal,
+                           assert_array_almost_equal)
 import pytest
 
 import matplotlib.cbook as cbook
@@ -26,24 +24,6 @@ def test_is_hashable():
     assert not cbook.is_hashable(lst)
 
 
-def test_restrict_dict():
-    d = {'foo': 'bar', 1: 2}
-    with pytest.warns(cbook.deprecation.MatplotlibDeprecationWarning) as rec:
-        d1 = cbook.restrict_dict(d, ['foo', 1])
-        assert d1 == d
-        d2 = cbook.restrict_dict(d, ['bar', 2])
-        assert d2 == {}
-        d3 = cbook.restrict_dict(d, {'foo': 1})
-        assert d3 == {'foo': 'bar'}
-        d4 = cbook.restrict_dict(d, {})
-        assert d4 == {}
-        d5 = cbook.restrict_dict(d, {'foo', 2})
-        assert d5 == {'foo': 'bar'}
-    assert len(rec) == 5
-    # check that d was not modified
-    assert d == {'foo': 'bar', 1: 2}
-
-
 class Test_delete_masked_points(object):
     def setup_method(self):
         self.mask1 = [False, False, True, True, False, False]
@@ -371,6 +351,15 @@ def test_normalize_kwargs_pass(inp, expected, kwargs_to_norm):
         assert len(w) == 0
 
 
+def test_warn_external_frame_embedded_python():
+    with patch.object(cbook, "sys") as mock_sys:
+        mock_sys._getframe = Mock(return_value=None)
+        with warnings.catch_warnings(record=True) as w:
+            cbook._warn_external("dummy")
+    assert len(w) == 1
+    assert str(w[0].message) == "dummy"
+
+
 def test_to_prestep():
     x = np.arange(4)
     y1 = np.arange(4)
@@ -505,62 +494,8 @@ def test_flatiter():
     assert 1 == next(it)
 
 
-class TestFuncParser(object):
-    x_test = np.linspace(0.01, 0.5, 3)
-    validstrings = ['linear', 'quadratic', 'cubic', 'sqrt', 'cbrt',
-                    'log', 'log10', 'log2', 'x**{1.5}', 'root{2.5}(x)',
-                    'log{2}(x)',
-                    'log(x+{0.5})', 'log10(x+{0.1})', 'log{2}(x+{0.1})',
-                    'log{2}(x+{0})']
-    results = [(lambda x: x),
-               np.square,
-               (lambda x: x**3),
-               np.sqrt,
-               (lambda x: x**(1. / 3)),
-               np.log,
-               np.log10,
-               np.log2,
-               (lambda x: x**1.5),
-               (lambda x: x**(1 / 2.5)),
-               (lambda x: np.log2(x)),
-               (lambda x: np.log(x + 0.5)),
-               (lambda x: np.log10(x + 0.1)),
-               (lambda x: np.log2(x + 0.1)),
-               (lambda x: np.log2(x))]
-
-    bounded_list = [True, True, True, True, True,
-                    False, False, False, True, True,
-                    False,
-                    True, True, True,
-                    False]
-
-    @pytest.mark.parametrize("string, func",
-                             zip(validstrings, results),
-                             ids=validstrings)
-    def test_values(self, string, func):
-        func_parser = cbook._StringFuncParser(string)
-        f = func_parser.function
-        assert_array_almost_equal(f(self.x_test), func(self.x_test))
-
-    @pytest.mark.parametrize("string", validstrings, ids=validstrings)
-    def test_inverse(self, string):
-        func_parser = cbook._StringFuncParser(string)
-        f = func_parser.func_info
-        fdir = f.function
-        finv = f.inverse
-        assert_array_almost_equal(finv(fdir(self.x_test)), self.x_test)
-
-    @pytest.mark.parametrize("string", validstrings, ids=validstrings)
-    def test_get_inverse(self, string):
-        func_parser = cbook._StringFuncParser(string)
-        finv1 = func_parser.inverse
-        finv2 = func_parser.func_info.inverse
-        assert_array_almost_equal(finv1(self.x_test), finv2(self.x_test))
-
-    @pytest.mark.parametrize("string, bounded",
-                             zip(validstrings, bounded_list),
-                             ids=validstrings)
-    def test_bounded(self, string, bounded):
-        func_parser = cbook._StringFuncParser(string)
-        b = func_parser.is_bounded_0_1
-        assert_array_equal(b, bounded)
+def test_safe_first_element_pandas_series(pd):
+    # delibrately create a pandas series with index not starting from 0
+    s = pd.Series(range(5), index=range(10, 15))
+    actual = cbook.safe_first_element(s)
+    assert actual == 0
