@@ -1,15 +1,10 @@
-from __future__ import absolute_import, division, print_function
-
 import copy
-import six
 import itertools
-import warnings
-from distutils.version import LooseVersion as V
 
 import numpy as np
 import pytest
 
-from numpy.testing.utils import assert_array_equal, assert_array_almost_equal
+from numpy.testing import assert_array_equal, assert_array_almost_equal
 
 from matplotlib import cycler
 import matplotlib
@@ -106,7 +101,7 @@ def test_BoundaryNorm():
     expected = [-1, 0, 1, 2]
     for v, ex in zip(vals, expected):
         ret = bn(v)
-        assert isinstance(ret, six.integer_types)
+        assert isinstance(ret, int)
         assert_array_equal(ret, ex)
         assert_array_equal(bn([v]), ex)
 
@@ -115,7 +110,7 @@ def test_BoundaryNorm():
     expected = [-1, 0, 2, 3]
     for v, ex in zip(vals, expected):
         ret = bn(v)
-        assert isinstance(ret, six.integer_types)
+        assert isinstance(ret, int)
         assert_array_equal(ret, ex)
         assert_array_equal(bn([v]), ex)
 
@@ -124,7 +119,7 @@ def test_BoundaryNorm():
     expected = [0, 0, 2, 2]
     for v, ex in zip(vals, expected):
         ret = bn(v)
-        assert isinstance(ret, six.integer_types)
+        assert isinstance(ret, int)
         assert_array_equal(ret, ex)
         assert_array_equal(bn([v]), ex)
 
@@ -191,6 +186,15 @@ def test_PowerNorm():
     assert pnorm(a[-1], clip=True) == expected[-1]
 
 
+def test_PowerNorm_translation_invariance():
+    a = np.array([0, 1/2, 1], dtype=float)
+    expected = [0, 1/8, 1]
+    pnorm = mcolors.PowerNorm(vmin=0, vmax=1, gamma=3)
+    assert_array_almost_equal(pnorm(a), expected)
+    pnorm = mcolors.PowerNorm(vmin=-2, vmax=-1, gamma=3)
+    assert_array_almost_equal(pnorm(a - 2), expected)
+
+
 def test_Normalize():
     norm = mcolors.Normalize()
     vals = np.arange(-10, 10, 1, dtype=float)
@@ -458,17 +462,9 @@ def test_light_source_shading_default():
           [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00]]
         ]).T
 
-    if (V(np.__version__) == V('1.9.0')):
-        # Numpy 1.9.0 uses a 2. order algorithm on the edges by default
-        # This was changed back again in 1.9.1
-        expect = expect[1:-1, 1:-1, :]
-        rgb = rgb[1:-1, 1:-1, :]
-
     assert_array_almost_equal(rgb, expect, decimal=2)
 
 
-@pytest.mark.xfail(V('1.7.0') <= V(np.__version__) <= V('1.9.0'),
-                   reason='NumPy version is not buggy')
 # Numpy 1.9.1 fixed a bug in masked arrays which resulted in
 # additional elements being masked when calculating the gradient thus
 # the output is different with earlier numpy versions.
@@ -538,14 +534,7 @@ def test_light_source_hillshading():
         dy = -dy
         dz = np.ones_like(dy)
         normals = np.dstack([dx, dy, dz])
-        dividers = np.zeros_like(z)[..., None]
-        for i, mat in enumerate(normals):
-            for j, vec in enumerate(mat):
-                dividers[i, j, 0] = np.linalg.norm(vec)
-        normals /= dividers
-        # once we drop support for numpy 1.7.x the above can be written as
-        # normals /= np.linalg.norm(normals, axis=2)[..., None]
-        # aviding the double loop.
+        normals /= np.linalg.norm(normals, axis=2)[..., None]
 
         intensity = np.tensordot(normals, illum, axes=(2, 0))
         intensity -= intensity.min()
@@ -717,13 +706,7 @@ def test_ndarray_subclass_norm(recwarn):
         fig, ax = plt.subplots()
         ax.imshow(mydata, norm=norm)
         fig.canvas.draw()
-        if isinstance(norm, mcolors.PowerNorm):
-            assert len(recwarn) == 1
-            warn = recwarn.pop(UserWarning)
-            assert ('Power-law scaling on negative values is ill-defined'
-                    in str(warn.message))
-        else:
-            assert len(recwarn) == 0
+        assert len(recwarn) == 0
         recwarn.clear()
 
 
