@@ -1,10 +1,9 @@
-from __future__ import absolute_import, division, print_function
-
 import sys
 import warnings
+import platform
 
 from matplotlib import rcParams
-from matplotlib.testing.decorators import image_comparison
+from matplotlib.testing.decorators import image_comparison, check_figures_equal
 from matplotlib.axes import Axes
 from matplotlib.ticker import AutoMinorLocator, FixedFormatter
 import matplotlib.pyplot as plt
@@ -14,7 +13,8 @@ import numpy as np
 import pytest
 
 
-@image_comparison(baseline_images=['figure_align_labels'])
+@image_comparison(baseline_images=['figure_align_labels'],
+                  tol={'aarch64': 0.02}.get(platform.machine(), 0.0))
 def test_align_labels():
     # Check the figure.align_labels() command
     fig = plt.figure(tight_layout=True)
@@ -215,7 +215,7 @@ def test_iterability_axes_argument():
 
     # This is a regression test for matplotlib/matplotlib#3196. If one of the
     # arguments returned by _as_mpl_axes defines __getitem__ but is not
-    # iterable, this would raise an execption. This is because we check
+    # iterable, this would raise an exception. This is because we check
     # whether the arguments are iterable, and if so we try and convert them
     # to a tuple. However, the ``iterable`` function returns True if
     # __getitem__ is present, but some classes can define __getitem__ without
@@ -223,8 +223,7 @@ def test_iterability_axes_argument():
     # case it fails.
 
     class MyAxes(Axes):
-        def __init__(self, *args, **kwargs):
-            kwargs.pop('myclass', None)
+        def __init__(self, *args, myclass=None, **kwargs):
             return Axes.__init__(self, *args, **kwargs)
 
     class MyClass(object):
@@ -366,11 +365,8 @@ def test_subplots_shareax_loglabels():
 
 def test_savefig():
     fig = plt.figure()
-    if sys.version_info[0] == 3:
-        msg = r"savefig\(\) takes 2 positional arguments but 3 were given"
-    else:
-        msg = r"savefig\(\) takes exactly 2 arguments \(3 given\)"
-    with pytest.raises(TypeError, match=msg):
+    msg = "savefig() takes 2 positional arguments but 3 were given"
+    with pytest.raises(TypeError, message=msg):
         fig.savefig("fname1.png", "fname2.png")
 
 
@@ -387,6 +383,35 @@ def test_warn_cl_plus_tl():
     assert not(fig.get_constrained_layout())
 
 
+@check_figures_equal(extensions=["png", "pdf"])
+def test_add_artist(fig_test, fig_ref):
+    fig_test.set_dpi(100)
+    fig_ref.set_dpi(100)
+
+    ax = fig_test.subplots()
+    l1 = plt.Line2D([.2, .7], [.7, .7], gid='l1')
+    l2 = plt.Line2D([.2, .7], [.8, .8], gid='l2')
+    r1 = plt.Circle((20, 20), 100, transform=None, gid='C1')
+    r2 = plt.Circle((.7, .5), .05, gid='C2')
+    r3 = plt.Circle((4.5, .8), .55, transform=fig_test.dpi_scale_trans,
+                    facecolor='crimson', gid='C3')
+    for a in [l1, l2, r1, r2, r3]:
+        fig_test.add_artist(a)
+    l2.remove()
+
+    ax2 = fig_ref.subplots()
+    l1 = plt.Line2D([.2, .7], [.7, .7], transform=fig_ref.transFigure,
+                    gid='l1', zorder=21)
+    r1 = plt.Circle((20, 20), 100, transform=None, clip_on=False, zorder=20,
+                    gid='C1')
+    r2 = plt.Circle((.7, .5), .05, transform=fig_ref.transFigure, gid='C2',
+                    zorder=20)
+    r3 = plt.Circle((4.5, .8), .55, transform=fig_ref.dpi_scale_trans,
+                    facecolor='crimson', clip_on=False, zorder=20, gid='C3')
+    for a in [l1, r1, r2, r3]:
+        ax2.add_artist(a)
+
+
 @pytest.mark.skipif(sys.version_info < (3, 6), reason="requires Python 3.6+")
 @pytest.mark.parametrize("fmt", ["png", "pdf", "ps", "eps", "svg"])
 def test_fspath(fmt, tmpdir):
@@ -397,3 +422,28 @@ def test_fspath(fmt, tmpdir):
         # All the supported formats include the format name (case-insensitive)
         # in the first 100 bytes.
         assert fmt.encode("ascii") in file.read(100).lower()
+
+
+def test_tightbbox():
+    fig, ax = plt.subplots()
+    ax.set_xlim(0, 1)
+    t = ax.text(1., 0.5, 'This dangles over end')
+    renderer = fig.canvas.get_renderer()
+    x1Nom0 = 9.035  # inches
+    assert np.abs(t.get_tightbbox(renderer).x1 - x1Nom0 * fig.dpi) < 2
+    assert np.abs(ax.get_tightbbox(renderer).x1 - x1Nom0 * fig.dpi) < 2
+    assert np.abs(fig.get_tightbbox(renderer).x1 - x1Nom0) < 0.05
+    assert np.abs(fig.get_tightbbox(renderer).x0 - 0.679) < 0.05
+    # now exclude t from the tight bbox so now the bbox is quite a bit
+    # smaller
+    t.set_in_layout(False)
+    x1Nom = 7.333
+    assert np.abs(ax.get_tightbbox(renderer).x1 - x1Nom * fig.dpi) < 2
+    assert np.abs(fig.get_tightbbox(renderer).x1 - x1Nom) < 0.05
+
+    t.set_in_layout(True)
+    x1Nom = 7.333
+    assert np.abs(ax.get_tightbbox(renderer).x1 - x1Nom0 * fig.dpi) < 2
+    # test bbox_extra_artists method...
+    assert np.abs(ax.get_tightbbox(renderer,
+                        bbox_extra_artists=[]).x1 - x1Nom * fig.dpi) < 2
