@@ -1,11 +1,10 @@
-from __future__ import absolute_import, division, print_function
-
-import six
-
+from contextlib import ExitStack
 from copy import copy
 import io
 import os
 import sys
+import platform
+import urllib.request
 import warnings
 
 import numpy as np
@@ -23,14 +22,6 @@ from matplotlib.transforms import Bbox, Affine2D, TransformedBbox
 import pytest
 
 
-try:
-    from PIL import Image
-    HAS_PIL = True
-except ImportError:
-    HAS_PIL = False
-needs_pillow = pytest.mark.xfail(not HAS_PIL, reason='Test requires Pillow')
-
-
 @image_comparison(baseline_images=['image_interps'], style='mpl20')
 def test_image_interps():
     'make the basic nearest, bilinear and bicubic interps'
@@ -113,17 +104,17 @@ def test_image_python_io():
     plt.imread(buffer)
 
 
-@needs_pillow
 def test_imread_pil_uint16():
+    pytest.importorskip("PIL")
     img = plt.imread(os.path.join(os.path.dirname(__file__),
                      'baseline_images', 'test_image', 'uint16.tif'))
-    assert (img.dtype == np.uint16)
+    assert img.dtype == np.uint16
     assert np.sum(img) == 134184960
 
 
 @pytest.mark.skipif(sys.version_info < (3, 6), reason="requires Python 3.6+")
-@needs_pillow
 def test_imread_fspath():
+    pytest.importorskip("PIL")
     from pathlib import Path
     img = plt.imread(
         Path(__file__).parent / 'baseline_images/test_image/uint16.tif')
@@ -208,6 +199,7 @@ def test_image_alpha():
     plt.subplot(133)
     plt.imshow(Z, alpha=0.5, interpolation='nearest')
 
+
 def test_cursor_data():
     from matplotlib.backend_bases import MouseEvent
 
@@ -289,7 +281,8 @@ def test_image_cliprect():
 
     im = ax.imshow(d, extent=(0,5,0,5))
 
-    rect = patches.Rectangle(xy=(1,1), width=2, height=2, transform=im.axes.transData)
+    rect = patches.Rectangle(
+        xy=(1,1), width=2, height=2, transform=im.axes.transData)
     im.set_clip_path(rect)
 
 
@@ -305,13 +298,10 @@ def test_imshow():
 @image_comparison(baseline_images=['no_interpolation_origin'],
                   remove_text=True)
 def test_no_interpolation_origin():
-    fig = plt.figure()
-    ax = fig.add_subplot(211)
-    ax.imshow(np.arange(100).reshape((2, 50)), origin="lower",
-              interpolation='none')
-
-    ax = fig.add_subplot(212)
-    ax.imshow(np.arange(100).reshape((2, 50)), interpolation='none')
+    fig, axs = plt.subplots(2)
+    axs[0].imshow(np.arange(100).reshape((2, 50)), origin="lower",
+                  interpolation='none')
+    axs[1].imshow(np.arange(100).reshape((2, 50)), interpolation='none')
 
 
 @image_comparison(baseline_images=['image_shift'], remove_text=True,
@@ -319,9 +309,9 @@ def test_no_interpolation_origin():
 def test_image_shift():
     from matplotlib.colors import LogNorm
 
-    imgData = [[1.0/(x) + 1.0/(y) for x in range(1,100)] for y in range(1,100)]
-    tMin=734717.945208
-    tMax=734717.946366
+    imgData = [[1 / x + 1 / y for x in range(1, 100)] for y in range(1, 100)]
+    tMin = 734717.945208
+    tMax = 734717.946366
 
     fig, ax = plt.subplots()
     ax.imshow(imgData, norm=LogNorm(), interpolation='none',
@@ -379,11 +369,13 @@ def test_image_composite_alpha():
     fig, ax = plt.subplots()
     arr = np.zeros((11, 21, 4))
     arr[:, :, 0] = 1
-    arr[:, :, 3] = np.concatenate((np.arange(0, 1.1, 0.1), np.arange(0, 1, 0.1)[::-1]))
+    arr[:, :, 3] = np.concatenate(
+        (np.arange(0, 1.1, 0.1), np.arange(0, 1, 0.1)[::-1]))
     arr2 = np.zeros((21, 11, 4))
     arr2[:, :, 0] = 1
     arr2[:, :, 1] = 1
-    arr2[:, :, 3] = np.concatenate((np.arange(0, 1.1, 0.1), np.arange(0, 1, 0.1)[::-1]))[:, np.newaxis]
+    arr2[:, :, 3] = np.concatenate(
+        (np.arange(0, 1.1, 0.1), np.arange(0, 1, 0.1)[::-1]))[:, np.newaxis]
     ax.imshow(arr, extent=[1, 2, 5, 0], alpha=0.3)
     ax.imshow(arr, extent=[2, 3, 5, 0], alpha=0.6)
     ax.imshow(arr, extent=[3, 4, 5, 0])
@@ -400,21 +392,22 @@ def test_image_composite_alpha():
                   remove_text=True, style='mpl20')
 def test_rasterize_dpi():
     # This test should check rasterized rendering with high output resolution.
-    # It plots a rasterized line and a normal image with implot. So it will catch
-    # when images end up in the wrong place in case of non-standard dpi setting.
-    # Instead of high-res rasterization i use low-res.  Therefore the fact that the
-    # resolution is non-standard is easily checked by image_comparison.
+    # It plots a rasterized line and a normal image with implot.  So it will
+    # catch when images end up in the wrong place in case of non-standard dpi
+    # setting.  Instead of high-res rasterization I use low-res.  Therefore
+    # the fact that the resolution is non-standard is easily checked by
+    # image_comparison.
     img = np.asarray([[1, 2], [3, 4]])
 
-    fig, axes = plt.subplots(1, 3, figsize = (3, 1))
+    fig, axes = plt.subplots(1, 3, figsize=(3, 1))
 
     axes[0].imshow(img)
 
-    axes[1].plot([0,1],[0,1], linewidth=20., rasterized=True)
-    axes[1].set(xlim = (0,1), ylim = (-1, 2))
+    axes[1].plot([0,1], [0,1], linewidth=20., rasterized=True)
+    axes[1].set(xlim=(0, 1), ylim=(-1, 2))
 
-    axes[2].plot([0,1],[0,1], linewidth=20.)
-    axes[2].set(xlim = (0,1), ylim = (-1, 2))
+    axes[2].plot([0,1], [0,1], linewidth=20.)
+    axes[2].set(xlim=(0, 1), ylim=(-1, 2))
 
     # Low-dpi PDF rasterization errors prevent proper image comparison tests.
     # Hide detailed structures like the axes spines.
@@ -444,8 +437,8 @@ def test_bbox_image_inverted():
 
     image = np.identity(10)
 
-    bbox_im = BboxImage(
-        TransformedBbox(Bbox([[0.1, 0.2], [0.3, 0.25]]), ax.figure.transFigure))
+    bbox_im = BboxImage(TransformedBbox(Bbox([[0.1, 0.2], [0.3, 0.25]]),
+                                        ax.figure.transFigure))
     bbox_im.set_data(image)
     bbox_im.set_clip_on(False)
     ax.add_artist(bbox_im)
@@ -462,7 +455,8 @@ def test_get_window_extent_for_AxisImage():
     ax.set_position([0, 0, 1, 1])
     ax.set_xlim(0, 1)
     ax.set_ylim(0, 1)
-    im_obj = ax.imshow(im, extent=[0.4, 0.7, 0.2, 0.9], interpolation='nearest')
+    im_obj = ax.imshow(
+        im, extent=[0.4, 0.7, 0.2, 0.9], interpolation='nearest')
 
     fig.canvas.draw()
     renderer = fig.canvas.renderer
@@ -497,8 +491,8 @@ def test_nonuniformimage_setnorm():
     im.set_norm(plt.Normalize())
 
 
-@needs_pillow
 def test_jpeg_2d():
+    Image = pytest.importorskip('PIL.Image')
     # smoke test that mode-L pillow images work.
     imd = np.ones((10, 10), dtype='uint8')
     for i in range(10):
@@ -509,8 +503,9 @@ def test_jpeg_2d():
     ax.imshow(im)
 
 
-@needs_pillow
 def test_jpeg_alpha():
+    Image = pytest.importorskip('PIL.Image')
+
     plt.figure(figsize=(1, 1), dpi=300)
     # Create an image that is all black, with a gradient from 0-1 in
     # the alpha channel from left to right.
@@ -575,12 +570,6 @@ def test_pcolorimage_setdata():
     assert im._A[0, 0] == im._Ax[0] == im._Ay[0] == 0, 'value changed'
 
 
-def test_pcolorimage_extent():
-    im = plt.hist2d([1, 2, 3], [3, 5, 6],
-                    bins=[[0, 3, 7], [1, 2, 3]])[-1]
-    assert im.get_extent() == (0, 7, 1, 3)
-
-
 def test_minimized_rasterized():
     # This ensures that the rasterized content in the colorbars is
     # only as thick as the colorbar, and doesn't extend to other parts
@@ -616,9 +605,9 @@ def test_minimized_rasterized():
 
 @pytest.mark.network
 def test_load_from_url():
-    req = six.moves.urllib.request.urlopen(
-        "http://matplotlib.org/_static/logo_sidebar_horiz.png")
-    plt.imread(req)
+    url = "http://matplotlib.org/_static/logo_sidebar_horiz.png"
+    plt.imread(url)
+    plt.imread(urllib.request.urlopen(url))
 
 
 @image_comparison(baseline_images=['log_scale_image'],
@@ -768,6 +757,7 @@ def test_imshow_endianess():
 
 
 @image_comparison(baseline_images=['imshow_masked_interpolation'],
+                  tol={'aarch64': 0.02}.get(platform.machine(), 0.0),
                   remove_text=True, style='mpl20')
 def test_imshow_masked_interpolation():
 
@@ -779,7 +769,6 @@ def test_imshow_masked_interpolation():
     N = 20
     n = colors.Normalize(vmin=0, vmax=N*N-1)
 
-    # data = np.random.random((N, N))*N*N
     data = np.arange(N*N, dtype='float').reshape(N, N)
 
     data[5, 5] = -1
@@ -887,6 +876,10 @@ def test_empty_imshow(make_norm):
 def test_imshow_float128():
     fig, ax = plt.subplots()
     ax.imshow(np.zeros((3, 3), dtype=np.longdouble))
+    with (ExitStack() if np.can_cast(np.longdouble, np.float64, "equiv")
+          else pytest.warns(UserWarning)):
+        # Ensure that drawing doesn't cause crash.
+        fig.canvas.draw()
 
 
 def test_imshow_bool():
@@ -894,14 +887,6 @@ def test_imshow_bool():
     ax.imshow(np.array([[True, False], [False, True]], dtype=bool))
 
 
-def test_imshow_deprecated_interd_warn():
-    im = plt.imshow([[1, 2], [3, np.nan]])
-    for k in ('_interpd', '_interpdr', 'iterpnames'):
-        with warnings.catch_warnings(record=True) as warns:
-            getattr(im, k)
-        assert len(warns) == 1
-
-
 def test_full_invalid():
     x = np.ones((10, 10))
     x[:] = np.nan
