@@ -2,19 +2,18 @@
 Tests specific to the lines module.
 """
 
-from __future__ import absolute_import, division, print_function
-
 from io import BytesIO
 import itertools
-import matplotlib.lines as mlines
-import pytest
-from timeit import repeat
-import numpy as np
+import timeit
+
 from cycler import cycler
+import numpy as np
+import pytest
 
 import matplotlib
+import matplotlib.lines as mlines
 import matplotlib.pyplot as plt
-from matplotlib.testing.decorators import image_comparison
+from matplotlib.testing.decorators import image_comparison, check_figures_equal
 
 
 # Runtimes on a loaded system are inherently flaky. Not so much that a rerun
@@ -33,7 +32,7 @@ def test_invisible_Line_rendering():
     """
     # Creates big x and y data:
     N = 10**7
-    x = np.linspace(0,1,N)
+    x = np.linspace(0, 1, N)
     y = np.random.normal(size=N)
 
     # Create a plot figure:
@@ -41,13 +40,13 @@ def test_invisible_Line_rendering():
     ax = plt.subplot(111)
 
     # Create a "big" Line instance:
-    l = mlines.Line2D(x,y)
+    l = mlines.Line2D(x, y)
     l.set_visible(False)
     # but don't add it to the Axis instance `ax`
 
     # [here Interactive panning and zooming is pretty responsive]
     # Time the canvas drawing:
-    t_no_line = min(repeat(fig.canvas.draw, number=1, repeat=3))
+    t_no_line = min(timeit.repeat(fig.canvas.draw, number=1, repeat=3))
     # (gives about 25 ms)
 
     # Add the big invisible Line:
@@ -55,11 +54,11 @@ def test_invisible_Line_rendering():
 
     # [Now interactive panning and zooming is very slow]
     # Time the canvas drawing:
-    t_unvisible_line = min(repeat(fig.canvas.draw, number=1, repeat=3))
+    t_unvisible_line = min(timeit.repeat(fig.canvas.draw, number=1, repeat=3))
     # gives about 290 ms for N = 10**7 pts
 
     slowdown_factor = (t_unvisible_line/t_no_line)
-    slowdown_threshold = 2 # trying to avoid false positive failures
+    slowdown_threshold = 2  # trying to avoid false positive failures
     assert slowdown_factor < slowdown_threshold
 
 
@@ -145,7 +144,8 @@ def test_set_drawstyle():
     assert len(line.get_path().vertices) == len(x)
 
 
-@image_comparison(baseline_images=['line_collection_dashes'], remove_text=True)
+@image_comparison(baseline_images=['line_collection_dashes'],
+                  remove_text=True, style='mpl20')
 def test_set_line_coll_dash_image():
     fig = plt.figure()
     ax = fig.add_subplot(1, 1, 1)
@@ -201,13 +201,7 @@ def test_nan_is_sorted():
     assert not line._is_sorted([3, 5] + [np.nan] * 100 + [0, 2])
 
 
-def test_step_markers():
-    fig, ax = plt.subplots()
-    ax.step([0, 1], "-o")
-    buf1 = BytesIO()
-    fig.savefig(buf1)
-    fig, ax = plt.subplots()
-    ax.plot([0, 0, 1], [0, 1, 1], "-o", markevery=[0, 2])
-    buf2 = BytesIO()
-    fig.savefig(buf2)
-    assert buf1.getvalue() == buf2.getvalue()
+@check_figures_equal()
+def test_step_markers(fig_test, fig_ref):
+    fig_test.subplots().step([0, 1], "-o")
+    fig_ref.subplots().plot([0, 0, 1], [0, 1, 1], "-o", markevery=[0, 2])
