@@ -1,8 +1,3 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-
 import tempfile
 import warnings
 
@@ -15,18 +10,21 @@ import pytest
 
 import matplotlib.mlab as mlab
 import matplotlib.cbook as cbook
+from matplotlib.cbook.deprecation import MatplotlibDeprecationWarning
 
 
-try:
-    from mpl_toolkits.natgrid import _natgrid
-    HAS_NATGRID = True
-except ImportError:
-    HAS_NATGRID = False
+'''
+A lot of mlab.py has been deprecated in Matplotlib 2.2 and is scheduled for
+removal in the future. The tests that use deprecated methods have a block
+to catch the deprecation warning, and can be removed with the mlab code is
+removed.
+'''
 
 
 def test_colinear_pca():
-    a = mlab.PCA._get_colinear()
-    pca = mlab.PCA(a)
+    with pytest.warns(MatplotlibDeprecationWarning):
+        a = mlab.PCA._get_colinear()
+        pca = mlab.PCA(a)
 
     assert_allclose(pca.fracs[2:], 0., atol=1e-8)
     assert_allclose(pca.Y[:, 2:], 0., atol=1e-8)
@@ -53,17 +51,9 @@ ids=[
     [0, 75, 100],
 ])
 def test_prctile(input, percentile):
-    assert_allclose(mlab.prctile(input, percentile),
-                    np.percentile(input, percentile))
-
-
-def test_norm():
-    np.random.seed(0)
-    N = 1000
-    x = np.random.standard_normal(N)
-    targ = np.linalg.norm(x)
-    res = mlab._norm(x)
-    assert_almost_equal(targ, res)
+    with pytest.warns(MatplotlibDeprecationWarning):
+        assert_allclose(mlab.prctile(input, percentile),
+                        np.percentile(input, percentile))
 
 
 @pytest.mark.parametrize('xmin, xmax, N', [
@@ -78,7 +68,8 @@ def test_norm():
     'single',
 ])
 def test_logspace(xmin, xmax, N):
-    res = mlab.logspace(xmin, xmax, N)
+    with pytest.warns(MatplotlibDeprecationWarning):
+        res = mlab.logspace(xmin, xmax, N)
     targ = np.logspace(np.log10(xmin), np.log10(xmax), N)
     assert_allclose(targ, res)
     assert res.size == N
@@ -210,28 +201,21 @@ class TestStride(object):
 
 @pytest.fixture
 def tempcsv():
-    if six.PY2:
-        fd = tempfile.TemporaryFile(suffix='csv', mode="wb+")
-    else:
-        fd = tempfile.TemporaryFile(suffix='csv', mode="w+", newline='')
-    with fd:
+    with tempfile.TemporaryFile(suffix='csv', mode="w+", newline='') as fd:
         yield fd
 
 
 def test_recarray_csv_roundtrip(tempcsv):
-    expected = np.recarray((99,),
-                           [(str('x'), float),
-                            (str('y'), float),
-                            (str('t'), float)])
+    expected = np.recarray((99,), [('x', float), ('y', float), ('t', float)])
     # initialising all values: uninitialised memory sometimes produces
     # floats that do not round-trip to string and back.
     expected['x'][:] = np.linspace(-1e9, -1, 99)
     expected['y'][:] = np.linspace(1, 1e9, 99)
     expected['t'][:] = np.linspace(0, 0.01, 99)
-
-    mlab.rec2csv(expected, tempcsv)
-    tempcsv.seek(0)
-    actual = mlab.csv2rec(tempcsv)
+    with pytest.warns(MatplotlibDeprecationWarning):
+        mlab.rec2csv(expected, tempcsv)
+        tempcsv.seek(0)
+        actual = mlab.csv2rec(tempcsv)
 
     assert_allclose(expected['x'], actual['x'])
     assert_allclose(expected['y'], actual['y'])
@@ -239,18 +223,20 @@ def test_recarray_csv_roundtrip(tempcsv):
 
 
 def test_rec2csv_bad_shape_ValueError(tempcsv):
-    bad = np.recarray((99, 4), [(str('x'), float),
-                                (str('y'), float)])
+    bad = np.recarray((99, 4), [('x', float), ('y', float)])
 
     # the bad recarray should trigger a ValueError for having ndim > 1.
-    with pytest.raises(ValueError):
-        mlab.rec2csv(bad, tempcsv)
+    with pytest.warns(MatplotlibDeprecationWarning):
+        with pytest.raises(ValueError):
+            mlab.rec2csv(bad, tempcsv)
 
 
 def test_csv2rec_names_with_comments(tempcsv):
+
     tempcsv.write('# comment\n1,2,3\n4,5,6\n')
     tempcsv.seek(0)
-    array = mlab.csv2rec(tempcsv, names='a,b,c')
+    with pytest.warns(MatplotlibDeprecationWarning):
+        array = mlab.csv2rec(tempcsv, names='a,b,c')
     assert len(array) == 2
     assert len(array.dtype) == 3
 
@@ -283,23 +269,23 @@ def test_csv2rec_dates(tempcsv, input, kwargs):
                 datetime.datetime(2054, 6, 20, 14, 31, 45),
                 datetime.datetime(2000, 10, 31, 11, 50, 23)]
     tempcsv.seek(0)
-    array = mlab.csv2rec(tempcsv, names='a', **kwargs)
+    with pytest.warns(MatplotlibDeprecationWarning):
+        array = mlab.csv2rec(tempcsv, names='a', **kwargs)
     assert_array_equal(array['a'].tolist(), expected)
 
 
 def test_rec2txt_basic():
-    # str() calls around field names necessary b/c as of numpy 1.11
-    # dtype doesn't like unicode names (caused by unicode_literals import)
     a = np.array([(1.0, 2, 'foo', 'bing'),
                   (2.0, 3, 'bar', 'blah')],
-                 dtype=np.dtype([(str('x'), np.float32),
-                                 (str('y'), np.int8),
-                                 (str('s'), str, 3),
-                                 (str('s2'), str, 4)]))
+                 dtype=np.dtype([('x', np.float32),
+                                 ('y', np.int8),
+                                 ('s', str, 3),
+                                 ('s2', str, 4)]))
     truth = ('       x   y   s     s2\n'
              '   1.000   2   foo   bing   \n'
              '   2.000   3   bar   blah   ').splitlines()
-    assert mlab.rec2txt(a).splitlines() == truth
+    with pytest.warns(MatplotlibDeprecationWarning):
+        assert mlab.rec2txt(a).splitlines() == truth
 
 
 class TestWindow(object):
@@ -1120,7 +1106,7 @@ class TestDetrend(object):
         res = mlab.detrend(input, key=mlab.detrend_linear, axis=0)
         assert_allclose(res, targ, atol=self.atol)
 
-    def test_detrend_str_linear_2d_slope_off_axis0(self):
+    def test_detrend_str_linear_2d_slope_off_axis0_notranspose(self):
         arri = [self.sig_off,
                 self.sig_slope,
                 self.sig_slope + self.sig_off]
@@ -1132,7 +1118,7 @@ class TestDetrend(object):
         res = mlab.detrend(input, key='linear', axis=1)
         assert_allclose(res, targ, atol=self.atol)
 
-    def test_detrend_detrend_linear_1d_slope_off_axis1(self):
+    def test_detrend_detrend_linear_1d_slope_off_axis1_notranspose(self):
         arri = [self.sig_off,
                 self.sig_slope,
                 self.sig_slope + self.sig_off]
@@ -2161,25 +2147,29 @@ def test_griddata_linear():
     z = get_z(x, y)
     xi = [0.2, 0.4, 0.6, 0.8]
     yi = [0.1, 0.3, 0.7, 0.9]
-    zi = mlab.griddata(x, y, z, xi, yi, interp='linear')
+    with pytest.warns(MatplotlibDeprecationWarning):
+        zi = mlab.griddata(x, y, z, xi, yi, interp='linear')
     xi, yi = np.meshgrid(xi, yi)
     np.testing.assert_array_almost_equal(zi, get_z(xi, yi))
 
     # Passing 2D xi and yi arrays to griddata.
-    zi = mlab.griddata(x, y, z, xi, yi, interp='linear')
+    with pytest.warns(MatplotlibDeprecationWarning):
+        zi = mlab.griddata(x, y, z, xi, yi, interp='linear')
     np.testing.assert_array_almost_equal(zi, get_z(xi, yi))
 
     # Masking z array.
     z_masked = np.ma.array(z, mask=[False, False, False, True, False])
     correct_zi_masked = np.ma.masked_where(xi + yi > 1.0, get_z(xi, yi))
-    zi = mlab.griddata(x, y, z_masked, xi, yi, interp='linear')
+    with pytest.warns(MatplotlibDeprecationWarning):
+        zi = mlab.griddata(x, y, z_masked, xi, yi, interp='linear')
     matest.assert_array_almost_equal(zi, correct_zi_masked)
     np.testing.assert_array_equal(np.ma.getmask(zi),
                                   np.ma.getmask(correct_zi_masked))
 
 
-@pytest.mark.xfail(not HAS_NATGRID, reason='natgrid not installed')
 def test_griddata_nn():
+    pytest.importorskip('mpl_toolkits.natgrid')
+
     # z is a linear function of x and y.
     def get_z(x, y):
         return 3.0*x - y
@@ -2194,24 +2184,28 @@ def test_griddata_nn():
                   [0.29999208, 0.8999978, 1.5000029, 2.1000059],
                   [-0.1000099, 0.4999943, 1.0999964, 1.6999979],
                   [-0.3000128, 0.2999894, 0.8999913, 1.4999933]]
-    zi = mlab.griddata(x, y, z, xi, yi, interp='nn')
+    with pytest.warns(MatplotlibDeprecationWarning):
+        zi = mlab.griddata(x, y, z, xi, yi, interp='nn')
     np.testing.assert_array_almost_equal(zi, correct_zi, 5)
 
-    # Decreasing xi or yi should raise ValueError.
-    with pytest.raises(ValueError):
-        mlab.griddata(x, y, z, xi[::-1], yi, interp='nn')
-    with pytest.raises(ValueError):
-        mlab.griddata(x, y, z, xi, yi[::-1], interp='nn')
+    with pytest.warns(MatplotlibDeprecationWarning):
+        # Decreasing xi or yi should raise ValueError.
+        with pytest.raises(ValueError):
+            mlab.griddata(x, y, z, xi[::-1], yi, interp='nn')
+        with pytest.raises(ValueError):
+            mlab.griddata(x, y, z, xi, yi[::-1], interp='nn')
 
     # Passing 2D xi and yi arrays to griddata.
     xi, yi = np.meshgrid(xi, yi)
-    zi = mlab.griddata(x, y, z, xi, yi, interp='nn')
+    with pytest.warns(MatplotlibDeprecationWarning):
+        zi = mlab.griddata(x, y, z, xi, yi, interp='nn')
     np.testing.assert_array_almost_equal(zi, correct_zi, 5)
 
     # Masking z array.
     z_masked = np.ma.array(z, mask=[False, False, False, True, False])
     correct_zi_masked = np.ma.masked_where(xi + yi > 1.0, correct_zi)
-    zi = mlab.griddata(x, y, z_masked, xi, yi, interp='nn')
+    with pytest.warns(MatplotlibDeprecationWarning):
+        zi = mlab.griddata(x, y, z_masked, xi, yi, interp='nn')
     np.testing.assert_array_almost_equal(zi, correct_zi_masked, 5)
     np.testing.assert_array_equal(np.ma.getmask(zi),
                                   np.ma.getmask(correct_zi_masked))
@@ -2219,7 +2213,7 @@ def test_griddata_nn():
 
 #*****************************************************************
 # These Tests where taken from SCIPY with some minor modifications
-# this can be retreived from:
+# this can be retrieved from:
 # https://github.com/scipy/scipy/blob/master/scipy/stats/tests/test_kdeoth.py
 #*****************************************************************
 
@@ -2415,28 +2409,32 @@ def test_contiguous_regions():
     # Starts and ends with True
     mask = [True]*a + [False]*b + [True]*c
     expected = [(0, a), (a+b, a+b+c)]
-    assert mlab.contiguous_regions(mask) == expected
+    with pytest.warns(MatplotlibDeprecationWarning):
+        assert mlab.contiguous_regions(mask) == expected
     d, e = 6, 7
     # Starts with True ends with False
     mask = mask + [False]*e
-    assert mlab.contiguous_regions(mask) == expected
+    with pytest.warns(MatplotlibDeprecationWarning):
+        assert mlab.contiguous_regions(mask) == expected
     # Starts with False ends with True
     mask = [False]*d + mask[:-e]
     expected = [(d, d+a), (d+a+b, d+a+b+c)]
-    assert mlab.contiguous_regions(mask) == expected
+    with pytest.warns(MatplotlibDeprecationWarning):
+        assert mlab.contiguous_regions(mask) == expected
     # Starts and ends with False
     mask = mask + [False]*e
-    assert mlab.contiguous_regions(mask) == expected
-    # No True in mask
-    assert mlab.contiguous_regions([False]*5) == []
-    # Empty mask
-    assert mlab.contiguous_regions([]) == []
+    with pytest.warns(MatplotlibDeprecationWarning):
+        assert mlab.contiguous_regions(mask) == expected
+        # No True in mask
+        assert mlab.contiguous_regions([False]*5) == []
+        # Empty mask
+        assert mlab.contiguous_regions([]) == []
 
 
 def test_psd_onesided_norm():
     u = np.array([0, 1, 2, 3, 1, 2, 1])
     dt = 1.0
-    Su = np.abs(np.fft.fft(u) * dt)**2 / float(dt * u.size)
+    Su = np.abs(np.fft.fft(u) * dt)**2 / (dt * u.size)
     P, f = mlab.psd(u, NFFT=u.size, Fs=1/dt, window=mlab.window_none,
                     detrend=mlab.detrend_none, noverlap=0, pad_to=None,
                     scale_by_freq=None,
@@ -2446,10 +2444,10 @@ def test_psd_onesided_norm():
 
 
 def test_psd_oversampling():
-    """Test the case len(x) < NFFT for psd(). """
+    """Test the case len(x) < NFFT for psd()."""
     u = np.array([0, 1, 2, 3, 1, 2, 1])
     dt = 1.0
-    Su = np.abs(np.fft.fft(u) * dt)**2 / float(dt * u.size)
+    Su = np.abs(np.fft.fft(u) * dt)**2 / (dt * u.size)
     P, f = mlab.psd(u, NFFT=u.size*2, Fs=1/dt, window=mlab.window_none,
                     detrend=mlab.detrend_none, noverlap=0, pad_to=None,
                     scale_by_freq=None,
