@@ -1,23 +1,17 @@
 """
 Tests specific to the patches module.
 """
-from __future__ import absolute_import, division, print_function
-
-import six
-
 import numpy as np
 from numpy.testing import assert_almost_equal, assert_array_equal
 import pytest
 
-from matplotlib.patches import Polygon
-from matplotlib.patches import Rectangle
-from matplotlib.testing.decorators import image_comparison
+from matplotlib.cbook import MatplotlibDeprecationWarning
+from matplotlib.patches import Polygon, Rectangle
+from matplotlib.testing.decorators import image_comparison, check_figures_equal
 import matplotlib.pyplot as plt
-import matplotlib.patches as mpatches
-import matplotlib.collections as mcollections
-from matplotlib import path as mpath
-from matplotlib import transforms as mtransforms
-import matplotlib.style as mstyle
+from matplotlib import (
+    collections as mcollections, colors as mcolors, patches as mpatches,
+    path as mpath, style as mstyle, transforms as mtransforms)
 
 import sys
 on_win = (sys.platform == 'win32')
@@ -257,10 +251,10 @@ def test_wedge_movement():
                   'theta1': (0, 30, 'set_theta1'),
                   'theta2': (45, 50, 'set_theta2')}
 
-    init_args = dict((k, v[0]) for (k, v) in six.iteritems(param_dict))
+    init_args = {k: v[0] for k, v in param_dict.items()}
 
     w = mpatches.Wedge(**init_args)
-    for attr, (old_v, new_v, func) in six.iteritems(param_dict):
+    for attr, (old_v, new_v, func) in param_dict.items():
         assert getattr(w, attr) == old_v
         getattr(w, func)(new_v)
         assert getattr(w, attr) == new_v
@@ -320,6 +314,44 @@ def test_patch_str():
     expected = 'Arc(xy=(1, 2), width=3, height=4, angle=5, theta1=6, theta2=7)'
     assert str(p) == expected
 
+    p = mpatches.RegularPolygon((1, 2), 20, radius=5)
+    assert str(p) == "RegularPolygon((1, 2), 20, radius=5, orientation=0)"
+
+    p = mpatches.CirclePolygon(xy=(1, 2), radius=5, resolution=20)
+    assert str(p) == "CirclePolygon((1, 2), radius=5, resolution=20)"
+
+    p = mpatches.FancyBboxPatch((1, 2), width=3, height=4)
+    assert str(p) == "FancyBboxPatch((1, 2), width=3, height=4)"
+
+    # Further nice __str__ which cannot be `eval`uated:
+    path_data = [([1, 2], mpath.Path.MOVETO), ([2, 2], mpath.Path.LINETO),
+                 ([1, 2], mpath.Path.CLOSEPOLY)]
+    p = mpatches.PathPatch(mpath.Path(*zip(*path_data)))
+    assert str(p) == "PathPatch3((1, 2) ...)"
+
+    data = [[1, 2], [2, 2], [1, 2]]
+    p = mpatches.Polygon(data)
+    assert str(p) == "Polygon3((1, 2) ...)"
+
+    p = mpatches.FancyArrowPatch(path=mpath.Path(*zip(*path_data)))
+    assert str(p)[:27] == "FancyArrowPatch(Path(array("
+
+    p = mpatches.FancyArrowPatch((1, 2), (3, 4))
+    assert str(p) == "FancyArrowPatch((1, 2)->(3, 4))"
+
+    p = mpatches.ConnectionPatch((1, 2), (3, 4), 'data')
+    assert str(p) == "ConnectionPatch((1, 2), (3, 4))"
+
+    s = mpatches.Shadow(p, 1, 1)
+    assert str(s) == "Shadow(ConnectionPatch((1, 2), (3, 4)))"
+
+    with pytest.warns(MatplotlibDeprecationWarning):
+        p = mpatches.YAArrow(plt.gcf(), (1, 0), (2, 1), width=0.1)
+        assert str(p) == "YAArrow()"
+
+    # Not testing Arrow, FancyArrow here
+    # because they seem to exist only for historical reasons.
+
 
 @image_comparison(baseline_images=['multi_color_hatch'],
                   remove_text=True, style='default')
@@ -410,3 +442,29 @@ def test_contains_points():
     expected = path.contains_points(points, transform, radius)
     result = ell.contains_points(points)
     assert np.all(result == expected)
+
+
+# Currently fails with pdf/svg, probably because some parts assume a dpi of 72.
+@check_figures_equal(extensions=["png"])
+def test_shadow(fig_test, fig_ref):
+    xy = np.array([.2, .3])
+    dxy = np.array([.1, .2])
+    # We need to work around the nonsensical (dpi-dependent) interpretation of
+    # offsets by the Shadow class...
+    plt.rcParams["savefig.dpi"] = "figure"
+    # Test image.
+    a1 = fig_test.subplots()
+    rect = mpatches.Rectangle(xy=xy, width=.5, height=.5)
+    shadow = mpatches.Shadow(rect, ox=dxy[0], oy=dxy[1])
+    a1.add_patch(rect)
+    a1.add_patch(shadow)
+    # Reference image.
+    a2 = fig_ref.subplots()
+    rect = mpatches.Rectangle(xy=xy, width=.5, height=.5)
+    shadow = mpatches.Rectangle(
+        xy=xy + fig_ref.dpi / 72 * dxy, width=.5, height=.5,
+        fc=np.asarray(mcolors.to_rgb(rect.get_fc())) * .3,
+        ec=np.asarray(mcolors.to_rgb(rect.get_fc())) * .3,
+        alpha=.5)
+    a2.add_patch(shadow)
+    a2.add_patch(rect)
