@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function
 import copy
 
 import numpy as np
@@ -13,6 +12,12 @@ import matplotlib.pyplot as plt
 from matplotlib import transforms
 
 
+def test_empty_closed_path():
+    path = Path(np.zeros((0, 2)), closed=True)
+    assert path.vertices.shape == (0, 2)
+    assert path.codes is None
+
+
 def test_readonly_path():
     path = Path.unit_circle()
 
@@ -105,7 +110,8 @@ def test_make_compound_path_empty():
     assert r.vertices.shape == (0, 2)
 
 
-@image_comparison(baseline_images=['xkcd'], remove_text=True)
+@image_comparison(baseline_images=['xkcd'], extensions=['png'],
+                  remove_text=True)
 def test_xkcd():
     np.random.seed(0)
 
@@ -117,6 +123,23 @@ def test_xkcd():
         ax.plot(x, y)
 
 
+@image_comparison(baseline_images=['xkcd_marker'], extensions=['png'],
+                  remove_text=True)
+def test_xkcd_marker():
+    np.random.seed(0)
+
+    x = np.linspace(0, 5, 8)
+    y1 = x
+    y2 = 5 - x
+    y3 = 2.5 * np.ones(8)
+
+    with plt.xkcd():
+        fig, ax = plt.subplots()
+        ax.plot(x, y1, '+', ms=10)
+        ax.plot(x, y2, 'o', ms=10)
+        ax.plot(x, y3, '^', ms=10)
+
+
 @image_comparison(baseline_images=['marker_paths'], extensions=['pdf'],
                   remove_text=True)
 def test_marker_paths_pdf():
