@@ -1,8 +1,3 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-from six.moves import zip
-
 import unittest
 
 import numpy as np
@@ -75,7 +70,7 @@ def test_external_transform_api():
 
 
 @image_comparison(baseline_images=['pre_transform_data'],
-                  tol=0.08)
+                  tol=0.08, remove_text=True, style='mpl20')
 def test_pre_transform_plotting():
     # a catch-all for as many as possible plot layouts which handle
     # pre-transforming the data NOTE: The axis range is important in this
@@ -198,7 +193,7 @@ def test_clipping_of_log():
                                  clip=(0, 0, 100, 100),
                                  simplify=False)
 
-    tpoints, tcodes = list(zip(*result))
+    tpoints, tcodes = zip(*result)
     assert_allclose(tcodes, [M, L, L, L, C])
 
 
