@@ -1,15 +1,13 @@
+from unittest.mock import MagicMock
+
 from matplotlib.cbook import iterable
 import matplotlib.pyplot as plt
 from matplotlib.testing.decorators import image_comparison
 import matplotlib.units as munits
 import numpy as np
 import datetime
-
-try:
-    # mock in python 3.3+
-    from unittest.mock import MagicMock
-except ImportError:
-    from mock import MagicMock
+import platform
+import pytest
 
 
 # Basic class that wraps numpy array and has units
@@ -41,11 +39,8 @@ class Quantity(object):
         return np.asarray(self.magnitude)
 
 
-# Tests that the conversion machinery works properly for classes that
-# work as a facade over numpy arrays (like pint)
-@image_comparison(baseline_images=['plot_pint'],
-                  extensions=['png'], remove_text=False, style='mpl20')
-def test_numpy_facade():
+@pytest.fixture
+def quantity_converter():
     # Create an instance of the conversion interface and
     # mock so we can check methods called
     qc = munits.ConversionInterface()
@@ -62,12 +57,29 @@ def test_numpy_facade():
         else:
             return Quantity(value, axis.get_units()).to(unit).magnitude
 
+    def default_units(value, axis):
+        if hasattr(value, 'units'):
+            return value.units
+        elif np.iterable(value):
+            for v in value:
+                if hasattr(v, 'units'):
+                    return v.units
+            return None
+
     qc.convert = MagicMock(side_effect=convert)
     qc.axisinfo = MagicMock(side_effect=lambda u, a: munits.AxisInfo(label=u))
-    qc.default_units = MagicMock(side_effect=lambda x, a: x.units)
+    qc.default_units = MagicMock(side_effect=default_units)
+    return qc
 
+
+# Tests that the conversion machinery works properly for classes that
+# work as a facade over numpy arrays (like pint)
+@image_comparison(baseline_images=['plot_pint'],
+                  tol={'aarch64': 0.02}.get(platform.machine(), 0.0),
+                  extensions=['png'], remove_text=False, style='mpl20')
+def test_numpy_facade(quantity_converter):
     # Register the class
-    munits.registry[Quantity] = qc
+    munits.registry[Quantity] = quantity_converter
 
     # Simple test
     y = Quantity(np.linspace(0, 30), 'miles')
@@ -81,13 +93,14 @@ def test_numpy_facade():
     ax.yaxis.set_units('inches')
     ax.xaxis.set_units('seconds')
 
-    assert qc.convert.called
-    assert qc.axisinfo.called
-    assert qc.default_units.called
+    assert quantity_converter.convert.called
+    assert quantity_converter.axisinfo.called
+    assert quantity_converter.default_units.called
 
 
 # Tests gh-8908
 @image_comparison(baseline_images=['plot_masked_units'],
+                  tol={'aarch64': 0.02}.get(platform.machine(), 0.0),
                   extensions=['png'], remove_text=True, style='mpl20')
 def test_plot_masked_units():
     data = np.linspace(-5, 5)
@@ -98,6 +111,15 @@ def test_plot_masked_units():
     ax.plot(data_masked_units)
 
 
+def test_empty_set_limits_with_units(quantity_converter):
+    # Register the class
+    munits.registry[Quantity] = quantity_converter
+
+    fig, ax = plt.subplots()
+    ax.set_xlim(Quantity(-1, 'meters'), Quantity(6, 'meters'))
+    ax.set_ylim(Quantity(-1, 'hours'), Quantity(16, 'hours'))
+
+
 @image_comparison(baseline_images=['jpl_bar_units'], extensions=['png'],
                   savefig_kwarg={'dpi': 120}, style='mpl20')
 def test_jpl_bar_units():
