@@ -1,6 +1,3 @@
-from __future__ import absolute_import, division, print_function
-import warnings
-
 import pytest
 
 import matplotlib
@@ -8,14 +5,8 @@ from matplotlib.testing.decorators import image_comparison
 import matplotlib.pyplot as plt
 
 
-with warnings.catch_warnings():
-    warnings.simplefilter('ignore')
-    needs_usetex = pytest.mark.skipif(
-        not matplotlib.checkdep_usetex(True),
-        reason='Missing TeX of Ghostscript or dvipng')
-
-
-@needs_usetex
+@pytest.mark.skipif(not matplotlib.checkdep_usetex(True),
+                    reason='Missing TeX or Ghostscript or dvipng')
 @image_comparison(baseline_images=['test_usetex'],
                   extensions=['pdf', 'png'],
                   tol=0.3)
