@@ -1,7 +1,7 @@
 """
 This module provides routines to adjust subplot params so that subplots are
-nicely fit in the figure. In doing so, only axis labels, tick labels and axes
-titles are currently considered.
+nicely fit in the figure. In doing so, only axis labels, tick labels, axes
+titles and offsetboxes that are anchored to axes are currently considered.
 
 Internally, it assumes that the margins (left_margin, etc.) which are
 differences between ax.get_tightbbox and ax.bbox are independent of axes
@@ -34,77 +34,63 @@ def _get_top(tight_bbox, axes_bbox):
     return tight_bbox.ymax - axes_bbox.ymax
 
 
-def auto_adjust_subplotpars(fig, renderer,
-                            nrows_ncols,
-                            num1num2_list,
-                            subplot_list,
-                            ax_bbox_list=None,
-                            pad=1.08, h_pad=None, w_pad=None,
-                            rect=None):
+def auto_adjust_subplotpars(
+        fig, renderer, nrows_ncols, num1num2_list, subplot_list,
+        ax_bbox_list=None, pad=1.08, h_pad=None, w_pad=None, rect=None):
     """
-    Return a dictionary of subplot parameters so that spacing between
-    subplots are adjusted. Note that this function ignore geometry
-    information of subplot itself, but uses what is given by
-    *nrows_ncols* and *num1num2_list* parameteres. Also, the results could be
-    incorrect if some subplots have ``adjustable=datalim``.
+    Return a dict of subplot parameters to adjust spacing between subplots
+    or ``None`` if resulting axes would have zero height or width.
 
-    Parameters:
-
-    nrows_ncols
-      number of rows and number of columns of the grid.
-
-    num1num2_list
-      list of numbers specifying the area occupied by the subplot
-
-    subplot_list
-      list of subplots that will be used to calcuate optimal subplot_params.
+    Note that this function ignores geometry information of subplot
+    itself, but uses what is given by the *nrows_ncols* and *num1num2_list*
+    parameters.  Also, the results could be incorrect if some subplots have
+    ``adjustable=datalim``.
 
+    Parameters
+    ----------
+    nrows_ncols : Tuple[int, int]
+        Number of rows and number of columns of the grid.
+    num1num2_list : List[int]
+        List of numbers specifying the area occupied by the subplot
+    subplot_list : list of subplots
+        List of subplots that will be used to calculate optimal subplot_params.
     pad : float
-      padding between the figure edge and the edges of subplots, as a fraction
-      of the font-size.
+        Padding between the figure edge and the edges of subplots, as a
+        fraction of the font size.
     h_pad, w_pad : float
-      padding (height/width) between edges of adjacent subplots.
-        Defaults to `pad_inches`.
-
-    rect
-      [left, bottom, right, top] in normalized (0, 1) figure coordinates.
+        Padding (height/width) between edges of adjacent subplots, as a
+        fraction of the font size.  Defaults to *pad*.
+    rect : Tuple[float, float, float, float]
+        [left, bottom, right, top] in normalized (0, 1) figure coordinates.
     """
     rows, cols = nrows_ncols
 
-    pad_inches = pad * FontProperties(
-                    size=rcParams["font.size"]).get_size_in_points() / 72.
-
+    font_size_inches = (
+        FontProperties(size=rcParams["font.size"]).get_size_in_points() / 72)
+    pad_inches = pad * font_size_inches
     if h_pad is not None:
-        vpad_inches = h_pad * FontProperties(
-                        size=rcParams["font.size"]).get_size_in_points() / 72.
+        vpad_inches = h_pad * font_size_inches
     else:
         vpad_inches = pad_inches
 
     if w_pad is not None:
-        hpad_inches = w_pad * FontProperties(
-                        size=rcParams["font.size"]).get_size_in_points() / 72.
+        hpad_inches = w_pad * font_size_inches
     else:
         hpad_inches = pad_inches
 
-    if len(subplot_list) == 0:
-        raise RuntimeError("")
-
-    if len(num1num2_list) != len(subplot_list):
-        raise RuntimeError("")
+    if len(num1num2_list) != len(subplot_list) or len(subplot_list) == 0:
+        raise ValueError
 
     if rect is None:
-        margin_left = None
-        margin_bottom = None
-        margin_right = None
-        margin_top = None
+        margin_left = margin_bottom = margin_right = margin_top = None
     else:
         margin_left, margin_bottom, _right, _top = rect
         if _right:
-            margin_right = 1. - _right
+            margin_right = 1 - _right
         else:
             margin_right = None
         if _top:
-            margin_top = 1. - _top
+            margin_top = 1 - _top
         else:
             margin_top = None
 
@@ -123,10 +109,11 @@ def auto_adjust_subplotpars(fig, renderer,
     for subplots, ax_bbox, (num1, num2) in zip(subplot_list,
                                                ax_bbox_list,
                                                num1num2_list):
+        if all(not ax.get_visible() for ax in subplots):
+            continue
 
-        #ax_bbox = union([ax.get_position(original=True) for ax in subplots])
-
-        tight_bbox_raw = union([ax.get_tightbbox(renderer) for ax in subplots])
+        tight_bbox_raw = union([ax.get_tightbbox(renderer) for ax in subplots
+                                if ax.get_visible()])
         tight_bbox = TransformedBbox(tight_bbox_raw,
                                      fig.transFigure.inverted())
 
@@ -185,29 +172,48 @@ def auto_adjust_subplotpars(fig, renderer,
         margin_bottom = max([sum(s) for s in vspaces[-cols:]] + [0])
         margin_bottom += pad_inches / fig_height_inch
 
+    if margin_left + margin_right >= 1:
+        warnings.warn('Tight layout not applied. The left and right margins '
+                      'cannot be made large '
+                      'enough to accommodate all axes decorations. ')
+        return None
+    if margin_bottom + margin_top >= 1:
+        warnings.warn('Tight layout not applied. '
+                      'The bottom and top margins cannot be made large '
+                      'enough to accommodate all axes decorations. ')
+        return None
+
     kwargs = dict(left=margin_left,
                   right=1 - margin_right,
                   bottom=margin_bottom,
                   top=1 - margin_top)
-
     if cols > 1:
-        hspace = max([sum(s)
-                      for i in range(rows)
-                      for s
-                      in hspaces[i * (cols + 1) + 1:(i + 1) * (cols + 1) - 1]])
-        hspace += hpad_inches / fig_width_inch
-        h_axes = ((1 - margin_right - margin_left) -
-                   hspace * (cols - 1)) / cols
-
-        kwargs["wspace"] = hspace / h_axes
+        hspace = (
+            max(sum(s)
+                for i in range(rows)
+                for s in hspaces[i * (cols + 1) + 1:(i + 1) * (cols + 1) - 1])
+            + hpad_inches / fig_width_inch)
+        # axes widths:
+        h_axes = (1 - margin_right - margin_left - hspace * (cols - 1)) / cols
+        if h_axes < 0:
+            warnings.warn('Tight layout not applied. '
+                          'tight_layout cannot make axes width small enough '
+                          'to accommodate all axes decorations')
+            return None
+        else:
+            kwargs["wspace"] = hspace / h_axes
 
     if rows > 1:
-        vspace = max([sum(s) for s in vspaces[cols:-cols]])
-        vspace += vpad_inches / fig_height_inch
-        v_axes = ((1 - margin_top - margin_bottom) -
-                  vspace * (rows - 1)) / rows
-
-        kwargs["hspace"] = vspace / v_axes
+        vspace = (max(sum(s) for s in vspaces[cols:-cols])
+                  + vpad_inches / fig_height_inch)
+        v_axes = (1 - margin_top - margin_bottom - vspace * (rows - 1)) / rows
+        if v_axes < 0:
+            warnings.warn('Tight layout not applied. '
+                          'tight_layout cannot make axes height small enough '
+                          'to accommodate all axes decorations')
+            return None
+        else:
+            kwargs["hspace"] = vspace / v_axes
 
     return kwargs
 
@@ -231,14 +237,13 @@ def get_renderer(fig):
 
 
 def get_subplotspec_list(axes_list, grid_spec=None):
-    """
-    Return a list of subplotspec from the given list of axes.  For an
-    instance of axes that does not support subplotspec, None is
-    inserted in the list.
+    """Return a list of subplotspec from the given list of axes.
 
-    If grid_spec is given, None is inserted for those not from
-    the given grid_spec.
+    For an instance of axes that does not support subplotspec, None is inserted
+    in the list.
 
+    If grid_spec is given, None is inserted for those not from the given
+    grid_spec.
     """
     subplotspec_list = []
     for ax in axes_list:
@@ -266,32 +271,32 @@ def get_subplotspec_list(axes_list, grid_spec=None):
 def get_tight_layout_figure(fig, axes_list, subplotspec_list, renderer,
                             pad=1.08, h_pad=None, w_pad=None, rect=None):
     """
-    Return subplot parameters for tight-layouted-figure with specified
-    padding.
+    Return subplot parameters for tight-layouted-figure with specified padding.
 
-    Parameters:
+    Parameters
+    ----------
+    fig : Figure
+    axes_list : list of Axes
+    subplotspec_list : list of `.SubplotSpec`
+        The subplotspecs of each axes.
+    renderer : renderer
+    pad : float
+        Padding between the figure edge and the edges of subplots, as a
+        fraction of the font size.
+    h_pad, w_pad : float
+        Padding (height/width) between edges of adjacent subplots.  Defaults to
+        *pad_inches*.
+    rect : Tuple[float, float, float, float], optional
+        (left, bottom, right, top) rectangle in normalized figure coordinates
+        that the whole subplots area (including labels) will fit into.
+        Defaults to using the entire figure.
 
-      *fig* : figure instance
+    Returns
+    -------
+    subplotspec or None
+        subplotspec kwargs to be passed to `.Figure.subplots_adjust` or
+        None if tight_layout could not be accomplished.
 
-      *axes_list* : a list of axes
-
-      *subplotspec_list* : a list of subplotspec associated with each
-        axes in axes_list
-
-      *renderer* : renderer instance
-
-      *pad* : float
-        padding between the figure edge and the edges of subplots,
-        as a fraction of the font-size.
-
-      *h_pad*, *w_pad* : float
-        padding (height/width) between edges of adjacent subplots.
-        Defaults to `pad_inches`.
-
-      *rect* : if rect is given, it is interpreted as a rectangle
-        (left, bottom, right, top) in the normalized figure
-        coordinate that the whole subplots area (including
-        labels) will fit into. Default is (0, 0, 1, 1).
     """
 
     subplot_list = []
@@ -299,14 +304,12 @@ def get_tight_layout_figure(fig, axes_list, subplotspec_list, renderer,
     ncols_list = []
     ax_bbox_list = []
 
-    subplot_dict = {}  # multiple axes can share
-                       # same subplot_interface (e.g., axes_grid1). Thus
-                       # we need to join them together.
+    subplot_dict = {}  # Multiple axes can share same subplot_interface (e.g.,
+                       # axes_grid1); thus we need to join them together.
 
     subplotspec_list2 = []
 
-    for ax, subplotspec in zip(axes_list,
-                               subplotspec_list):
+    for ax, subplotspec in zip(axes_list, subplotspec_list):
         if subplotspec is None:
             continue
 
@@ -322,6 +325,9 @@ def get_tight_layout_figure(fig, axes_list, subplotspec_list, renderer,
 
         subplots.append(ax)
 
+    if len(nrows_list) == 0 or len(ncols_list) == 0:
+        return {}
+
     max_nrows = max(nrows_list)
     max_ncols = max(ncols_list)
 
@@ -330,8 +336,16 @@ def get_tight_layout_figure(fig, axes_list, subplotspec_list, renderer,
         rows, cols, num1, num2 = subplotspec.get_geometry()
         div_row, mod_row = divmod(max_nrows, rows)
         div_col, mod_col = divmod(max_ncols, cols)
-        if (mod_row != 0) or (mod_col != 0):
-            raise RuntimeError("")
+        if mod_row != 0:
+            warnings.warn('tight_layout not applied: '
+                          'number of rows in subplot specifications must'
+                          'be multiples of one another.')
+            return {}
+        if mod_col != 0:
+            warnings.warn('tight_layout not applied: '
+                          'number of columns in subplot specifications must'
+                          'be multiples of one another.')
+            return {}
 
         rowNum1, colNum1 = divmod(num1, cols)
         if num2 is None:
