@@ -1,11 +1,10 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.testing.decorators import image_comparison
+from matplotlib.transforms import IdentityTransform
 
 from mpl_toolkits.axisartist.axislines import SubplotZero, Subplot
+from mpl_toolkits.axisartist import SubplotHost, ParasiteAxesAuxTrans
 
 from mpl_toolkits.axisartist import Axes
 
@@ -56,3 +55,35 @@ def test_Axes():
     ax.set_xscale('log')
 
     plt.show()
+
+
+@image_comparison(baseline_images=['ParasiteAxesAuxTrans_meshplot'],
+                  extensions=['png'], remove_text=True, style='default',
+                  tol=0.075)
+def test_ParasiteAxesAuxTrans():
+
+    data = np.ones((6, 6))
+    data[2, 2] = 2
+    data[0, :] = 0
+    data[-2, :] = 0
+    data[:, 0] = 0
+    data[:, -2] = 0
+    x = np.arange(6)
+    y = np.arange(6)
+    xx, yy = np.meshgrid(x, y)
+
+    funcnames = ['pcolor', 'pcolormesh', 'contourf']
+
+    fig = plt.figure()
+    for i, name in enumerate(funcnames):
+
+        ax1 = SubplotHost(fig, 1, 3, i+1)
+        fig.add_subplot(ax1)
+
+        ax2 = ParasiteAxesAuxTrans(ax1, IdentityTransform())
+        ax1.parasites.append(ax2)
+        getattr(ax2, name)(xx, yy, data)
+        ax1.set_xlim((0, 5))
+        ax1.set_ylim((0, 5))
+
+    ax2.contour(xx, yy, data, colors='k')
