@@ -3,8 +3,8 @@
 /* _backend_agg.h
 */
 
-#ifndef __BACKEND_AGG_H__
-#define __BACKEND_AGG_H__
+#ifndef MPL_BACKEND_AGG_H
+#define MPL_BACKEND_AGG_H
 
 #include <cmath>
 #include <vector>
@@ -43,8 +43,8 @@
 
 /**********************************************************************/
 
-// a helper class to pass agg::buffer objects around.  agg::buffer is
-// a class in the swig wrapper
+// a helper class to pass agg::buffer objects around.  
+
 class BufferRegion
 {
   public:
@@ -205,7 +205,6 @@ class RendererAgg
 
     void tostring_rgb(uint8_t *buf);
     void tostring_argb(uint8_t *buf);
-    void tostring_bgra(uint8_t *buf);
     agg::rect_i get_content_extents();
     void clear();
 
@@ -281,8 +280,8 @@ class RendererAgg
                                        DashesVector &linestyles,
                                        AntialiasedArray &antialiaseds,
                                        e_offset_position offset_position,
-                                       int check_snap,
-                                       int has_curves);
+                                       bool check_snap,
+                                       bool has_curves);
 
     template <class PointArray, class ColorArray>
     void _draw_gouraud_triangle(PointArray &points,
@@ -915,8 +914,8 @@ inline void RendererAgg::_draw_path_collection_generic(GCAgg &gc,
                                                        DashesVector &linestyles,
                                                        AntialiasedArray &antialiaseds,
                                                        e_offset_position offset_position,
-                                                       int check_snap,
-                                                       int has_curves)
+                                                       bool check_snap,
+                                                       bool has_curves)
 {
     typedef agg::conv_transform<typename PathGenerator::path_iterator> transformed_path_t;
     typedef PathNanRemover<transformed_path_t> nan_removed_t;
@@ -1068,8 +1067,8 @@ inline void RendererAgg::draw_path_collection(GCAgg &gc,
                                   linestyles,
                                   antialiaseds,
                                   offset_position,
-                                  1,
-                                  1);
+                                  true,
+                                  true);
 }
 
 template <class CoordinateArray>
@@ -1186,8 +1185,8 @@ inline void RendererAgg::draw_quad_mesh(GCAgg &gc,
                                   linestyles,
                                   antialiaseds,
                                   OFFSET_POSITION_FIGURE,
-                                  0,
-                                  0);
+                                  false,
+                                  false);
 }
 
 template <class PointArray, class ColorArray>
