@@ -1,16 +1,10 @@
-#define PY_SSIZE_T_CLEAN
 #include <Cocoa/Cocoa.h>
 #include <ApplicationServices/ApplicationServices.h>
 #include <sys/socket.h>
 #include <Python.h>
 
-#define PYOSINPUTHOOK_REPETITIVE 1 /* Remove this once Python is fixed */
-
-#if PY_MAJOR_VERSION >= 3
-#define PY3K 1
-#else
-#define PY3K 0
-#endif
+/* Remove this once Python is fixed: https://bugs.python.org/issue23237 */
+#define PYOSINPUTHOOK_REPETITIVE 1
 
 /* Proper way to check for the OS X version we are compiling for, from
    http://developer.apple.com/documentation/DeveloperTools/Conceptual/cross_development */
@@ -273,6 +267,39 @@ static int wait_for_stdin(void)
 
 /* ---------------------------- Python classes ---------------------------- */
 
+static bool backend_inited = false;
+
+static void lazy_init(void) {
+    if (backend_inited) {
+        return;
+    }
+    backend_inited = true;
+
+    NSApp = [NSApplication sharedApplication];
+
+    PyOS_InputHook = wait_for_stdin;
+
+    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
+    WindowServerConnectionManager* connectionManager = [WindowServerConnectionManager sharedManager];
+    NSWorkspace* workspace = [NSWorkspace sharedWorkspace];
+    NSNotificationCenter* notificationCenter = [workspace notificationCenter];
+    [notificationCenter addObserver: connectionManager
+                           selector: @selector(launch:)
+                               name: NSWorkspaceDidLaunchApplicationNotification
+                             object: nil];
+    [pool release];
+}
+
+static PyObject*
+event_loop_is_running(PyObject* self)
+{
+    if (backend_inited) {
+        Py_RETURN_TRUE;
+    } else {
+        Py_RETURN_FALSE;
+    }
+}
+
 static CGFloat _get_device_scale(CGContextRef cr)
 {
     CGSize pixelSize = CGContextConvertSizeToDeviceSpace(cr, CGSizeMake(1, 1));
@@ -287,6 +314,7 @@ typedef struct {
 static PyObject*
 FigureCanvas_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
+    lazy_init();
     FigureCanvas *self = (FigureCanvas*)type->tp_alloc(type, 0);
     if (!self) return NULL;
     self->view = [View alloc];
@@ -326,13 +354,8 @@ FigureCanvas_dealloc(FigureCanvas* self)
 static PyObject*
 FigureCanvas_repr(FigureCanvas* self)
 {
-#if PY3K
     return PyUnicode_FromFormat("FigureCanvas object %p wrapping NSView %p",
                                (void*)self, (void*)(self->view));
-#else
-    return PyString_FromFormat("FigureCanvas object %p wrapping NSView %p",
-                               (void*)self, (void*)(self->view));
-#endif
 }
 
 static PyObject*
@@ -652,6 +675,7 @@ typedef struct {
 static PyObject*
 FigureManager_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
+    lazy_init();
     Window* window = [Window alloc];
     if (!window) return NULL;
     FigureManager *self = (FigureManager*)type->tp_alloc(type, 0);
@@ -731,13 +755,8 @@ FigureManager_init(FigureManager *self, PyObject *args, PyObject *kwds)
 static PyObject*
 FigureManager_repr(FigureManager* self)
 {
-#if PY3K
     return PyUnicode_FromFormat("FigureManager object %p wrapping NSWindow %p",
                                (void*) self, (void*)(self->window));
-#else
-    return PyString_FromFormat("FigureManager object %p wrapping NSWindow %p",
-                               (void*) self, (void*)(self->window));
-#endif
 }
 
 static void
@@ -893,489 +912,6 @@ static PyTypeObject FigureManagerType = {
     FigureManager_new,          /* tp_new */
 };
 
-@interface NavigationToolbarHandler : NSObject
-{   PyObject* toolbar;
-}
-- (NavigationToolbarHandler*)initWithToolbar:(PyObject*)toolbar;
--(void)left:(id)sender;
--(void)right:(id)sender;
--(void)up:(id)sender;
--(void)down:(id)sender;
--(void)zoominx:(id)sender;
--(void)zoominy:(id)sender;
--(void)zoomoutx:(id)sender;
--(void)zoomouty:(id)sender;
-@end
-
-typedef struct {
-    PyObject_HEAD
-    NSPopUpButton* menu;
-    NavigationToolbarHandler* handler;
-} NavigationToolbar;
-
-@implementation NavigationToolbarHandler
-- (NavigationToolbarHandler*)initWithToolbar:(PyObject*)theToolbar
-{   [self init];
-    toolbar = theToolbar;
-    return self;
-}
-
--(void)left:(id)sender
-{   PyObject* result;
-    PyGILState_STATE gstate;
-    gstate = PyGILState_Ensure();
-    result = PyObject_CallMethod(toolbar, "panx", "i", -1);
-    if(result)
-        Py_DECREF(result);
-    else
-        PyErr_Print();
-    PyGILState_Release(gstate);
-}
-
--(void)right:(id)sender
-{   PyObject* result;
-    PyGILState_STATE gstate;
-    gstate = PyGILState_Ensure();
-    result = PyObject_CallMethod(toolbar, "panx", "i", 1);
-    if(result)
-        Py_DECREF(result);
-    else
-        PyErr_Print();
-    PyGILState_Release(gstate);
-}
-
--(void)up:(id)sender
-{   PyObject* result;
-    PyGILState_STATE gstate;
-    gstate = PyGILState_Ensure();
-    result = PyObject_CallMethod(toolbar, "pany", "i", 1);
-    if(result)
-        Py_DECREF(result);
-    else
-        PyErr_Print();
-    PyGILState_Release(gstate);
-}
-
--(void)down:(id)sender
-{   PyObject* result;
-    PyGILState_STATE gstate;
-    gstate = PyGILState_Ensure();
-    result = PyObject_CallMethod(toolbar, "pany", "i", -1);
-    if(result)
-        Py_DECREF(result);
-    else
-        PyErr_Print();
-    PyGILState_Release(gstate);
-}
-
--(void)zoominx:(id)sender
-{   PyObject* result;
-    PyGILState_STATE gstate;
-    gstate = PyGILState_Ensure();
-    result = PyObject_CallMethod(toolbar, "zoomx", "i", 1);
-    if(result)
-        Py_DECREF(result);
-    else
-        PyErr_Print();
-    PyGILState_Release(gstate);
-}
-
--(void)zoomoutx:(id)sender
-{   PyObject* result;
-    PyGILState_STATE gstate;
-    gstate = PyGILState_Ensure();
-    result = PyObject_CallMethod(toolbar, "zoomx", "i", -1);
-    if(result)
-        Py_DECREF(result);
-    else
-        PyErr_Print();
-    PyGILState_Release(gstate);
-}
-
--(void)zoominy:(id)sender
-{   PyObject* result;
-    PyGILState_STATE gstate;
-    gstate = PyGILState_Ensure();
-    result = PyObject_CallMethod(toolbar, "zoomy", "i", 1);
-    if(result)
-        Py_DECREF(result);
-    else
-        PyErr_Print();
-    PyGILState_Release(gstate);
-}
-
--(void)zoomouty:(id)sender
-{   PyObject* result;
-    PyGILState_STATE gstate;
-    gstate = PyGILState_Ensure();
-    result = PyObject_CallMethod(toolbar, "zoomy", "i", -1);
-    if(result)
-        Py_DECREF(result);
-    else
-        PyErr_Print();
-    PyGILState_Release(gstate);
-}
-
--(void)save_figure:(id)sender
-{   PyObject* result;
-    PyGILState_STATE gstate;
-    gstate = PyGILState_Ensure();
-    result = PyObject_CallMethod(toolbar, "save_figure", "");
-    if(result)
-        Py_DECREF(result);
-    else
-        PyErr_Print();
-    PyGILState_Release(gstate);
-}
-@end
-
-static PyObject*
-NavigationToolbar_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
-{
-    NavigationToolbarHandler* handler = [NavigationToolbarHandler alloc];
-    if (!handler) return NULL;
-    NavigationToolbar *self = (NavigationToolbar*)type->tp_alloc(type, 0);
-    if (!self)
-    {   [handler release];
-        return NULL;
-    }
-    self->handler = handler;
-    return (PyObject*)self;
-}
-
-static int
-NavigationToolbar_init(NavigationToolbar *self, PyObject *args, PyObject *kwds)
-{
-    int i;
-    NSRect rect;
-
-    const float smallgap = 2;
-    const float biggap = 10;
-    const int height = 32;
-
-    PyObject* images;
-    PyObject* obj;
-
-    FigureCanvas* canvas;
-    View* view;
-
-    obj = PyObject_GetAttrString((PyObject*)self, "canvas");
-    if (obj==NULL)
-    {
-        PyErr_SetString(PyExc_AttributeError, "Attempt to install toolbar for NULL canvas");
-        return -1;
-    }
-    Py_DECREF(obj); /* Don't increase the reference count */
-    if (!PyObject_IsInstance(obj, (PyObject*) &FigureCanvasType))
-    {
-        PyErr_SetString(PyExc_TypeError, "Attempt to install toolbar for object that is not a FigureCanvas");
-        return -1;
-    }
-    canvas = (FigureCanvas*)obj;
-    view = canvas->view;
-    if(!view)
-    {
-        PyErr_SetString(PyExc_RuntimeError, "NSView* is NULL");
-        return -1;
-    }
-
-    if(!PyArg_ParseTuple(args, "O", &images)) return -1;
-    if(!PyDict_Check(images)) return -1;
-
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
-    NSRect bounds = [view bounds];
-    NSWindow* window = [view window];
-
-    bounds.origin.y += height;
-    [view setFrame: bounds];
-
-    bounds.size.height += height;
-    [window setContentSize: bounds.size];
-
-    char* imagenames[9] = {"stock_left",
-                           "stock_right",
-                           "stock_zoom-in",
-                           "stock_zoom-out",
-                           "stock_up",
-                           "stock_down",
-                           "stock_zoom-in",
-                           "stock_zoom-out",
-                           "stock_save_as"};
-
-    NSString* tooltips[9] = {
-        @"Pan left with click or wheel mouse (bidirectional)",
-        @"Pan right with click or wheel mouse (bidirectional)",
-        @"Zoom In X (shrink the x axis limits) with click or wheel mouse (bidirectional)",
-        @"Zoom Out X (expand the x axis limits) with click or wheel mouse (bidirectional)",
-        @"Pan up with click or wheel mouse (bidirectional)",
-        @"Pan down with click or wheel mouse (bidirectional)",
-        @"Zoom in Y (shrink the y axis limits) with click or wheel mouse (bidirectional)",
-        @"Zoom Out Y (expand the y axis limits) with click or wheel mouse (bidirectional)",
-        @"Save the figure"};
-
-    SEL actions[9] = {@selector(left:),
-                      @selector(right:),
-                      @selector(zoominx:),
-                      @selector(zoomoutx:),
-                      @selector(up:),
-                      @selector(down:),
-                      @selector(zoominy:),
-                      @selector(zoomouty:),
-                      @selector(save_figure:)};
-
-    SEL scroll_actions[9][2] = {{@selector(left:),    @selector(right:)},
-                                {@selector(left:),    @selector(right:)},
-                                {@selector(zoominx:), @selector(zoomoutx:)},
-                                {@selector(zoominx:), @selector(zoomoutx:)},
-                                {@selector(up:),      @selector(down:)},
-                                {@selector(up:),      @selector(down:)},
-                                {@selector(zoominy:), @selector(zoomouty:)},
-                                {@selector(zoominy:), @selector(zoomouty:)},
-                                {nil,nil},
-                               };
-
-
-    rect.size.width = 120;
-    rect.size.height = 24;
-    rect.origin.x = biggap;
-    rect.origin.y = 0.5*(height - rect.size.height);
-    self->menu = [[NSPopUpButton alloc] initWithFrame: rect
-                                            pullsDown: YES];
-    [self->menu setAutoenablesItems: NO];
-    [[window contentView] addSubview: self->menu];
-    [self->menu release];
-    rect.origin.x += rect.size.width + biggap;
-    rect.size.width = 24;
-
-    self->handler = [self->handler initWithToolbar: (PyObject*)self];
-    for (i = 0; i < 9; i++)
-    {
-        NSButton* button;
-        SEL scrollWheelUpAction = scroll_actions[i][0];
-        SEL scrollWheelDownAction = scroll_actions[i][1];
-        if (scrollWheelUpAction && scrollWheelDownAction)
-        {
-            ScrollableButton* scrollable_button = [ScrollableButton alloc];
-            [scrollable_button initWithFrame: rect];
-            [scrollable_button setScrollWheelUpAction: scrollWheelUpAction];
-            [scrollable_button setScrollWheelDownAction: scrollWheelDownAction];
-            button = (NSButton*)scrollable_button;
-        }
-        else
-        {
-            button = [NSButton alloc];
-            [button initWithFrame: rect];
-        }
-        PyObject* imagedata = PyDict_GetItemString(images, imagenames[i]);
-        NSImage* image = _read_ppm_image(imagedata);
-        [button setBezelStyle: NSShadowlessSquareBezelStyle];
-        [button setButtonType: NSMomentaryLightButton];
-        if(image)
-        {
-            [button setImage: image];
-            [image release];
-        }
-        [button setToolTip: tooltips[i]];
-        [button setTarget: self->handler];
-        [button setAction: actions[i]];
-        [[window contentView] addSubview: button];
-        [button release];
-        rect.origin.x += rect.size.width + smallgap;
-    }
-    [[window contentView] display];
-    [pool release];
-
-    return 0;
-}
-
-static void
-NavigationToolbar_dealloc(NavigationToolbar *self)
-{
-    [self->handler release];
-    Py_TYPE(self)->tp_free((PyObject*)self);
-}
-
-static PyObject*
-NavigationToolbar_repr(NavigationToolbar* self)
-{
-#if PY3K
-    return PyUnicode_FromFormat("NavigationToolbar object %p", (void*)self);
-#else
-    return PyString_FromFormat("NavigationToolbar object %p", (void*)self);
-#endif
-}
-
-static char NavigationToolbar_doc[] =
-"NavigationToolbar\n";
-
-static PyObject*
-NavigationToolbar_update (NavigationToolbar* self)
-{
-    int n;
-    NSPopUpButton* button = self->menu;
-    if (!button)
-    {
-        PyErr_SetString(PyExc_RuntimeError, "Menu button is NULL");
-        return NULL;
-    }
-
-    PyObject* canvas = PyObject_GetAttrString((PyObject*)self, "canvas");
-    if (canvas==NULL)
-    {
-        PyErr_SetString(PyExc_AttributeError, "Failed to find canvas");
-        return NULL;
-    }
-    Py_DECREF(canvas); /* Don't keep a reference here */
-    PyObject* figure = PyObject_GetAttrString(canvas, "figure");
-    if (figure==NULL)
-    {
-        PyErr_SetString(PyExc_AttributeError, "Failed to find figure");
-        return NULL;
-    }
-    Py_DECREF(figure); /* Don't keep a reference here */
-    PyObject* axes = PyObject_GetAttrString(figure, "axes");
-    if (axes==NULL)
-    {
-        PyErr_SetString(PyExc_AttributeError, "Failed to find figure axes");
-        return NULL;
-    }
-    Py_DECREF(axes); /* Don't keep a reference here */
-    if (!PyList_Check(axes))
-    {
-        PyErr_SetString(PyExc_TypeError, "Figure axes is not a list");
-        return NULL;
-    }
-    n = PyList_GET_SIZE(axes);
-
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
-    [button removeAllItems];
-
-    NSMenu* menu = [button menu];
-    [menu addItem: [MenuItem menuItemWithTitle: @"Axes"]];
-
-    if (n==0)
-    {
-        [button setEnabled: NO];
-    }
-    else
-    {
-        int i;
-        [menu addItem: [MenuItem menuItemSelectAll]];
-        [menu addItem: [MenuItem menuItemInvertAll]];
-        [menu addItem: [NSMenuItem separatorItem]];
-        for (i = 0; i < n; i++)
-        {
-            [menu addItem: [MenuItem menuItemForAxis: i]];
-        }
-        [button setEnabled: YES];
-    }
-    [pool release];
-    Py_RETURN_NONE;
-}
-
-static PyObject*
-NavigationToolbar_get_active (NavigationToolbar* self)
-{
-    NSPopUpButton* button = self->menu;
-    if (!button)
-    {
-        PyErr_SetString(PyExc_RuntimeError, "Menu button is NULL");
-        return NULL;
-    }
-    NSMenu* menu = [button menu];
-    NSArray* items = [menu itemArray];
-    size_t n = [items count];
-    int* states = calloc(n, sizeof(int));
-    if (!states)
-    {
-        PyErr_SetString(PyExc_RuntimeError, "calloc failed");
-        return NULL;
-    }
-    int i;
-    unsigned int m = 0;
-    NSEnumerator* enumerator = [items objectEnumerator];
-    MenuItem* item;
-    while ((item = [enumerator nextObject]))
-    {
-        if ([item isSeparatorItem]) continue;
-        i = [item index];
-        if (i < 0) continue;
-        if ([item state]==NSOnState)
-        {
-            states[i] = 1;
-            m++;
-        }
-    }
-    Py_ssize_t list_index = 0;
-    PyObject* list = PyList_New(m);
-
-    size_t state_index;
-    for (state_index = 0; state_index < n; state_index++)
-    {
-        if(states[state_index]==1)
-        {
-            PyList_SET_ITEM(list, list_index++, PyLong_FromSize_t(state_index));
-        }
-    }
-    free(states);
-    return list;
-}
-
-static PyMethodDef NavigationToolbar_methods[] = {
-    {"update",
-     (PyCFunction)NavigationToolbar_update,
-     METH_NOARGS,
-     "Updates the toolbar menu."
-    },
-    {"get_active",
-     (PyCFunction)NavigationToolbar_get_active,
-     METH_NOARGS,
-     "Returns a list of integers identifying which items in the menu are selected."
-    },
-    {NULL}  /* Sentinel */
-};
-
-static PyTypeObject NavigationToolbarType = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    "_macosx.NavigationToolbar", /*tp_name*/
-    sizeof(NavigationToolbar), /*tp_basicsize*/
-    0,                         /*tp_itemsize*/
-    (destructor)NavigationToolbar_dealloc,     /*tp_dealloc*/
-    0,                         /*tp_print*/
-    0,                         /*tp_getattr*/
-    0,                         /*tp_setattr*/
-    0,                         /*tp_compare*/
-    (reprfunc)NavigationToolbar_repr,     /*tp_repr*/
-    0,                         /*tp_as_number*/
-    0,                         /*tp_as_sequence*/
-    0,                         /*tp_as_mapping*/
-    0,                         /*tp_hash */
-    0,                         /*tp_call*/
-    0,                         /*tp_str*/
-    0,                         /*tp_getattro*/
-    0,                         /*tp_setattro*/
-    0,                         /*tp_as_buffer*/
-    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,        /*tp_flags*/
-    NavigationToolbar_doc,     /* tp_doc */
-    0,                         /* tp_traverse */
-    0,                         /* tp_clear */
-    0,                         /* tp_richcompare */
-    0,                         /* tp_weaklistoffset */
-    0,                         /* tp_iter */
-    0,                         /* tp_iternext */
-    NavigationToolbar_methods, /* tp_methods */
-    0,                         /* tp_members */
-    0,                         /* tp_getset */
-    0,                         /* tp_base */
-    0,                         /* tp_dict */
-    0,                         /* tp_descr_get */
-    0,                         /* tp_descr_set */
-    0,                         /* tp_dictoffset */
-    (initproc)NavigationToolbar_init,      /* tp_init */
-    0,                         /* tp_alloc */
-    NavigationToolbar_new,     /* tp_new */
-};
-
 @interface NavigationToolbar2Handler : NSObject
 {   PyObject* toolbar;
     NSButton* panbutton;
@@ -1575,6 +1111,7 @@ typedef struct {
 static PyObject*
 NavigationToolbar2_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
+    lazy_init();
     NavigationToolbar2Handler* handler = [NavigationToolbar2Handler alloc];
     if (!handler) return NULL;
     NavigationToolbar2 *self = (NavigationToolbar2*)type->tp_alloc(type, 0);
@@ -1744,11 +1281,7 @@ NavigationToolbar2_dealloc(NavigationToolbar2 *self)
 static PyObject*
 NavigationToolbar2_repr(NavigationToolbar2* self)
 {
-#if PY3K
     return PyUnicode_FromFormat("NavigationToolbar2 object %p", (void*)self);
-#else
-    return PyString_FromFormat("NavigationToolbar2 object %p", (void*)self);
-#endif
 }
 
 static char NavigationToolbar2_doc[] =
@@ -1759,11 +1292,7 @@ NavigationToolbar2_set_message(NavigationToolbar2 *self, PyObject* args)
 {
     const char* message;
 
-#if PY3K
     if(!PyArg_ParseTuple(args, "y", &message)) return NULL;
-#else
-    if(!PyArg_ParseTuple(args, "s", &message)) return NULL;
-#endif
 
     NSText* messagebox = self->messagebox;
 
@@ -1870,11 +1399,7 @@ choose_save_file(PyObject* unused, PyObject* args)
         unsigned int n = [filename length];
         unichar* buffer = malloc(n*sizeof(unichar));
         [filename getCharacters: buffer];
-#if PY3K
-        PyObject* string =  PyUnicode_FromKindAndData(PyUnicode_2BYTE_KIND, buffer, n);
-#else
-        PyObject* string =  PyUnicode_FromUnicode(buffer, n);
-#endif
+        PyObject* string = PyUnicode_FromKindAndData(PyUnicode_2BYTE_KIND, buffer, n);
         free(buffer);
         return string;
     }
@@ -2279,8 +1804,16 @@ static int _copy_agg_buffer(CGContextRef cr, PyObject *renderer)
     NSWindow* window = [self window];
     if ([window isKeyWindow]==false) return;
 
+    int x, y;
+    NSPoint location = [event locationInWindow];
+    location = [self convertPoint: location fromView: nil];
+    x = location.x * device_scale;
+    y = location.y * device_scale;
+
     gstate = PyGILState_Ensure();
-    result = PyObject_CallMethod(canvas, "enter_notify_event", "");
+    result = PyObject_CallMethod(canvas, "enter_notify_event", "O(ii)",
+            Py_None, x, y);
+
     if(result)
         Py_DECREF(result);
     else
@@ -2847,6 +2380,7 @@ typedef struct {
 static PyObject*
 Timer_new(PyTypeObject* type, PyObject *args, PyObject *kwds)
 {
+    lazy_init();
     Timer* self = (Timer*)type->tp_alloc(type, 0);
     if (!self) return NULL;
     self->timer = NULL;
@@ -2856,13 +2390,8 @@ Timer_new(PyTypeObject* type, PyObject *args, PyObject *kwds)
 static PyObject*
 Timer_repr(Timer* self)
 {
-#if PY3K
     return PyUnicode_FromFormat("Timer object %p wrapping CFRunLoopTimerRef %p",
                                (void*) self, (void*)(self->timer));
-#else
-    return PyString_FromFormat("Timer object %p wrapping CFRunLoopTimerRef %p",
-                               (void*) self, (void*)(self->timer));
-#endif
 }
 
 static char Timer_doc[] =
@@ -2893,6 +2422,7 @@ Timer__timer_start(Timer* self, PyObject* args)
     CFRunLoopTimerRef timer;
     CFRunLoopTimerContext context;
     double milliseconds;
+    CFAbsoluteTime firstFire;
     CFTimeInterval interval;
     PyObject* attribute;
     PyObject* failure;
@@ -2917,12 +2447,15 @@ Timer__timer_start(Timer* self, PyObject* args)
         PyErr_SetString(PyExc_AttributeError, "Timer has no attribute '_single'");
         return NULL;
     }
+    // Need to tell when to first fire this timer, so get the current time
+    // and add an interval.
+    interval = milliseconds / 1000.0;
+    firstFire = CFAbsoluteTimeGetCurrent() + interval;
     switch (PyObject_IsTrue(attribute)) {
         case 1:
             interval = 0;
             break;
-        case 0:
-            interval = milliseconds / 1000.0;
+        case 0: // Set by default above
             break;
         case -1:
         default:
@@ -2946,7 +2479,7 @@ Timer__timer_start(Timer* self, PyObject* args)
     context.copyDescription = NULL;
     context.info = attribute;
     timer = CFRunLoopTimerCreate(kCFAllocatorDefault,
-                                 0,
+                                 firstFire,
                                  interval,
                                  0,
                                  0,
@@ -3044,41 +2577,49 @@ static PyTypeObject TimerType = {
 
 static bool verify_framework(void)
 {
-#ifdef COMPILING_FOR_10_6
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
-    NSRunningApplication* app = [NSRunningApplication currentApplication];
-    NSApplicationActivationPolicy activationPolicy = [app activationPolicy];
-    [pool release];
-    switch (activationPolicy) {
-        case NSApplicationActivationPolicyRegular:
-        case NSApplicationActivationPolicyAccessory:
-            return true;
-        case NSApplicationActivationPolicyProhibited:
-            break;
-    }
-#else
     ProcessSerialNumber psn;
+    /* These methods are deprecated, but they don't require the app to
+       have started  */
+#ifdef COMPILING_FOR_10_6
+         NSApp = [NSApplication sharedApplication];
+         NSApplicationActivationPolicy activationPolicy = [NSApp activationPolicy];
+         switch (activationPolicy) {
+             case NSApplicationActivationPolicyRegular:
+             case NSApplicationActivationPolicyAccessory:
+                 return true;
+             case NSApplicationActivationPolicyProhibited:
+                 break;
+         }
+#else
     if (CGMainDisplayID()!=0
      && GetCurrentProcess(&psn)==noErr
      && SetFrontProcess(&psn)==noErr) return true;
 #endif
-    PyErr_SetString(PyExc_RuntimeError,
+    PyErr_SetString(PyExc_ImportError,
         "Python is not installed as a framework. The Mac OS X backend will "
         "not be able to function correctly if Python is not installed as a "
         "framework. See the Python documentation for more information on "
         "installing Python as a framework on Mac OS X. Please either reinstall "
         "Python as a framework, or try one of the other backends. If you are "
-        "using (Ana)Conda please install python.app and replace the use of 'python' "
-        "with 'pythonw'. See 'Working with Matplotlib on OSX' "
-        "in the Matplotlib FAQ for more information.");
+        "using (Ana)Conda please install python.app and replace the use of "
+        "'python' with 'pythonw'. See 'Working with Matplotlib on OSX' in the "
+        "Matplotlib FAQ for more information.");
     return false;
 }
 
 static struct PyMethodDef methods[] = {
+   {"event_loop_is_running",
+    (PyCFunction)event_loop_is_running,
+    METH_NOARGS,
+    "Return whether the OSX backend has set up the NSApp main event loop."
+   },
    {"show",
     (PyCFunction)show,
     METH_NOARGS,
-    "Show all the figures and enter the main loop.\nThis function does not return until all Matplotlib windows are closed,\nand is normally not needed in interactive sessions."
+    "Show all the figures and enter the main loop.\n"
+    "\n"
+    "This function does not return until all Matplotlib windows are closed,\n"
+    "and is normally not needed in interactive sessions."
    },
    {"choose_save_file",
     (PyCFunction)choose_save_file,
@@ -3090,11 +2631,9 @@ static struct PyMethodDef methods[] = {
     METH_VARARGS,
     "Sets the active cursor."
    },
-   {NULL,          NULL, 0, NULL}/* sentinel */
+   {NULL, NULL, 0, NULL} /* sentinel */
 };
 
-#if PY3K
-
 static struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
     "_macosx",
@@ -3108,68 +2647,30 @@ static struct PyModuleDef moduledef = {
 };
 
 PyObject* PyInit__macosx(void)
-
-#else
-
-void init_macosx(void)
-#endif
 {
     PyObject *module;
 
     if (PyType_Ready(&FigureCanvasType) < 0
      || PyType_Ready(&FigureManagerType) < 0
-     || PyType_Ready(&NavigationToolbarType) < 0
      || PyType_Ready(&NavigationToolbar2Type) < 0
      || PyType_Ready(&TimerType) < 0)
-#if PY3K
         return NULL;
-#else
-        return;
-#endif
-
-    NSApp = [NSApplication sharedApplication];
 
     if (!verify_framework())
-#if PY3K
         return NULL;
-#else
-        return;
-#endif
 
-#if PY3K
     module = PyModule_Create(&moduledef);
-    if (module==NULL) return NULL;
-#else
-    module = Py_InitModule4("_macosx",
-                            methods,
-                            "Mac OS X native backend",
-                            NULL,
-                            PYTHON_API_VERSION);
-#endif
+    if (!module)
+        return NULL;
 
     Py_INCREF(&FigureCanvasType);
     Py_INCREF(&FigureManagerType);
-    Py_INCREF(&NavigationToolbarType);
     Py_INCREF(&NavigationToolbar2Type);
     Py_INCREF(&TimerType);
     PyModule_AddObject(module, "FigureCanvas", (PyObject*) &FigureCanvasType);
     PyModule_AddObject(module, "FigureManager", (PyObject*) &FigureManagerType);
-    PyModule_AddObject(module, "NavigationToolbar", (PyObject*) &NavigationToolbarType);
     PyModule_AddObject(module, "NavigationToolbar2", (PyObject*) &NavigationToolbar2Type);
     PyModule_AddObject(module, "Timer", (PyObject*) &TimerType);
 
-    PyOS_InputHook = wait_for_stdin;
-
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
-    WindowServerConnectionManager* connectionManager = [WindowServerConnectionManager sharedManager];
-    NSWorkspace* workspace = [NSWorkspace sharedWorkspace];
-    NSNotificationCenter* notificationCenter = [workspace notificationCenter];
-    [notificationCenter addObserver: connectionManager
-                           selector: @selector(launch:)
-                               name: NSWorkspaceDidLaunchApplicationNotification
-                             object: nil];
-    [pool release];
-#if PY3K
     return module;
-#endif
 }
