@@ -1,11 +1,15 @@
 /* -*- mode: c++; c-basic-offset: 4 -*- */
 
+// this code is heavily adapted from the paint license, which is in
+// the file paint.license (BSD compatible) included in this
+// distribution.  TODO, add license file to MANIFEST.in and CVS
+
 /* For linux, png.h must be imported before Python.h because
    png.h needs to be the one to define setjmp.
    Undefining _POSIX_C_SOURCE and _XOPEN_SOURCE stops a couple
    of harmless warnings.
 */
-#define PY_SSIZE_T_CLEAN
+
 
 extern "C" {
 #   include <png.h>
@@ -59,11 +63,7 @@ static void write_png_data(png_structp png_ptr, png_bytep data, png_size_t lengt
     PyObject *write_method = PyObject_GetAttrString(py_file_obj, "write");
     PyObject *result = NULL;
     if (write_method) {
-#if PY3K
         result = PyObject_CallFunction(write_method, (char *)"y#", data, length);
-#else
-        result = PyObject_CallFunction(write_method, (char *)"s#", data, length);
-#endif
     }
     Py_XDECREF(write_method);
     Py_XDECREF(result);
@@ -130,15 +130,15 @@ const char *Py_write_png__doc__ =
     "    - Comment: Miscellaneous comment; conversion\n"
     "               from other image format\n"
     "\n"
+    "    For more details see the PNG specification:\n"
+    "    https://www.w3.org/TR/2003/REC-PNG-20031110/#11keywords\n"
+    "\n"
     "Returns\n"
     "-------\n"
     "buffer : bytes or None\n"
     "    Byte string containing the PNG content if None was passed in for\n"
     "    file, otherwise None is returned.\n";
 
-// this code is heavily adapted from
-// https://www.object-craft.com.au/projects/paint/ which licensed under the
-// (BSD compatible) LICENSE_PAINT which is included in this distribution.
 static PyObject *Py_write_png(PyObject *self, PyObject *args, PyObject *kwds)
 {
     numpy::array_view<unsigned char, 3> buffer;
@@ -174,7 +174,7 @@ static PyObject *Py_write_png(PyObject *self, PyObject *args, PyObject *kwds)
 
     png_uint_32 width = (png_uint_32)buffer.dim(1);
     png_uint_32 height = (png_uint_32)buffer.dim(0);
-    int channels = buffer.dim(2);
+    npy_intp channels = buffer.dim(2);
     std::vector<png_bytep> row_pointers(height);
     for (png_uint_32 row = 0; row < (png_uint_32)height; ++row) {
         row_pointers[row] = (png_bytep)&buffer(row, 0, 0);
@@ -233,11 +233,7 @@ static PyObject *Py_write_png(PyObject *self, PyObject *args, PyObject *kwds)
         }
         buff.cursor = 0;
     } else {
-        #if PY3K
         if (close_file) {
-        #else
-        if (close_file || PyFile_Check(py_file)) {
-        #endif
             fp = mpl_PyFile_Dup(py_file, (char *)"wb", &offset);
         }
 
@@ -310,7 +306,6 @@ static PyObject *Py_write_png(PyObject *self, PyObject *args, PyObject *kwds)
 
         while (PyDict_Next(metadata, &pos, &meta_key, &meta_val)) {
             text[meta_pos].compression = PNG_TEXT_COMPRESSION_NONE;
-#if PY3K
             if (PyUnicode_Check(meta_key)) {
                 PyObject *temp_key = PyUnicode_AsEncodedString(meta_key, "latin_1", "strict");
                 if (temp_key != NULL) {
@@ -333,10 +328,6 @@ static PyObject *Py_write_png(PyObject *self, PyObject *args, PyObject *kwds)
             } else {
                 text[meta_pos].text = (char *)"Invalid value in metadata";
             }
-#else
-            text[meta_pos].key = PyString_AsString(meta_key);
-            text[meta_pos].text = PyString_AsString(meta_val);
-#endif
 #ifdef PNG_iTXt_SUPPORTED
             text[meta_pos].lang = NULL;
 #endif
@@ -467,11 +458,7 @@ static PyObject *_read_png(PyObject *filein, bool float_result)
         py_file = filein;
     }
 
-    #if PY3K
     if (close_file) {
-    #else
-    if (close_file || PyFile_Check(py_file)) {
-    #endif
         fp = mpl_PyFile_Dup(py_file, (char *)"rb", &offset);
     }
 
@@ -600,12 +587,12 @@ static PyObject *_read_png(PyObject *filein, bool float_result)
                 if (bit_depth == 16) {
                     png_uint_16 *ptr = &reinterpret_cast<png_uint_16 *>(row)[x * dimensions[2]];
                     for (png_uint_32 p = 0; p < (png_uint_32)dimensions[2]; p++) {
-                        A(y, x, p) = (float)(ptr[p]) / max_value;
+                        A(y, x, p) = (float)(ptr[p] / max_value);
                     }
                 } else {
                     png_byte *ptr = &(row[x * dimensions[2]]);
                     for (png_uint_32 p = 0; p < (png_uint_32)dimensions[2]; p++) {
-                        A(y, x, p) = (float)(ptr[p]) / max_value;
+                        A(y, x, p) = (float)(ptr[p] / max_value);
                     }
                 }
             }
@@ -739,7 +726,6 @@ static PyMethodDef module_methods[] = {
 
 extern "C" {
 
-#if PY3K
     static struct PyModuleDef moduledef = {
         PyModuleDef_HEAD_INIT,
         "_png",
@@ -752,27 +738,14 @@ extern "C" {
         NULL
     };
 
-#define INITERROR return NULL
-
     PyMODINIT_FUNC PyInit__png(void)
-
-#else
-#define INITERROR return
-
-    PyMODINIT_FUNC init_png(void)
-#endif
-
     {
         PyObject *m;
 
-#if PY3K
         m = PyModule_Create(&moduledef);
-#else
-        m = Py_InitModule3("_png", module_methods, NULL);
-#endif
 
         if (m == NULL) {
-            INITERROR;
+            return NULL;
         }
 
         import_array();
@@ -782,12 +755,10 @@ extern "C" {
             PyModule_AddIntConstant(m, "PNG_FILTER_UP", PNG_FILTER_UP) ||
             PyModule_AddIntConstant(m, "PNG_FILTER_AVG", PNG_FILTER_AVG) ||
             PyModule_AddIntConstant(m, "PNG_FILTER_PAETH", PNG_FILTER_PAETH)) {
-            INITERROR;
+            return NULL;
         }
 
 
-#if PY3K
         return m;
-#endif
     }
 }
