@@ -5,7 +5,7 @@
 
   Python wrapper for TrueType conversion library in ../ttconv.
  */
-#define PY_SSIZE_T_CLEAN
+
 #include "mplutils.h"
 
 #include <Python.h>
@@ -86,11 +86,7 @@ int pyiterable_to_vector_int(PyObject *object, void *address)
 
     PyObject *item;
     while ((item = PyIter_Next(iterator))) {
-#if PY3K
         long value = PyLong_AsLong(item);
-#else
-        long value = PyInt_AsLong(item);
-#endif
         Py_DECREF(item);
         if (value == -1 && PyErr_Occurred()) {
             return 0;
@@ -113,11 +109,7 @@ static PyObject *convert_ttf_to_ps(PyObject *self, PyObject *args, PyObject *kwd
     static const char *kwlist[] = { "filename", "output", "fonttype", "glyph_ids", NULL };
     if (!PyArg_ParseTupleAndKeywords(args,
                                      kwds,
-#if PY_MAJOR_VERSION == 3
                                      "yO&i|O&:convert_ttf_to_ps",
-#else
-                                     "sO&i|O&:convert_ttf_to_ps",
-#endif
                                      (char **)kwlist,
                                      &filename,
                                      fileobject_to_PythonFileWriter,
@@ -193,11 +185,7 @@ static PyObject *py_get_pdf_charprocs(PyObject *self, PyObject *args, PyObject *
     static const char *kwlist[] = { "filename", "glyph_ids", NULL };
     if (!PyArg_ParseTupleAndKeywords(args,
                                      kwds,
-#if PY_MAJOR_VERSION == 3
                                      "y|O&:get_pdf_charprocs",
-#else
-                                     "s|O&:get_pdf_charprocs",
-#endif
                                      (char **)kwlist,
                                      &filename,
                                      pyiterable_to_vector_int,
@@ -279,7 +267,6 @@ static const char *module_docstring =
     "fonts to Postscript Type 3, Postscript Type 42 and "
     "Pdf Type 3 fonts.";
 
-#if PY3K
 static PyModuleDef ttconv_module = {
     PyModuleDef_HEAD_INIT,
     "ttconv",
@@ -298,10 +285,3 @@ PyInit_ttconv(void)
 
     return m;
 }
-#else
-PyMODINIT_FUNC
-initttconv(void)
-{
-    Py_InitModule3("ttconv", ttconv_methods, module_docstring);
-}
-#endif
