@@ -1,8 +1,8 @@
 /* -*- mode: c++; c-basic-offset: 4 -*- */
 
-#ifndef __PY_ADAPTORS_H__
-#define __PY_ADAPTORS_H__
-#define PY_SSIZE_T_CLEAN
+#ifndef MPL_PY_ADAPTORS_H
+#define MPL_PY_ADAPTORS_H
+
 /***************************************************************************
  * This module contains a number of C++ classes that adapt Python data
  * structures to C++ and Agg-friendly interfaces.
