@@ -9,8 +9,24 @@ How to use constrained-layout to fit plots within your figure cleanly.
 legends and colorbars so that they fit in the figure window while still
 preserving, as best they can, the logical layout requested by the user.
 
-*constrained_layout* is similar to *tight_layout*, but uses a constraint
-solver to determine the size of axes that allows them to fit.
+*constrained_layout* is similar to
+:doc:`tight_layout</tutorials/intermediate/tight_layout_guide>`,
+but uses a constraint solver to determine the size of axes that allows
+them to fit.
+
+*constrained_layout* needs to be activated before any axes are added to
+a figure. Two ways of doing so are
+
+* using the respective argument to :func:`~.pyplot.subplots` or
+  :func:`~.pyplot.figure`, e.g.::
+
+      plt.subplots(constrained_layout=True)
+
+* activate it via :ref:`rcParams<matplotlib-rcparams>`, like::
+
+      plt.rcParams['figure.constrained_layout.use'] = True
+
+Those are described in detail throughout the following sections.
 
 .. warning::
 
@@ -62,7 +78,7 @@ def example_plot(ax, fontsize=12, nodec=False):
         ax.set_yticklabels('')
 
 
-fig, ax = plt.subplots()
+fig, ax = plt.subplots(constrained_layout=False)
 example_plot(ax, fontsize=24)
 
 ###############################################################################
@@ -83,7 +99,7 @@ for ax in axs.flatten():
     example_plot(ax)
 
 ###############################################################################
-# Specifying `constrained_layout=True` in the call to `plt.subplots`
+# Specifying ``constrained_layout=True`` in the call to ``plt.subplots``
 # causes the layout to be properly constrained.
 
 fig, axs = plt.subplots(2, 2, constrained_layout=True)
@@ -102,28 +118,40 @@ for ax in axs.flatten():
 #
 # .. note::
 #
-#   For the `pcolormesh` kwargs (``pc_kwargs``) we use a dictionary.
-#   Below we will assign one colorbar to a number of axes each containing
-#   a `ScalarMappable`; specifying the norm and colormap ensures
-#   the colorbar is accurate for all the axes.
+#   For the `~.axes.Axes.pcolormesh` kwargs (``pc_kwargs``) we use a
+#   dictionary. Below we will assign one colorbar to a number of axes each
+#   containing a `~.cm.ScalarMappable`; specifying the norm and colormap
+#   ensures the colorbar is accurate for all the axes.
 
 arr = np.arange(100).reshape((10, 10))
 norm = mcolors.Normalize(vmin=0., vmax=100.)
 # see note above: this makes all pcolormesh calls consistent:
-pc_kwargs = {'rasterized':True, 'cmap':'viridis', 'norm':norm}
+pc_kwargs = {'rasterized': True, 'cmap': 'viridis', 'norm': norm}
 fig, ax = plt.subplots(figsize=(4, 4), constrained_layout=True)
 im = ax.pcolormesh(arr, **pc_kwargs)
 fig.colorbar(im, ax=ax, shrink=0.6)
 
 ############################################################################
 # If you specify a list of axes (or other iterable container) to the
-# ``ax`` argument of ``colorbar``, constrained_layout will take space from all # axes that share the same gridspec.
+# ``ax`` argument of ``colorbar``, constrained_layout will take space from
+# the specified axes.
 
 fig, axs = plt.subplots(2, 2, figsize=(4, 4), constrained_layout=True)
 for ax in axs.flatten():
     im = ax.pcolormesh(arr, **pc_kwargs)
 fig.colorbar(im, ax=axs, shrink=0.6)
 
+############################################################################
+# If you specify a list of axes from inside a grid of axes, the colorbar
+# will steal space appropriately, and leave a gap, but all subplots will
+# still be the same size.
+
+fig, axs = plt.subplots(3, 3, figsize=(4, 4), constrained_layout=True)
+for ax in axs.flatten():
+    im = ax.pcolormesh(arr, **pc_kwargs)
+fig.colorbar(im, ax=axs[1:, ][:, 1], shrink=0.8)
+fig.colorbar(im, ax=axs[:, -1], shrink=0.6)
+
 ############################################################################
 # Note that there is a bit of a subtlety when specifying a single axes
 # as the parent.  In the following, it might be desirable and expected
@@ -153,7 +181,7 @@ fig.colorbar(im, ax=[axs[2]], shrink=0.6)
 # Suptitle
 # =========
 #
-# ``constrained_layout`` can also make room for ``suptitle``.
+# ``constrained_layout`` can also make room for `~.figure.Figure.suptitle`.
 
 fig, axs = plt.subplots(2, 2, figsize=(4, 4), constrained_layout=True)
 for ax in axs.flatten():
@@ -165,23 +193,71 @@ fig.suptitle('Big Suptitle')
 # Legends
 # =======
 #
-# Legends can be placed outside
-# of their parent axis.  Constrained-layout is designed to handle this.
+# Legends can be placed outside of their parent axis.
+# Constrained-layout is designed to handle this for :meth:`.Axes.legend`.
 # However, constrained-layout does *not* handle legends being created via
-# ``fig.legend()`` (yet).
+# :meth:`.Figure.legend` (yet).
 
 fig, ax = plt.subplots(constrained_layout=True)
 ax.plot(np.arange(10), label='This is a plot')
-ax.legend(loc='center left', bbox_to_anchor=(0.9, 0.5))
+ax.legend(loc='center left', bbox_to_anchor=(0.8, 0.5))
 
 #############################################
 # However, this will steal space from a subplot layout:
 
-fig, axs = plt.subplots(2, 2, constrained_layout=True)
-for ax in axs.flatten()[:-1]:
-    ax.plot(np.arange(10))
-axs[1, 1].plot(np.arange(10), label='This is a plot')
-axs[1, 1].legend(loc='center left', bbox_to_anchor=(0.9, 0.5))
+fig, axs = plt.subplots(1, 2, figsize=(4, 2), constrained_layout=True)
+axs[0].plot(np.arange(10))
+axs[1].plot(np.arange(10), label='This is a plot')
+axs[1].legend(loc='center left', bbox_to_anchor=(0.8, 0.5))
+
+#############################################
+# In order for a legend or other artist to *not* steal space
+# from the subplot layout, we can ``leg.set_in_layout(False)``.
+# Of course this can mean the legend ends up
+# cropped, but can be useful if the plot is subsequently called
+# with ``fig.savefig('outname.png', bbox_inches='tight')``.  Note,
+# however, that the legend's ``get_in_layout`` status will have to be
+# toggled again to make the saved file work, and we must manually
+# trigger a draw if we want constrained_layout to adjust the size
+# of the axes before printing.
+
+fig, axs = plt.subplots(1, 2, figsize=(4, 2), constrained_layout=True)
+
+axs[0].plot(np.arange(10))
+axs[1].plot(np.arange(10), label='This is a plot')
+leg = axs[1].legend(loc='center left', bbox_to_anchor=(0.8, 0.5))
+leg.set_in_layout(False)
+# trigger a draw so that constrained_layout is executed once
+# before we turn it off when printing....
+fig.canvas.draw()
+# we want the legend included in the bbox_inches='tight' calcs.
+leg.set_in_layout(True)
+# we don't want the layout to change at this point.
+fig.set_constrained_layout(False)
+fig.savefig('CL01.png', bbox_inches='tight', dpi=100)
+
+#############################################
+# The saved file looks like:
+#
+# .. image:: /_static/constrained_layout/CL01.png
+#    :align: center
+#
+# A better way to get around this awkwardness is to simply
+# use the legend method provided by `.Figure.legend`:
+fig, axs = plt.subplots(1, 2, figsize=(4, 2), constrained_layout=True)
+axs[0].plot(np.arange(10))
+lines = axs[1].plot(np.arange(10), label='This is a plot')
+labels = [l.get_label() for l in lines]
+leg = fig.legend(lines, labels, loc='center left',
+                 bbox_to_anchor=(0.8, 0.5), bbox_transform=axs[1].transAxes)
+fig.savefig('CL02.png', bbox_inches='tight', dpi=100)
+
+#############################################
+# The saved file looks like:
+#
+# .. image:: /_static/constrained_layout/CL02.png
+#    :align: center
+#
 
 ###############################################################################
 # Padding and Spacing
@@ -190,8 +266,8 @@ axs[1, 1].legend(loc='center left', bbox_to_anchor=(0.9, 0.5))
 # For constrained_layout, we have implemented a padding around the edge of
 # each axes.  This padding sets the distance from the edge of the plot,
 # and the minimum distance between adjacent plots.  It is specified in
-# inches by the keyword arguments `w_pad` and `h_pad` to the function
-# `fig.set_constrained_layout_pads`:
+# inches by the keyword arguments ``w_pad`` and ``h_pad`` to the function
+# `~.figure.Figure.set_constrained_layout_pads`:
 
 fig, axs = plt.subplots(2, 2, constrained_layout=True)
 for ax in axs.flatten():
@@ -210,7 +286,7 @@ fig.set_constrained_layout_pads(w_pad=2./72., h_pad=2./72.,
         hspace=0., wspace=0.)
 
 ##########################################
-# Spacing between subplots is set by `wspace` and `hspace`.  There are
+# Spacing between subplots is set by ``wspace`` and ``hspace``. There are
 # specified as a fraction of the size of the subplot group as a whole.
 # If the size of the figure is changed, then these spaces change in
 # proportion.  Note in the blow how the space at the edges doesn't change from
@@ -229,15 +305,17 @@ fig.set_constrained_layout_pads(w_pad=2./72., h_pad=2./72.,
 # Spacing with colorbars
 # -----------------------
 #
-# Colorbars still respect the `w_pad` and `h_pad` values.  However they will
-# be `wspace` and `hsapce` apart from other subplots.  Note the use of a `pad`
-# kwarg here in the `colorbar` call.  It defaults to 0.02 of the size of the
-# axis it is attached to.
+# Colorbars will be placed ``wspace`` and ``hsapce`` apart from other
+# subplots. The padding between the colorbar and the axis it is
+# attached to will never be less than ``w_pad`` (for a vertical colorbar)
+# or ``h_pad`` (for a horizontal colorbar). Note the use of the ``pad`` kwarg
+# here in the ``colorbar`` call.  It defaults to 0.02 of the size
+# of the axis it is attached to.
 
 fig, axs = plt.subplots(2, 2, constrained_layout=True)
 for ax in axs.flatten():
     pc = ax.pcolormesh(arr, **pc_kwargs)
-    fig.colorbar(im, ax=ax, shrink=0.6, pad=0)
+    fig.colorbar(pc, ax=ax, shrink=0.6, pad=0)
     ax.set_xticklabels('')
     ax.set_yticklabels('')
 fig.set_constrained_layout_pads(w_pad=2./72., h_pad=2./72.,
@@ -246,7 +324,7 @@ fig.set_constrained_layout_pads(w_pad=2./72., h_pad=2./72.,
 ##########################################
 # In the above example, the colorbar will not ever be closer than 2 pts to
 # the plot, but if we want it a bit further away, we can specify its value
-# for `pad` to be non-zero.
+# for ``pad`` to be non-zero.
 
 fig, axs = plt.subplots(2, 2, constrained_layout=True)
 for ax in axs.flatten():
@@ -259,16 +337,16 @@ fig.set_constrained_layout_pads(w_pad=2./72., h_pad=2./72.,
 
 ##########################################
 # rcParams
-# --------
+# ========
 #
-# There are five `rcParams` that can be set, either in a script
-# or in the `matplotlibrc` file.  They all have the prefix
-# `figure.constrained_layout`:
+# There are five :ref:`rcParams<matplotlib-rcparams>` that can be set,
+# either in a script or in the `matplotlibrc` file.
+# They all have the prefix ``figure.constrained_layout``:
 #
-# - `use`: Whether to use constrained_layout. Default is False
-# - `w_pad`, `h_pad`    Padding around axes objects.
+# - ``use``: Whether to use constrained_layout. Default is False
+# - ``w_pad``, ``h_pad``:    Padding around axes objects.
 #    Float representing inches.  Default is 3./72. inches (3 pts)
-# - `wspace`, `hspace`  Space between subplot groups.
+# - ``wspace``, ``hspace``:  Space between subplot groups.
 #    Float representing a fraction of the subplot widths being separated.
 #    Default is 0.02.
 
@@ -285,8 +363,10 @@ for ax in axs.flatten():
 # with :func:`~matplotlib.figure.Figure.subplots` or
 # :func:`~matplotlib.gridspec.GridSpec` and
 # :func:`~matplotlib.figure.Figure.add_subplot`.
+#
+# Note that in what follows ``constrained_layout=True``
 
-fig = plt.figure(constrained_layout=True)
+fig = plt.figure()
 
 gs1 = gridspec.GridSpec(2, 1, figure=fig)
 ax1 = fig.add_subplot(gs1[0])
@@ -296,20 +376,21 @@ example_plot(ax1)
 example_plot(ax2)
 
 ###############################################################################
-# More complicated gridspec layouts are possible...
+# More complicated gridspec layouts are possible.  Note here we use the
+# convenenience functions ``add_gridspec`` and ``subgridspec``
 
-fig = plt.figure(constrained_layout=True)
+fig = plt.figure()
 
-gs0 = gridspec.GridSpec(1, 2, figure=fig)
+gs0 = fig.add_gridspec(1, 2)
 
-gs1 = gridspec.GridSpecFromSubplotSpec(2, 1, gs0[0])
+gs1 = gs0[0].subgridspec(2, 1)
 ax1 = fig.add_subplot(gs1[0])
 ax2 = fig.add_subplot(gs1[1])
 
 example_plot(ax1)
 example_plot(ax2)
 
-gs2 = gridspec.GridSpecFromSubplotSpec(3, 1, gs0[1])
+gs2 = gs0[1].subgridspec(3, 1)
 
 for ss in gs2:
     ax = fig.add_subplot(ss)
@@ -324,9 +405,9 @@ ax.set_xlabel("x-label", fontsize=12)
 # extent.  If we want the top and bottom of the two grids to line up then
 # they need to be in the same gridspec:
 
-fig = plt.figure(constrained_layout=True)
+fig = plt.figure()
 
-gs0 = gridspec.GridSpec(6, 2, figure=fig)
+gs0 = fig.add_gridspec(6, 2)
 
 ax1 = fig.add_subplot(gs0[:3, 0])
 ax2 = fig.add_subplot(gs0[3:, 0])
@@ -349,10 +430,10 @@ example_plot(ax)
 
 
 def docomplicated(suptitle=None):
-    fig = plt.figure(constrained_layout=True)
-    gs0 = gridspec.GridSpec(1, 2, figure=fig, width_ratios=[1., 2.])
-    gsl = gridspec.GridSpecFromSubplotSpec(2, 1, gs0[0])
-    gsr = gridspec.GridSpecFromSubplotSpec(2, 2, gs0[1])
+    fig = plt.figure()
+    gs0 = fig.add_gridspec(1, 2, figure=fig, width_ratios=[1., 2.])
+    gsl = gs0[0].subgridspec(2, 1)
+    gsr = gs0[1].subgridspec(2, 2)
 
     for gs in gsl:
         ax = fig.add_subplot(gs)
@@ -377,17 +458,17 @@ docomplicated()
 # ================================
 #
 # There can be good reasons to manually set an axes position.  A manual call
-# to `ax.set_position()` will set the axes so constrained_layout has no
-# effect on it anymore.  (Note that constrained_layout still leaves the space
-# for the axes that is moved).
+# to `~.axes.Axes.set_position` will set the axes so constrained_layout has
+# no effect on it anymore. (Note that constrained_layout still leaves the
+# space for the axes that is moved).
 
-fig, axs = plt.subplots(1, 2, constrained_layout=True)
+fig, axs = plt.subplots(1, 2)
 example_plot(axs[0], fontsize=12)
 axs[1].set_position([0.2, 0.2, 0.4, 0.4])
 
 ###############################################################################
 # If you want an inset axes in data-space, you need to manually execute the
-# layout using `fig.execute_constrained_layout()` call.  The inset figure
+# layout using ``fig.execute_constrained_layout()`` call.  The inset figure
 # will then be properly positioned.  However, it will not be properly
 # positioned if the size of the figure is subsequently changed.  Similarly,
 # if the figure is printed to another backend, there may be slight changes
@@ -395,7 +476,7 @@ axs[1].set_position([0.2, 0.2, 0.4, 0.4])
 
 from matplotlib.transforms import Bbox
 
-fig, axs = plt.subplots(1, 2, constrained_layout=True)
+fig, axs = plt.subplots(1, 2)
 example_plot(axs[0], fontsize=12)
 fig.execute_constrained_layout()
 # put into data-space:
@@ -406,6 +487,21 @@ bb_ax2 = Bbox(fig_coords_ax2)
 ax2 = fig.add_axes(bb_ax2)
 
 ###############################################################################
+# Manually turning off ``constrained_layout``
+# ===========================================
+#
+# ``constrained_layout`` usually adjusts the axes positions on each draw
+# of the figure.  If you want to get the spacing provided by
+# ``constrained_layout`` but not have it update, then do the initial
+# draw and then call ``fig.set_constrained_layout(False)``.
+# This is potentially useful for animations where the tick labels may
+# change length.
+#
+# Note that ``constrained_layout`` is turned off for ``ZOOM`` and ``PAN``
+# GUI events for the backends that use the toolbar.  This prevents the
+# axes from changing position during zooming and panning.
+#
+#
 # Limitations
 # ========================
 #
@@ -414,12 +510,12 @@ ax2 = fig.add_axes(bb_ax2)
 #
 # ``constrained_layout`` will not work on subplots
 # created via the `subplot` command.  The reason is that each of these
-# commands creates a separate `GridSpec` instance and `constrained_layout`
+# commands creates a separate `GridSpec` instance and ``constrained_layout``
 # uses (nested) gridspecs to carry out the layout.  So the following fails
 # to yield a nice layout:
 
 
-fig = plt.figure(constrained_layout=True)
+fig = plt.figure()
 
 ax1 = plt.subplot(221)
 ax2 = plt.subplot(223)
@@ -432,8 +528,8 @@ example_plot(ax3)
 ###############################################################################
 # Of course that layout is possible using a gridspec:
 
-fig = plt.figure(constrained_layout=True)
-gs = gridspec.GridSpec(2, 2, figure=fig)
+fig = plt.figure()
+gs = fig.add_gridspec(2, 2)
 
 ax1 = fig.add_subplot(gs[0, 0])
 ax2 = fig.add_subplot(gs[1, 0])
@@ -448,7 +544,7 @@ example_plot(ax3)
 # :func:`~matplotlib.pyplot.subplot2grid` doesn't work for the same reason:
 # each call creates a different parent gridspec.
 
-fig = plt.figure(constrained_layout=True)
+fig = plt.figure()
 
 ax1 = plt.subplot2grid((3, 3), (0, 0))
 ax2 = plt.subplot2grid((3, 3), (0, 1), colspan=2)
@@ -464,8 +560,8 @@ example_plot(ax4)
 # The way to make this plot compatible with ``constrained_layout`` is again
 # to use ``gridspec`` directly
 
-fig = plt.figure(constrained_layout=True)
-gs = gridspec.GridSpec(3, 3, figure=fig)
+fig = plt.figure()
+gs = fig.add_gridspec(3, 3)
 
 ax1 = fig.add_subplot(gs[0, 0])
 ax2 = fig.add_subplot(gs[0, 1:])
@@ -520,33 +616,33 @@ example_plot(ax4)
 #
 # Figures are laid out in a hierarchy:
 #
-# 1. Figure: `fig = plt.figure()`
+# 1. Figure: ``fig = plt.figure()``
 #
-#   a. Gridspec `gs0 = gridspec.GridSpec(1, 2, figure=fig)`
+#   a. Gridspec ``gs0 = gridspec.GridSpec(1, 2, figure=fig)``
 #
-#       i. Subplotspec: `ss = gs[0, 0]`
+#       i. Subplotspec: ``ss = gs[0, 0]``
 #
-#          1. Axes: `ax0 = fig.add_subplot(ss)`
+#          1. Axes: ``ax0 = fig.add_subplot(ss)``
 #
-#       ii. Subplotspec: `ss = gs[0, 1]`
+#       ii. Subplotspec: ``ss = gs[0, 1]``
 #
-#          1. Gridspec: `gsR = gridspec.GridSpecFromSubplotSpec(2, 1, ss)`
+#          1. Gridspec: ``gsR = gridspec.GridSpecFromSubplotSpec(2, 1, ss)``
 #
-#             - Subplotspec: ss = gsR[0, 0]
+#             - Subplotspec: ``ss = gsR[0, 0]``
 #
-#                - Axes: `axR0 = fig.add_subplot(ss)`
+#                - Axes: ``axR0 = fig.add_subplot(ss)``
 #
-#             - Subplotspec: ss = gsR[1, 0]
+#             - Subplotspec: ``ss = gsR[1, 0]``
 #
-#                - Axes: `axR1 = fig.add_subplot(ss)`
+#                - Axes: ``axR1 = fig.add_subplot(ss)``
 #
 # Each item has a layoutbox associated with it. The nesting of gridspecs
 # created with `.GridSpecFromSubplotSpec` can be arbitrarily deep.
 #
-# Each ``~matplotlib.axes.Axes` has *two* layoutboxes.  The first one,
+# Each `~matplotlib.axes.Axes` has *two* layoutboxes.  The first one,
 # ``ax._layoutbox`` represents the outside of the Axes and all its
 # decorations (i.e. ticklabels,axis labels, etc.).
-# The second layoutbox corresponds to the Axes' `ax.position`, which sets
+# The second layoutbox corresponds to the Axes' ``ax.position``, which sets
 # where in the figure the spines are placed.
 #
 # Why so many stacked containers?  Ideally, all that would be needed are the
@@ -569,8 +665,8 @@ example_plot(ax4)
 # the difference between the red ``pos`` box and the green ``ax`` box
 # is set by the size of the decorations around the Axes.
 #
-# In the code, this is accomplished by the entries in `.do_constrained_layout`
-# like::
+# In the code, this is accomplished by the entries in
+# ``do_constrained_layout()`` like::
 #
 #     ax._poslayoutbox.edit_left_margin_min(-bbox.x0 + pos.x0 + w_padt)
 #
@@ -589,7 +685,7 @@ plot_children(fig, fig._layoutbox, printit=False)
 # much smaller than the left-hand, so the right-hand layoutboxes are smaller.
 #
 # The Subplotspec boxes are laid out in the code in the subroutine
-# `.arange_subplotspecs`, which simply checks the subplotspecs in the code
+# ``arange_subplotspecs()``, which simply checks the subplotspecs in the code
 # against one another and stacks them appropriately.
 #
 # The two ``pos`` axes are lined up.  Because they have the same
@@ -627,10 +723,10 @@ plot_children(fig, fig._layoutbox, printit=False)
 # number of columns (one in this example).
 #
 # The colorbar layout logic is contained in `~matplotlib.colorbar.make_axes`
-# which calls `._constrained_layout.layoutcolorbarsingle`
+# which calls ``_constrained_layout.layoutcolorbarsingle()``
 # for cbars attached to a single axes, and
-# `._constrained_layout.layoutcolorbargridspec` if the colorbar is associated
-# with a gridspec.
+# ``_constrained_layout.layoutcolorbargridspec()`` if the colorbar is
+# associated with a gridspec.
 
 fig, ax = plt.subplots(1, 2, constrained_layout=True)
 im = ax[0].pcolormesh(arr, **pc_kwargs)
@@ -644,7 +740,7 @@ plot_children(fig, fig._layoutbox, printit=False)
 #
 # This example shows the Subplotspec layoutboxes being made smaller by
 # a colorbar layoutbox.  The size of the colorbar layoutbox is
-# set to be *shrink* smaller than the vertical extent of the ``pos``
+# set to be ``shrink`` smaller than the vertical extent of the ``pos``
 # layoutboxes in the gridspec, and it is made to be centered between
 # those two points.
 
@@ -713,7 +809,7 @@ plot_children(fig, fig._layoutbox, printit=False)
 # the axes position in made the same size as the occupied Axes positions.
 #
 # This is done at the start of
-# `~._constrained_layout.do_constrained_layout` (``hassubplotspec``).
+# ``_constrained_layout.do_constrained_layout()`` (``hassubplotspec``).
 
 fig = plt.figure(constrained_layout=True)
 gs = gridspec.GridSpec(1, 3, figure=fig)
