@@ -184,7 +184,8 @@ plt.show()
 # :mod:`pylab` is a convenience module that bulk imports
 # :mod:`matplotlib.pyplot` (for plotting) and :mod:`numpy`
 # (for mathematics and working with arrays) in a single name space.
-# Although many examples use :mod:`pylab`, it is no longer recommended.
+# pylab is deprecated and its use is strongly discouraged because
+# of namespace pollution. Use pyplot instead.
 #
 # For non-interactive plotting it is suggested
 # to use pyplot to create the figures and then the OO interface for
@@ -283,6 +284,9 @@ my_plotter(ax2, data3, data4, {'marker': 'o'})
 # Again, for these simple examples this style seems like overkill, however
 # once the graphs get slightly more complex it pays off.
 #
+#
+# .. _backends:
+#
 # Backends
 # ========
 #
@@ -295,11 +299,13 @@ my_plotter(ax2, data3, data4, {'marker': 'o'})
 # to the "backend" and many new users are confused by this term.
 # matplotlib targets many different use cases and output formats.  Some
 # people use matplotlib interactively from the python shell and have
-# plotting windows pop up when they type commands.  Some people embed
-# matplotlib into graphical user interfaces like wxpython or pygtk to
-# build rich applications.  Others use matplotlib in batch scripts to
-# generate postscript images from some numerical simulations, and still
-# others in web application servers to dynamically serve up graphs.
+# plotting windows pop up when they type commands.  Some people run
+# `Jupyter <https://jupyter.org>`_ notebooks and draw inline plots for
+# quick data analysis. Others embed matplotlib into graphical user
+# interfaces like wxpython or pygtk to build rich applications.  Some
+# people use matplotlib in batch scripts to generate postscript images
+# from numerical simulations, and still others run web application
+# servers to dynamically serve up graphs.
 #
 # To support all of these use cases, matplotlib can target different
 # outputs, and each of these capabilities is called a backend; the
@@ -353,8 +359,8 @@ my_plotter(ax2, data3, data4, {'marker': 'o'})
 #    :func:`~matplotlib.use` unless absolutely necessary.
 #
 # .. note::
-#    Backend name specifications are not case-sensitive; e.g., 'GTKAgg'
-#    and 'gtkagg' are equivalent.
+#    Backend name specifications are not case-sensitive; e.g., 'GTK3Agg'
+#    and 'gtk3agg' are equivalent.
 #
 # With a typical installation of matplotlib, such as from a
 # binary installer or a linux distribution package, a good default
@@ -372,11 +378,10 @@ my_plotter(ax2, data3, data4, {'marker': 'o'})
 # renderer for user interfaces is ``Agg`` which uses the `Anti-Grain
 # Geometry`_ C++ library to make a raster (pixel) image of the figure.
 # All of the user interfaces except ``macosx`` can be used with
-# agg rendering, e.g.,
-# ``WXAgg``, ``GTKAgg``, ``QT4Agg``, ``QT5Agg``, ``TkAgg``.  In
-# addition, some of the user interfaces support other rendering engines.
-# For example, with GTK, you can also select GDK rendering (backend
-# ``GTK`` deprecated in 2.0) or Cairo rendering (backend ``GTKCairo``).
+# agg rendering, e.g., ``WXAgg``, ``GTK3Agg``, ``QT4Agg``, ``QT5Agg``,
+# ``TkAgg``.  In addition, some of the user interfaces support other rendering
+# engines.  For example, with GTK+ 3, you can also select Cairo rendering
+# (backend ``GTK3Cairo``).
 #
 # For the rendering engines, one can also distinguish between `vector
 # <https://en.wikipedia.org/wiki/Vector_graphics>`_ or `raster
@@ -387,7 +392,7 @@ my_plotter(ax2, data3, data4, {'marker': 'o'})
 # DPI setting.
 #
 # Here is a summary of the matplotlib renderers (there is an eponymous
-# backend for each; these are *non-interactive backends*, capable of
+# backed for each; these are *non-interactive backends*, capable of
 # writing to a file):
 #
 # =============   ============   ================================================
@@ -437,14 +442,8 @@ my_plotter(ax2, data3, data4, {'marker': 'o'})
 # Qt4Agg    Agg rendering to a :term:`Qt4` canvas (requires PyQt4_ or
 #           ``pyside``).  This backend can be activated in IPython with
 #           ``%matplotlib qt4``.
-# GTKAgg    Agg rendering to a :term:`GTK` 2.x canvas (requires PyGTK_, and
-#           pycairo_ or cairocffi_; Python2 only).  This backend can be
-#           activated in IPython with ``%matplotlib gtk``.
-# GTKCairo  Cairo rendering to a :term:`GTK` 2.x canvas (requires PyGTK_,
-#           and pycairo_ or cairocffi_; Python2 only).
-# WXAgg     Agg rendering to a :term:`wxWidgets` canvas (requires wxPython_;
-#           v4.0 (in beta) is required for Python3). This backend can be
-#           activated in IPython with ``%matplotlib wx``.#
+# WXAgg     Agg rendering to a :term:`wxWidgets` canvas (requires wxPython_ 4).
+#           This backend can be activated in IPython with ``%matplotlib wx``.
 # ========= ================================================================
 #
 # .. _`Anti-Grain Geometry`: http://antigrain.com/
@@ -452,8 +451,6 @@ my_plotter(ax2, data3, data4, {'marker': 'o'})
 # .. _`Portable Document Format`: https://en.wikipedia.org/wiki/Portable_Document_Format
 # .. _`Scalable Vector Graphics`: https://en.wikipedia.org/wiki/Scalable_Vector_Graphics
 # .. _`Cairo graphics`: https://wwW.cairographics.org
-# .. _`Gimp Drawing Kit`: https://en.wikipedia.org/wiki/GDK
-# .. _PyGTK: http://www.pygtk.org
 # .. _PyGObject: https://wiki.gnome.org/action/show/Projects/PyGObject
 # .. _pycairo: https://www.cairographics.org/pycairo/
 # .. _cairocffi: https://pythonhosted.org/cairocffi/
@@ -485,11 +482,8 @@ my_plotter(ax2, data3, data4, {'marker': 'o'})
 # GTK and Cairo
 # -------------
 #
-# Both `GTK2` and `GTK3` have implicit dependencies on PyCairo regardless of the
-# specific Matplotlib backend used. Unfortunately the latest release of PyCairo
-# for Python3 does not implement the Python wrappers needed for the `GTK3Agg`
-# backend. `Cairocffi` can be used as a replacement which implements the correct
-# wrapper.
+# `GTK3` backends (*both* `GTK3Agg` and `GTK3Cairo`) depend on Cairo
+# (pycairo>=1.11.0 or cairocffi).
 #
 # How do I select PyQt4 or PySide?
 # --------------------------------
@@ -609,18 +603,18 @@ my_plotter(ax2, data3, data4, {'marker': 'o'})
 # Prior to version 1.0, show() generally could not be called
 # more than once in a single script (although sometimes one
 # could get away with it); for version 1.0.1 and above, this
-# restriction is lifted, so one can write a script like this:
-
-import numpy as np
-import matplotlib.pyplot as plt
-
-plt.ioff()
-for i in range(3):
-    plt.plot(np.random.rand(10))
-    plt.show()
-
-###############################################################################
-# which makes three plots, one at a time.
+# restriction is lifted, so one can write a script like this::
+#
+#     import numpy as np
+#     import matplotlib.pyplot as plt
+#
+#     plt.ioff()
+#     for i in range(3):
+#         plt.plot(np.random.rand(10))
+#         plt.show()
+#
+# which makes three plots, one at a time. I.e. the second plot will show up,
+# once the first plot is closed.
 #
 # Summary
 # -------
@@ -769,6 +763,14 @@ for i in range(3):
 #   plt.plot(y)
 #   plt.show()
 #
+# Legends
+# -------
+#
+# The default legend behavior for axes attempts to find the location
+# that covers the fewest data points (`loc='best'`). This can be a
+# very expensive computation if there are lots of data points. In
+# this case, you may want to provide a specific location.
+#
 # Using the *fast* style
 # ----------------------
 #
